/**
 * Created by PanJiaChen on 16/11/18.
 */

/**
 * Parse the time to string
 * @param {(Object|string|number)} time
 * @param {string} cFormat
 * @returns {string | null}
 */
export function parseTime (time, cFormat) {
  if (arguments.length === 0 || !time) {
    return null
  }
  const format = cFormat || '{y}-{m}-{d} {h}:{i}:{s}'
  let date
  if (typeof time === 'object') {
    date = time
  } else {
    if ((typeof time === 'string')) {
      if ((/^[0-9]+$/.test(time))) {
        // support "1548221490638"
        time = parseInt(time)
      } else {
        // support safari
        // https://stackoverflow.com/questions/4310953/invalid-date-in-safari
        time = time.replace(new RegExp(/-/gm), '/')
      }
    }

    if ((typeof time === 'number') && (time.toString().length === 10)) {
      time = time * 1000
    }
    date = new Date(time)
  }
  const formatObj = {
    y: date.getFullYear(),
    m: date.getMonth() + 1,
    d: date.getDate(),
    h: date.getHours(),
    i: date.getMinutes(),
    s: date.getSeconds(),
    a: date.getDay()
  }
  const time_str = format.replace(/{([ymdhisa])+}/g, (result, key) => {
    const value = formatObj[key]
    // Note: getDay() returns 0 on Sunday
    if (key === 'a') { return ['日', '一', '二', '三', '四', '五', '六'][value] }
    return value.toString().padStart(2, '0')
  })
  return time_str
}

/**
 * @param {number} time
 * @param {string} option
 * @returns {string}
 */
export function formatTime (time, option) {
  if (('' + time).length === 10) {
    time = parseInt(time) * 1000
  } else {
    time = +time
  }
  const d = new Date(time)
  const now = Date.now()

  const diff = (now - d) / 1000

  if (diff < 30) {
    return '刚刚'
  } else if (diff < 3600) {
    // less 1 hour
    return Math.ceil(diff / 60) + '分钟前'
  } else if (diff < 3600 * 24) {
    return Math.ceil(diff / 3600) + '小时前'
  } else if (diff < 3600 * 24 * 2) {
    return '1天前'
  }
  if (option) {
    return parseTime(time, option)
  } else {
    return (
      d.getMonth() +
      1 +
      '月' +
      d.getDate() +
      '日' +
      d.getHours() +
      '时' +
      d.getMinutes() +
      '分'
    )
  }
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function getQueryObject (url) {
  url = url == null ? window.location.href : url
  const search = url.substring(url.lastIndexOf('?') + 1)
  const obj = {}
  const reg = /([^?&=]+)=([^?&=]*)/g
  search.replace(reg, (rs, $1, $2) => {
    const name = decodeURIComponent($1)
    let val = decodeURIComponent($2)
    val = String(val)
    obj[name] = val
    return rs
  })
  return obj
}

/**
 * @param {string} input value
 * @returns {number} output value
 */
export function byteLength (str) {
  // returns the byte length of an utf8 string
  let s = str.length
  for (var i = str.length - 1; i >= 0; i--) {
    const code = str.charCodeAt(i)
    if (code > 0x7f && code <= 0x7ff) s++
    else if (code > 0x7ff && code <= 0xffff) s += 2
    if (code >= 0xDC00 && code <= 0xDFFF) i--
  }
  return s
}

/**
 * @param {Array} actual
 * @returns {Array}
 */
export function cleanArray (actual) {
  const newArray = []
  for (let i = 0; i < actual.length; i++) {
    if (actual[i]) {
      newArray.push(actual[i])
    }
  }
  return newArray
}

/**
 * @param {Object} json
 * @returns {Array}
 */
export function param (json) {
  if (!json) return ''
  return cleanArray(
    Object.keys(json).map(key => {
      if (json[key] === undefined) return ''
      return encodeURIComponent(key) + '=' + encodeURIComponent(json[key])
    })
  ).join('&')
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function param2Obj (url) {
  const search = decodeURIComponent(url.split('?')[1]).replace(/\+/g, ' ')
  if (!search) {
    return {}
  }
  const obj = {}
  const searchArr = search.split('&')
  searchArr.forEach(v => {
    const index = v.indexOf('=')
    if (index !== -1) {
      const name = v.substring(0, index)
      const val = v.substring(index + 1, v.length)
      obj[name] = val
    }
  })
  return obj
}

/**
 * @param {string} val
 * @returns {string}
 */
export function html2Text (val) {
  const div = document.createElement('div')
  div.innerHTML = val
  return div.textContent || div.innerText
}

/**
 * Merges two objects, giving the last one precedence
 * @param {Object} target
 * @param {(Object|Array)} source
 * @returns {Object}
 */
export function objectMerge (target, source) {
  if (typeof target !== 'object') {
    target = {}
  }
  if (Array.isArray(source)) {
    return source.slice()
  }
  Object.keys(source).forEach(property => {
    const sourceProperty = source[property]
    if (typeof sourceProperty === 'object') {
      target[property] = objectMerge(target[property], sourceProperty)
    } else {
      target[property] = sourceProperty
    }
  })
  return target
}

/**
 * @param {HTMLElement} element
 * @param {string} className
 */
export function toggleClass (element, className) {
  if (!element || !className) {
    return
  }
  let classString = element.className
  const nameIndex = classString.indexOf(className)
  if (nameIndex === -1) {
    classString += '' + className
  } else {
    classString =
      classString.substr(0, nameIndex) +
      classString.substr(nameIndex + className.length)
  }
  element.className = classString
}

/**
 * @param {string} type
 * @returns {Date}
 */
export function getTime (type) {
  if (type === 'start') {
    return new Date().getTime() - 3600 * 1000 * 24 * 90
  } else {
    return new Date(new Date().toDateString())
  }
}

/**
 * @param {Function} func
 * @param {number} wait
 * @param {boolean} immediate
 * @return {*}
 */
export function debounce (func, wait, immediate) {
  let timeout, args, context, timestamp, result

  const later = function () {
    // 据上一次触发时间间隔
    const last = +new Date() - timestamp

    // 上次被包装函数被调用时间间隔 last 小于设定时间间隔 wait
    if (last < wait && last > 0) {
      timeout = setTimeout(later, wait - last)
    } else {
      timeout = null
      // 如果设定为immediate===true，因为开始边界已经调用过了此处无需调用
      if (!immediate) {
        result = func.apply(context, args)
        if (!timeout) context = args = null
      }
    }
  }

  return function (...args) {
    context = this
    timestamp = +new Date()
    const callNow = immediate && !timeout
    // 如果延时不存在，重新设定延时
    if (!timeout) timeout = setTimeout(later, wait)
    if (callNow) {
      result = func.apply(context, args)
      context = args = null
    }

    return result
  }
}

/**
 * This is just a simple version of deep copy
 * Has a lot of edge cases bug
 * If you want to use a perfect deep copy, use lodash's _.cloneDeep
 * @param {Object} source
 * @returns {Object}
 */
export function deepClone (source) {
  if (!source && typeof source !== 'object') {
    throw new Error('error arguments', 'deepClone')
  }
  const targetObj = source.constructor === Array ? [] : {}
  Object.keys(source).forEach(keys => {
    if (source[keys] && typeof source[keys] === 'object') {
      targetObj[keys] = deepClone(source[keys])
    } else {
      targetObj[keys] = source[keys]
    }
  })
  return targetObj
}

/**
 * @param {Array} arr
 * @returns {Array}
 */
export function uniqueArr (arr) {
  return Array.from(new Set(arr))
}

/**
 * @returns {string}
 */
export function createUniqueString () {
  const timestamp = +new Date() + ''
  const randomNum = parseInt((1 + Math.random()) * 65536) + ''
  return (+(randomNum + timestamp)).toString(32)
}

/**
 * Check if an element has a class
 * @param {HTMLElement} elm
 * @param {string} cls
 * @returns {boolean}
 */
export function hasClass (ele, cls) {
  return !!ele.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'))
}

/**
 * Add class to element
 * @param {HTMLElement} elm
 * @param {string} cls
 */
export function addClass (ele, cls) {
  if (!hasClass(ele, cls)) ele.className += ' ' + cls
}

/**
 * Remove class from element
 * @param {HTMLElement} elm
 * @param {string} cls
 */
export function removeClass (ele, cls) {
  if (hasClass(ele, cls)) {
    const reg = new RegExp('(\\s|^)' + cls + '(\\s|$)')
    ele.className = ele.className.replace(reg, ' ')
  }
}







/**
 * 判断是否为空
 * @param {*} value 
 * @returns 
 */
export function isNull (value) {
  return (value === null) || (value === undefined)
}

/**
 * 判断不为空
 * @param {*} value 
 * @returns 
 */
export function isNotNull (value) {
  return (value !== null) && (value !== undefined)
}

/**
 * 判断字符串是否为空
 * @param {*} str 
 * @returns 
 */
export function isEmptyStr (str) {
  //return (str === undefined) || (!str) || (!/[^\s]/.test(str));
  return (str === undefined) || (!str && (str !== 0) && (str !== '0')) || (!/[^\s]/.test(str))
}

/**
 * 随机数生成 -- 不唯一
 * @returns 
 */
export const generateId = function () {
  return Math.floor(Math.random() * 100000 + Math.random() * 20000 + Math.random() * 5000)
}

// /**
//  * 对象深拷贝
//  * @param {*} origin 
//  * @returns 
//  */
// export const deepClone = function (origin) {
//   if (origin === undefined) {
//     return undefined
//   }

//   return JSON.parse(JSON.stringify(origin))
// }

/**
 * 判断是否是json 字符串
 * @param {*} str 
 * @returns 
 */
export function isJsonString (str) {
  try {
    if (typeof JSON.parse(str) == "object") {
      return true
    }
  }
  catch (e) { }
  return false
}


/**
 * 是否是纯数字
 * @param {*} val 
 * @returns 
 */
export function isRealNum (val) {
  if (val == null || val.toString().replace(/\s/g, "") === "") {
    return false
  }

  if (typeof val == "boolean") {
    return false
  }

  if (!isNaN(val)) {
    return true
  }
  else {
    return false
  }
}

/**
 * 是否有中文
 * @param {*} s 
 * @returns 
 */
export function hasChinaword (s) {
  let patrn = /[\u4E00-\u9FA5]|[\uFE30-\uFFA0]/gi

  if (!patrn.exec(s)) {
    return false
  }
  else {
    return true
  }
}


/*
 * 判断obj是否为一个整数
 */
export function isInteger (obj) {
  return Math.floor(obj) === obj
}

/**
 * 做小数点的四舍五入计算
 * @param {*} num
 * @param {*} precision
 */
export function fixed (num, precision) {
  if (!precision) {
    precision = 2
  }
  if (!isRealNum(num)) return num
  let s = Math.round(num * Math.pow(10, precision)) / Math.pow(10, precision)
  s = '' + s
  // let s = num.toFixed(precision);
  let index = s.indexOf('.')
  if (index === -1) {// num为整数时
    return Number(s + '.00')
  }
  let prefix = s.substring(0, index)
  let suffix = s.substring(index + 1, s.length)
  if (suffix) {
    for (let i = suffix.length - 1; i != 0; i--) {
      //最末位不为0，直接break;
      if (suffix.charAt(i) != '0' && i == suffix.length - 1) {
        break
      } else {
        suffix = suffix.substring(0, i)
      }
    }
  }
  return Number(prefix + '.' + suffix)
}


/**
 * 计算百分比
 * 注：保留2位小数；eg：16.67%
 * @param {*} num1
 * @param {*} num2
 */
export function getRate (num1, num2) {
  if (isEmptyStr(num1) || isEmptyStr(num2) || parseFloat(num2) === 0.00) {
    return 0.00
  }
  return fixed(parseFloat(num1) * 100 / parseFloat(num2), 2)
}




/**
 * 判断参数是否是Object类型
 * @param {*} o 
 */
export function isObject (o) {
  return (
    !isMap(o) &&
    (typeof o === 'object' || typeof o === 'function') &&
    o !== null
  )
}


/**
 * 判断参数是否是Map类型
 * @param {*} obj 
 */
function isMap (obj) {
  if (obj instanceof Map) {
    return true
  } else {
    return false
  }
}


/**
 * list  转换 form 提交方案
 * @param {string} listName 对应的变量名称 
 * @param {Object/Array} obj 
 * @param {Integer} index 当对象的时候使用 
 * @returns 
 */
export function convertFormFormat (listName, obj, index = 0) {
  let result = {}
  if (Array.isArray(obj)) {
    for (let i = 0; i < obj.length; i++) {
      let objItem = obj[i]
      for (let key in objItem) {
        result[listName + '[' + i + '].' + key] = objItem[key]
      }
    }
  } else {
    for (let key in obj) {
      result[listName + '[' + index + '].' + key] = obj[key]
    }
  }
  return result
}

/**
 * 延时操作
 */
export function delayedHandler (second) {
  const time = second * 1000
  const promise = new Promise(resolve => setTimeout(() => resolve(time + ' 秒到了'), time))
  return promise
}

export function getKeyValueObject (url_) {
  return [{}].concat(url_.match(/(?<=[\?&=])[^&=]+(?=(=|&|$))/g)).
    reduce((o, v, i, arr) => (i % 2 && (o[v] = arr[i + 1]), o))
}

export function getValue (url_, key_) {
  return url_.match(new RegExp(`(?<=[\\?&]${key_}=)[^&]+(?=(&|$))`))[0]
}

export function isPageIsReadOnly () {
  const search = window.location.search
  if (search && search.indexOf('pageType=view') > -1) {
    return true
  }
  return false
}

/**
 * 判断审批页面是否是子流程
 * @returns 
 */
export function isSubFlow () {
  const search = window.location.search
  if ((search?.indexOf('isFlowable=true') > -1 || search?.indexOf('isInput=') > -1) && search?.indexOf('parentFlow=N') > -1) {
    return true
  }
  return false
}

/**
 * 获取流程发起人法人编号
 * @returns 
 */
export function queryParentUpbrNo () {
  const urlObj = new URL(window.location.href)
  const paramObj = Object.fromEntries(urlObj.searchParams)
  if((paramObj.isFlowable || paramObj.isInput) && paramObj.parentFlow === 'N' && !['1','2'].includes(paramObj.isInput) ){
    return paramObj.parentUpBrNo
  }
  return null
}

export function removeEmptyProperties (obj) {
  for (let key in obj) {
    if (
      obj[key] === null ||                // 检查 null 值
      obj[key] === undefined ||           // 检查 undefined 值
      Number.isNaN(obj[key]) ||           // 检查 NaN 值
      obj[key] === "" ||                  // 检查空字符串
      (Array.isArray(obj[key]) && obj[key].length === 0)   // 检查空数组
    ) {
      delete obj[key]                    // 删除属性
    }
  }
  return obj
}
//删除一个对象中的翻译值
export function removeZh(obj){
  for (let key in obj) {
    if (obj.hasOwnProperty(key)) {
      if (!key.endsWith("Zh")) {
        let zhKey = key + "Zh";
        if (obj.hasOwnProperty(zhKey)) {
          delete obj[zhKey];
        }
      }
    }
  }
}

//该方法用户配合表格查询
export function initQueryParam(a={_tableinit:false,values:{}},b ={}){
   const _tableinit = a._tableinit;
   const values = a.values;
   let queryParam = null
   if(!_tableinit){
    queryParam = {...b,...values}
   }
   return queryParam
}
