// 定义相关接口
interface Message {
    FileFlag: boolean;
    BitMapInstance: {
        FdBitMap: Map<string, FdItem>;
    };
    IntegrateFd(bitMapByte: Uint8Array): Uint8Array;
    GetFdFormatValue(code: string): string;
}

interface FdItem {
    Code: string;
    FieldType: string;
    Length: number;
    Children?: FdItem[];
}

// 模拟 PlatformSettings 类
class PlatformSettings {
    static Encoding = {
        getBytes: (str: string) => new TextEncoder().encode(str),
        getString: (bytes: Uint8Array) => new TextDecoder().decode(bytes)
    };
}

// 模拟 PlatformLogger 类
class PlatformLogger {
    static CommunicationInfo(info: string) {
        console.log(info);
    }

    static SystemErrorInfo(message: string, ex: any) {
        console.error(message, ex);
    }
}

export class Msg858303 {
    private static instance: Msg858303;

    private constructor() {}

    public static GetInstance(): Msg858303 {
        if (!this.instance) {
            this.instance = new Msg858303();
        }
        return this.instance;
    }

    public Integrate(msg: Message): Uint8Array {
        let returnData: Uint8Array | null = null;

        try {
            returnData = this.DoIntegrate(msg);
        } catch (ex) {
            throw ex;
        }
        return returnData!;
    }

    public Analyze(msg: Message, data: Uint8Array): boolean {
        try {
            this.DoAnalyze(msg, data);
        } catch (ex) {
            return false;
        }
        return true;
    }

    private DoIntegrate(msg: Message): Uint8Array {
        let msgData: Uint8Array;
        let totalLen: number;
        let strLen: string;
        let msgDataLen: number;
        let msgTitle: string;
        let msgKindByte: Uint8Array;
        let bitMapByte = new Uint8Array(16);
        let bitDataByte: Uint8Array;
        let totalMsgData: Uint8Array;

        let fileFlag: number;
        if (msg.FileFlag) {
            fileFlag = 0x7f;
        } else {
            fileFlag = 0xff;
        }

        try {
            msgKindByte = PlatformSettings.Encoding.getBytes("     ");
            bitDataByte = msg.IntegrateFd(bitMapByte);

            msgDataLen = msgKindByte.length + bitMapByte.length + bitDataByte.length;
            totalLen = msgDataLen + 63;
            strLen = totalLen.toString().padStart(8, '0');
            msgTitle = msgDataLen.toString().padStart(4, '0');
            msgTitle = "0000" + "MDBS1" + "    ";
            msgTitle = msgTitle.padEnd(63, ' ');

            msgData = new Uint8Array(msgDataLen);
            for (let i = 0; i < msgData.length; i++) {
                if (i < msgKindByte.length) {
                    msgData[i] = msgKindByte[i];
                } else if (i < msgKindByte.length + bitMapByte.length) {
                    msgData[i] = bitMapByte[i - msgKindByte.length];
                } else {
                    msgData[i] = bitDataByte[i - msgKindByte.length - bitMapByte.length];
                }
            }

            let len8 = PlatformSettings.Encoding.getBytes(strLen);
            let msgTitleByte = PlatformSettings.Encoding.getBytes(msgTitle);
            totalMsgData = new Uint8Array(len8.length + msgTitleByte.length + msgData.length);
            for (let j = 0; j < totalMsgData.length; j++) {
                if (j < len8.length) {
                    totalMsgData[j] = len8[j];
                } else if (j < len8.length + msgTitleByte.length) {
                    totalMsgData[j] = msgTitleByte[j - len8.length];
                } else {
                    totalMsgData[j] = msgData[j - len8.length - msgTitleByte.length];
                }
            }
            // 设置文件标志位
            // totalMsgData[totalMsgData.length - 1] = fileFlag;
        } catch (ex) {
            throw ex;
        }
        return totalMsgData;
    }

    private GetBitData(msg: Message, bitMap: Uint8Array): string {
        let curData = "";
        let tmpStr = "";
        let logStr = "";

        try {
            let bitStr = "0".padStart(128, '0');
            let bitByte = PlatformSettings.Encoding.getBytes(bitStr);
            bitByte[0] = PlatformSettings.Encoding.getBytes("1")[0];

            for (let i = 1; i < 128; i++) {
                curData = "";
                logStr += this.GetBitItemData(msg, i + 1, curData);

                if (curData !== "") {
                    bitByte[i] = PlatformSettings.Encoding.getBytes("1")[0];
                    curData = curData.replace(/\0/g, ' ').replace(/\n/g, ' ').replace(/\r/g, ' ');
                    tmpStr += curData;
                }
            }

            bitStr = PlatformSettings.Encoding.getString(bitByte);
            PlatformLogger.CommunicationInfo("\n请求数据：\n" + logStr);

            for (let j = 0; j < 16; j++) {
                let k = j * 8;
                bitMap[j] = parseInt(bitStr.substring(k, k + 8), 2);
            }
        } catch (ex) {
            PlatformLogger.SystemErrorInfo("获取8583位图数据出现异常！", ex);
            throw ex;
        }
        return tmpStr;
    }

    private static GetBitItemData(msg: Message, i: number, curData: string): string {
        let result = "";
        let fdName = i.toString().padStart(3, '0') + "0";
        if (!msg.BitMapInstance.FdBitMap.has(fdName)) {
            throw new Error(`Fd报文定义集合中未找到Code等于[${fdName}]的域定义！`);
        }
        let fdItem = msg.BitMapInstance.FdBitMap.get(fdName)!;

        if (!fdItem.Children || fdItem.Children.length === 0) {
            result = msg.GetFdFormatValue(fdItem.Code);
        } else {
            let child = "";
            fdItem.Children.forEach(item => {
                child += msg.GetFdFormatValue(item.Code);
            });
            result = child;
        }

        switch (fdItem.FieldType) {
            case "-1":
                result = "";
                break;
            case "0":
                // 定长域
                break;
            default:
                let len = parseInt(fdItem.FieldType);
                let data = PlatformSettings.Encoding.getBytes(result);
                if (data.length > 0) {
                    if (data.length > fdItem.Length) {
                        let tmp = new Uint8Array(fdItem.Length);
                        tmp.set(data.subarray(0, fdItem.Length));
                        data = tmp;
                    }
                    result = data.length.toString().padStart(len, '0') + PlatformSettings.Encoding.getString(data);
                }
                break;
        }
        curData = result;
        return `Fd${i + 1}:[${curData}]\n`;
    }

    private DoAnalyze(msg: Message, data: Uint8Array): void {
        // 原代码未实现具体逻辑，可根据需求补充
    }
}