// 定义 Message 接口
interface Message {
    Fd16: string;
    Fd96_Q: string;
    Fd5: string;
    Fd6: string;
    Fd2: string;
    Fd7: string;
    Fd12: string;
    FileData: string;
}

// 模拟 PlatformSettings 类
class PlatformSettings {
    static Encoding = {
        GetBytes: (str: string): Uint8Array => {
            return new TextEncoder().encode(str);
        },
        GetString: (bytes: Uint8Array): string => {
            return new TextDecoder('gbk').decode(bytes);
        }
    };
}

// 模拟 PlatformLogger 类
class PlatformLogger {
    static SystemErrorInfo(message: string, ex: Error) {
        console.error(`${message}`, ex);
    }

    static CommunicationInfo(info: string) {
        console.log(info);
    }
}

export class MsgXml01 {
    private static instance: MsgXml01;

    private constructor() {}

    static GetInstance(): MsgXml01 {
        if (!this.instance) {
            this.instance = new MsgXml01();
        }
        return this.instance;
    }

    Integrate(msg: Message): Uint8Array {
        let totalMsgData: Uint8Array;
        let dataPartLength: Uint8Array;
        let dataPart: Uint8Array;

        try {
            const xmlFirstLine = '<?xml version="1.0" encoding="GBK"?>';
            const upTagBegin = '<service>';
            const xmlHeader = this.Msg8583ToHeader(msg);
            const xmlData = msg.FileData;
            const upTagEnd = '</service>';
            const dataPartStr = xmlFirstLine + upTagBegin + xmlHeader + xmlData + upTagEnd;

            dataPart = PlatformSettings.Encoding.GetBytes(dataPartStr);
            const dataPartLengthInt = dataPart.length;
            const dataPartLengthStr = dataPartLengthInt.toString().padStart(8, '0');
            dataPartLength = PlatformSettings.Encoding.GetBytes(dataPartLengthStr);

            totalMsgData = new Uint8Array(8 + dataPartLengthInt);
            for (let j = 0; j < totalMsgData.length; j++) {
                if (j < 8) {
                    totalMsgData[j] = dataPartLength[j];
                } else {
                    totalMsgData[j] = dataPart[j - 8];
                }
            }
        } catch (ex) {
            PlatformLogger.SystemErrorInfo('组包发生异常！', ex as Error);
            throw ex;
        }

        return totalMsgData;
    }

    private Msg8583ToHeader(msg: Message): string {
        let result = '<SysHead>';
        result += `<Txcode>${msg.Fd16.trim()}</Txcode>`;
        result += `<FrontTrace>${msg.Fd96_Q.trim()}</FrontTrace>`;
        result += `<Txdate>${msg.Fd5.trim()}</Txdate>`;
        result += `<Txtime>${msg.Fd6.trim()}</Txtime>`;
        result += `<InstNo>${msg.Fd2.trim()}</InstNo>`;
        result += `<UserNo>${msg.Fd7.trim()}</UserNo>`;
        result += '</SysHead>';
        return result;
    }

    Analyze(msg: Message, data: Uint8Array): boolean {
        try {
            this.DoAnalyze(msg, data);
            return true;
        } catch (ex) {
            return false;
        }
    }

    DoAnalyze(msg: Message, returnData: Uint8Array): void {
        try {
            let xmlDataLen = 0;
            const tmpLen = returnData.slice(0, 8);
            xmlDataLen = parseInt(PlatformSettings.Encoding.GetString(tmpLen), 10);

            const ret = returnData.slice(8, 8 + xmlDataLen);
            const xmlStr = PlatformSettings.Encoding.GetString(ret);
            PlatformLogger.CommunicationInfo('\n返回报文数据:\n' + xmlStr);

            // 模拟 XML 解析
            const xmlDoc = this.parseXML(xmlStr);
            const node = xmlDoc.querySelector('/service/SysHead');
            if (node) {
                const childNodes = node.childNodes;
                msg.Fd16 = childNodes[0].textContent || '';
                msg.Fd5 = childNodes[1].textContent || '';
                msg.Fd6 = childNodes[2].textContent || '';
                msg.Fd2 = childNodes[3].textContent || '';
                msg.Fd7 = childNodes[4].textContent || '';
                msg.Fd12 = childNodes[5].textContent || '';
            }

            const nodeOfBody = xmlDoc.querySelector('/service/Body');
            if (nodeOfBody) {
                msg.FileData = nodeOfBody.outerHTML;
            }
        } catch (ex) {
            PlatformLogger.SystemErrorInfo('解析报文失败！', ex as Error);
            throw ex;
        }
    }

    private parseXML(xmlStr: string): Document {
        const parser = new DOMParser();
        return parser.parseFromString(xmlStr, 'text/xml');
    }
}