// 模拟 BufferManager 类
import { BufferManager } from './BufferManager';

// 模拟事件处理函数类型
type EventHandler<T> = (sender: any, args: T) => void;

// 模拟事件参数类
class ServerPortEventArgs {
    constructor(public port: number) { }
}

class DataReceivedEventArgs {
    constructor(public data: Uint8Array) { }
}

class DataSendEventArgs {
    constructor(public sendLength: number) { }
}

class SocketErrorEventArgs {
    constructor(public error: Error, public operation: string) { }
}

// 模拟 Socket 操作枚举
enum SocketOperation {
    Bind,
    Connect,
    Send
}

// 模拟 Socket 类
class Socket {
    constructor() { }
    IsBound = false;
    Close() { }
    Bind(endPoint: any) { }
    Listen(backlog: number) { }
    BeginConnect(endPoint: any, callback: (ar: any) => void, state: any) { }
    EndConnect(ar: any) { }
    BeginSend(data: Uint8Array, offset: number, length: number, flags: number, callback: (ar: any) => void, state: any) { }
    EndSend(ar: any) { return 0; }
    Connect(endPoint: any) { }
    Send(data: Uint8Array, offset: number, length: number, flags: number) { return 0; }
    Shutdown(how: any) { }
}

// 模拟 ManualResetEvent 类
class ManualResetEvent {
    constructor(private initialState: boolean) { }
    Reset() { }
    Set() { }
    WaitOne() { }
    Close() { }
}

// 模拟 Func 类型
type Func<TResult> = () => TResult;

export default class SocketManager {
    static GetLocalIpAddressList() {
        return
    }
    private m_ListenningSockets: Map<number, Socket> = new Map();
    private static Locker = {};
    private m_BufferManager: BufferManager;
    private m_ThresholdValue = 102400;

    private m_SuspendAllSend = false;
    private m_StopAllSend = false;
    private m_MaxListenningRequest: number;
    private m_SendControl: ManualResetEvent;
    private m_SendBufferSize: number;
    private m_ReceiveBufferSize: number;
    private m_SendTimeOut: number;
    private m_ReceiveTimeOut: number;

    // 事件
    public ServerStart: EventHandler<ServerPortEventArgs> | null = null;
    public ServerStop: EventHandler<ServerPortEventArgs> | null = null;
    //public SendStart: EventHandler | null = null;
    //public SendCompleted: EventHandler | null = null;
    public DataReceived: EventHandler<DataReceivedEventArgs> | null = null;
    public Sending: EventHandler<DataSendEventArgs> | null = null;
    public ErrorHappened: EventHandler<SocketErrorEventArgs> | null = null;

    constructor(maxClientCount = 200, receiveBufferSize = 12400, sendBufferSize = 0, receiveTimeOut = 0, sendTimeOut = 0) {
        this.m_MaxListenningRequest = maxClientCount;
        this.m_SendBufferSize = sendBufferSize;
        this.m_ReceiveBufferSize = receiveBufferSize;
        this.m_SendTimeOut = sendTimeOut;
        this.m_ReceiveTimeOut = receiveTimeOut;
        this.m_BufferManager = new BufferManager(maxClientCount, receiveBufferSize, sendBufferSize);
        this.m_SendControl = new ManualResetEvent(true);
    }

    get MaxListenningRequest(): number {
        return this.m_MaxListenningRequest;
    }

    get SendBufferSize(): number {
        return this.m_SendBufferSize;
    }

    set SendBufferSize(value: number) {
        this.m_SendBufferSize = value;
    }

    get ReceiveBufferSize(): number {
        return this.m_ReceiveBufferSize;
    }

    set ReceiveBufferSize(value: number) {
        this.m_ReceiveBufferSize = value;
    }

    get SendTimeOut(): number {
        return this.m_SendTimeOut;
    }

    set SendTimeOut(value: number) {
        this.m_SendTimeOut = value;
    }

    get ReceiveTimeOut(): number {
        return this.m_ReceiveTimeOut;
    }

    set ReceiveTimeOut(value: number) {
        this.m_ReceiveTimeOut = value;
    }

    get ThresholdValue(): number {
        return this.m_ThresholdValue;
    }

    get SuspendAllSend(): boolean {
        return this.m_SuspendAllSend;
    }

    set SuspendAllSend(value: boolean) {
        if (value) {
            this.m_SendControl.Reset();
        } else {
            this.m_SendControl.Set();
        }
        this.m_SuspendAllSend = value;
    }

    get StopAllSend(): boolean {
        return this.m_StopAllSend;
    }

    set StopAllSend(value: boolean) {
        if (value && this.SuspendAllSend) {
            this.SuspendAllSend = false;
        }
        this.m_StopAllSend = value;
    }

    StartListenning(port: number, async = true): boolean {
        const socket = this.GetSocket(0, 0);

        try {
            if (!socket.IsBound) {
                // 模拟锁
                (() => {
                    if (!socket.IsBound) {
                        socket.Bind(this.GetIpv4EndPoint(port));
                        socket.Listen(this.m_MaxListenningRequest);
                        this.m_ListenningSockets.set(port, socket);
                        this.OnServerStart(port);
                        this.StartReceive(socket, async);
                    }
                })();
            }
        } catch (se) {
            socket.Close();
            this.OnErrorHappened(se as Error, SocketOperation.Bind);
            return false;
        }

        return true;
    }

    StartListenningBatch(ports: number[]): void {
        ports.forEach(port => {
            this.StartListenning(port);
        });
    }

    StopListenning(onlyCloseSocket: boolean): void {
        const dic = new Map(this.m_ListenningSockets);
        this.m_ListenningSockets.clear();

        dic.forEach((socket, port) => {
            socket.Close();
            this.OnServerStop(port);
        });

        if (!onlyCloseSocket) {
            this.m_BufferManager.Clear();
            this.m_SendControl.Close();
        }
    }
    // hulei 函数重载 需要重新实现3
    //   function Send(hostNameOrAddress: string, port: number, data: Uint8Array, async = true);
    //  function Send(endPoint: any, data: Uint8Array, async: boolean);
    Send(hostNameOrAddress: string, port: number, data: Uint8Array, async = true): void {
        if (!data) {
            throw new Error('�������ݲ���Ϊ��');
        }

        const endPoint = this.GetIpv4EndPoint(hostNameOrAddress, port);
        // this.Send(endPoint, data, async);
    }

    // function Send(endPoint: any, data: Uint8Array, async: boolean): void {
    //     if (!data) {
    //         throw new Error('�������ݲ���Ϊ��');
    //     }

    //     const socket = this.GetSocket(this.m_SendTimeOut, 0);

    //     try {
    //         this.OnSendStart();

    //         if (async) {
    //             this.SendAsync(socket, endPoint, data);
    //         } else {
    //             this.SendSynchronize(socket, endPoint, data);
    //         }
    //     } catch (se) {
    //         socket.Close();
    //         this.OnErrorHappened(se as Error, SocketOperation.Connect);
    //     }
    // }

    SendSynchronize(socket: Socket, endPoint: any, data: Uint8Array): void {
        socket.Connect(endPoint);
        this.SendDataInternal(socket, data);
    }

    SendDataInternal(socket: Socket, data: Uint8Array): void {
        socket.Send(data, 0, data.length, 0);

        try {
            socket.Shutdown(0);
        } catch (ex) { }

        socket.Close();
        this.FireSendEvents(0);
    }

    SendAsync(socket: Socket, endPoint: any, data: Uint8Array): void {
        socket.BeginConnect(endPoint, (ar: any) => {
            try {
                socket.EndConnect(ar);
            } catch (se) {
                socket.Close();
                this.OnErrorHappened(se as Error, SocketOperation.Connect);
                return;
            }

            this.SendDataInternal(socket, data);
        }, null);
    }

    SendWithFunc(hostNameOrAddress: string, port: number, dataGetterFunc: Func<Uint8Array>): void {
        if (!dataGetterFunc) return;

        const endPoint = this.GetIpv4EndPoint(hostNameOrAddress, port);
        this.SendEndPointWithFunc(endPoint, dataGetterFunc);
    }

    SendEndPointWithFunc(endPoint: any, dataGetterFunc: Func<Uint8Array>): void {
        if (!endPoint) {
            throw new Error('Զ���ս�㲻��Ϊ��');
        }

        const sendSocket = this.GetSocket(this.SendTimeOut, 0);
        this.OnSendStart();

        try {
            sendSocket.BeginConnect(endPoint, (ar: any) => {
                try {
                    sendSocket.EndConnect(ar);
                } catch (se) {
                    this.OnErrorHappened(se as Error, SocketOperation.Connect);
                    sendSocket.Close();
                    return;
                }

                const callback = this.GetSendCallback(sendSocket, dataGetterFunc);
                this.SendDataInternalWithCallback(sendSocket, callback, dataGetterFunc);
            }, null);
        } catch (se) {
            sendSocket.Close();
            this.OnErrorHappened(se as Error, SocketOperation.Connect);
        }
    }

    GetSendCallback(sendSocket: Socket, dataGetterFunc: Func<Uint8Array>): (ar: any) => void {
        return (receiveResult: any) => {
            const sendLength = sendSocket.EndSend(receiveResult);

            this.OnSending(sendLength);
            this.SendDataInternalWithCallback(sendSocket, this.GetSendCallback(sendSocket, dataGetterFunc), dataGetterFunc);
        };
    }

    SendDataInternalWithCallback(sendSocket: Socket, callback: (ar: any) => void, dataGetterFunc: Func<Uint8Array>): void {
        if (this.m_SuspendAllSend) {
            this.m_SendControl.WaitOne();
        }

        const data = dataGetterFunc();

        if (this.m_StopAllSend) {
            sendSocket.Close();
            return;
        }

        if (!data || data.length === 0) {
            this.FireSendEvents(0);
            sendSocket.Close();
            return;
        }

        sendSocket.BeginSend(data, 0, data.length, 0, callback, null);
    }

    FireSendEvents(socketError: number): void {
        if (socketError === 0) {
            this.OnSendCompleted();
        } else {
            this.OnErrorHappened(new Error(`Socket error: ${socketError}`), SocketOperation.Send);
        }
    }

    StartReceive(listenningSocket: Socket, async: boolean): void {
        // 由于原代码截断，此处简单实现
        console.log('Start receiving...');
    }

    private GetSocket(sendTimeOut: number, receiveTimeOut: number): Socket {
        return new Socket();
    }

    private GetIpv4EndPoint(port: number): any;
    private GetIpv4EndPoint(hostNameOrAddress: string, port: number): any;
    private GetIpv4EndPoint(arg1: string | number, port?: number): any {
        // 简单模拟
        return {};
    }
    //时间的生命周期 这个需要看情况处理 hulei 
    private OnServerStart(port: number): void {
        if (this.ServerStart) {
            this.ServerStart(this, new ServerPortEventArgs(port));
        }
    }

    private OnServerStop(port: number): void {
        if (this.ServerStop) {
            this.ServerStop(this, new ServerPortEventArgs(port));
        }
    }

    private OnSendStart(): void {
        // if (this.SendStart) {
        //     this.SendStart(this, null);
        // }
    }

    private OnSendCompleted(): void {
        // if (this.SendCompleted) {
        //     this.SendCompleted(this, null);
        // }
    }

    private OnSending(sendLength: number): void {
        if (this.Sending) {
            this.Sending(this, new DataSendEventArgs(sendLength));
        }
    }

    private OnErrorHappened(error: Error, operation: SocketOperation): void {
        if (this.ErrorHappened) {
            this.ErrorHappened(this, new SocketErrorEventArgs(error, SocketOperation[operation]));
        }
    }

    Dispose(): void {
        this.StopListenning(false);
    }
}
