import { reactive, ref, computed } from 'vue'
import type { Connector } from './Connector'
import { PathFinder } from './PathFinder'

export enum ArrowSymbol {
    None = 0,
    Arrow = 1,
    Diamond = 2
}

export class Connection {
    private _source: Connector | null = null
    private _sink: Connector | null = null
    private _pathGeometry: any = null
    private _anchorPositionSource = { x: 0, y: 0 }
    private _anchorAngleSource = 0
    private _anchorPositionSink = { x: 0, y: 0 }
    private _anchorAngleSink = 0
    private _sourceArrowSymbol = ArrowSymbol.None
    private _sinkArrowSymbol = ArrowSymbol.Arrow
    private _labelPosition = { x: 0, y: 0 }
    private _strokeDashArray: number[] = []
    private _brach = ""
    private _isSelected = false
    private _connectionAdorner: any = null

    get source(): Connector | null {
        return this._source
    }
    set source(value: Connector | null) {
        if (this._source !== value) {
            if (this._source) {
                this._source.onPositionChanged.unsubscribe(this.onConnectorPositionChanged)
                this._source.connections.delete(this)
            }

            this._source = value

            if (this._source) {
                this._source.connections.add(this)
                this._source.onPositionChanged.subscribe(this.onConnectorPositionChanged)
            }

            this.updatePathGeometry()
        }
    }

    get sink(): Connector | null {
        return this._sink
    }
    set sink(value: Connector | null) {
        if (this._sink !== value) {
            if (this._sink) {
                this._sink.onPositionChanged.unsubscribe(this.onConnectorPositionChanged)
                this._sink.connections.delete(this)
            }

            this._sink = value

            if (this._sink) {
                this._sink.connections.add(this)
                this._sink.onPositionChanged.subscribe(this.onConnectorPositionChanged)
            }

            this.updatePathGeometry()
        }
    }

    get pathGeometry(): any {
        return this._pathGeometry
    }
    set pathGeometry(value: any) {
        if (this._pathGeometry !== value) {
            this._pathGeometry = value
            this.updateAnchorPosition()
        }
    }

    get anchorPositionSource(): { x: number, y: number } {
        return this._anchorPositionSource
    }
    set anchorPositionSource(value: { x: number, y: number }) {
        if (this._anchorPositionSource.x !== value.x || this._anchorPositionSource.y !== value.y) {
            this._anchorPositionSource = value
        }
    }

    get anchorAngleSource(): number {
        return this._anchorAngleSource
    }
    set anchorAngleSource(value: number) {
        if (this._anchorAngleSource !== value) {
            this._anchorAngleSource = value
        }
    }

    get anchorPositionSink(): { x: number, y: number } {
        return this._anchorPositionSink
    }
    set anchorPositionSink(value: { x: number, y: number }) {
        if (this._anchorPositionSink.x !== value.x || this._anchorPositionSink.y !== value.y) {
            this._anchorPositionSink = value
        }
    }

    get anchorAngleSink(): number {
        return this._anchorAngleSink
    }
    set anchorAngleSink(value: number) {
        if (this._anchorAngleSink !== value) {
            this._anchorAngleSink = value
        }
    }

    get sourceArrowSymbol(): ArrowSymbol {
        return this._sourceArrowSymbol
    }
    set sourceArrowSymbol(value: ArrowSymbol) {
        if (this._sourceArrowSymbol !== value) {
            this._sourceArrowSymbol = value
        }
    }

    get sinkArrowSymbol(): ArrowSymbol {
        return this._sinkArrowSymbol
    }
    set sinkArrowSymbol(value: ArrowSymbol) {
        if (this._sinkArrowSymbol !== value) {
            this._sinkArrowSymbol = value
        }
    }

    get labelPosition(): { x: number, y: number } {
        return this._labelPosition
    }
    set labelPosition(value: { x: number, y: number }) {
        if (this._labelPosition.x !== value.x || this._labelPosition.y !== value.y) {
            this._labelPosition = value
        }
    }

    get strokeDashArray(): number[] {
        return this._strokeDashArray
    }
    set strokeDashArray(value: number[]) {
        if (JSON.stringify(this._strokeDashArray) !== JSON.stringify(value)) {
            this._strokeDashArray = value
        }
    }

    get brach(): string {
        return this._brach
    }
    set brach(value: string) {
        if (this._brach !== value) {
            this._brach = value
        }
    }

    get isSelected(): boolean {
        return this._isSelected
    }
    set isSelected(value: boolean) {
        if (this._isSelected !== value) {
            this._isSelected = value
            if (this._isSelected) {
                this.showAdorner()
            } else {
                this.hideAdorner()
            }
        }
    }

    constructor(source: Connector | null, sink: Connector | null, strContent = "") {
        this.source = source
        this.sink = sink
        this.brach = strContent
    }

    private onConnectorPositionChanged = () => {
        this.updatePathGeometry()
    }

    private updatePathGeometry(): void {
        if (this.source && this.sink) {
            const linePoints = PathFinder.getConnectionLine(this.source.getInfo(), this.sink.getInfo(), true)
            if (linePoints.length > 0) {
                // 创建路径几何图形
                this.pathGeometry = {
                    startPoint: linePoints[0],
                    segments: linePoints.slice(1)
                }
            }
        }
    }

    private updateAnchorPosition(): void {
        if (!this.pathGeometry) return

        // 计算路径起点、中点和终点的位置和切线
        const pathStartPoint = this.pathGeometry.startPoint
        const pathEndPoint = this.pathGeometry.segments[this.pathGeometry.segments.length - 1]
        const pathMidPoint = this.pathGeometry.segments[Math.floor(this.pathGeometry.segments.length / 2)]

        // 计算角度
        this.anchorAngleSource = Math.atan2(-pathStartPoint.y, -pathStartPoint.x) * (180 / Math.PI)
        this.anchorAngleSink = Math.atan2(pathEndPoint.y, pathEndPoint.x) * (180 / Math.PI)

        // 添加边距
        pathStartPoint.x -= pathStartPoint.x * 0.1
        pathStartPoint.y -= pathStartPoint.y * 0.1
        pathEndPoint.x += pathEndPoint.x * 0.1
        pathEndPoint.y += pathEndPoint.y * 0.1

        this.anchorPositionSource = pathStartPoint
        this.anchorPositionSink = pathEndPoint
        this.labelPosition = pathMidPoint
    }

    private showAdorner(): void {
        if (!this._connectionAdorner) {
            // 创建装饰器
            this._connectionAdorner = {
                visible: true
            }
        }
        this._connectionAdorner.visible = true
    }

    private hideAdorner(): void {
        if (this._connectionAdorner) {
            this._connectionAdorner.visible = false
        }
    }

    public dispose(): void {
        if (this.source) {
            this.source.onPositionChanged.unsubscribe(this.onConnectorPositionChanged)
        }
        if (this.sink) {
            this.sink.onPositionChanged.unsubscribe(this.onConnectorPositionChanged)
        }
        this.hideAdorner()
    }
}

// Vue组合式API封装
export const useConnection = (source: Connector | null, sink: Connector | null, strContent = "") => {
    const connection = reactive(new Connection(source, sink, strContent))
    return {
        connection,
        ArrowSymbol
    }
}