import { reactive, ref, onMounted, onUnmounted } from 'vue'
import type { Connection } from './Connection'
import type { DesignerCanvas } from './DesignerCanvas'
import type { Connector } from './Connector'
import type { DesignerItem } from './DesignerItem'
import { PathFinder } from './PathFinder'

export class ConnectionAdorner {
    private designerCanvas: DesignerCanvas
    private connection: Connection
    private pathGeometry: any = null
    private fixConnector: Connector | null = null
    private dragConnector: Connector | null = null
    private hitDesignerItem: DesignerItem | null = null
    private hitConnector: Connector | null = null
    private isVisible = false

    constructor(designerCanvas: DesignerCanvas, connection: Connection) {
        this.designerCanvas = designerCanvas
        this.connection = connection
        this.connection.onAnchorPositionChanged(this.onAnchorPositionChanged)
        this.initDragHandlers()
    }

    private onAnchorPositionChanged = (propertyName: string) => {
        if (!this.isVisible) return
        
        if (propertyName === 'AnchorPositionSource') {
            this.updateSourceThumbPosition()
        } else if (propertyName === 'AnchorPositionSink') {
            this.updateSinkThumbPosition()
        }
    }

    private initDragHandlers() {
        // 初始化拖拽处理逻辑
        this.designerCanvas.onDragStart(this.onDragStart)
        this.designerCanvas.onDrag(this.onDrag)
        this.designerCanvas.onDragEnd(this.onDragEnd)
    }

    private onDragStart = (event: MouseEvent, thumbType: 'source' | 'sink') => {
        this.isVisible = true
        this.hitDesignerItem = null
        this.hitConnector = null
        this.pathGeometry = null
        this.connection.strokeDashArray = [1, 2]

        if (thumbType === 'source') {
            this.fixConnector = this.connection.sink
            this.dragConnector = this.connection.source
        } else {
            this.dragConnector = this.connection.sink
            this.fixConnector = this.connection.source
        }
    }

    private onDrag = (event: MouseEvent) => {
        if (!this.isVisible) return
        
        const position = { x: event.clientX, y: event.clientY }
        this.hitTesting(position)
        this.pathGeometry = this.updatePathGeometry(position)
    }

    private onDragEnd = (event: MouseEvent) => {
        if (!this.isVisible) return
        
        if (this.hitConnector) {
            if (this.connection.source === this.fixConnector) {
                this.connection.sink = this.hitConnector
            } else {
                this.connection.source = this.hitConnector
            }
        }

        this.hitDesignerItem = null
        this.hitConnector = null
        this.pathGeometry = null
        this.connection.strokeDashArray = []
        this.isVisible = false
    }

    private hitTesting(position: { x: number, y: number }) {
        let hitConnectorFlag = false
        const hitElement = this.designerCanvas.getElementFromPoint(position)

        // 模拟WPF的视觉树遍历
        let current = hitElement
        while (current) {
            if (current.isConnector) {
                this.hitConnector = current as Connector
                hitConnectorFlag = true
            }

            if (current.isDesignerItem) {
                this.hitDesignerItem = current as DesignerItem
                if (!hitConnectorFlag) {
                    this.hitConnector = null
                }
                return
            }

            current = current.parentElement
        }

        this.hitConnector = null
        this.hitDesignerItem = null
    }

    private updatePathGeometry(position: { x: number, y: number }): any {
        if (!this.fixConnector) return null

        const targetOrientation = this.hitConnector 
            ? this.hitConnector.orientation 
            : (this.dragConnector?.orientation || 'Right')

        const linePoints = PathFinder.getConnectionLine(
            this.fixConnector.getInfo(),
            position,
            targetOrientation
        )

        if (linePoints.length > 0) {
            return {
                startPoint: linePoints[0],
                segments: linePoints.slice(1)
            }
        }
        return null
    }

    private updateSourceThumbPosition() {
        if (!this.isVisible) return
        // 更新源点拖拽thumb位置
    }

    private updateSinkThumbPosition() {
        if (!this.isVisible) return
        // 更新目标点拖拽thumb位置
    }

    public dispose() {
        this.designerCanvas.offDragStart(this.onDragStart)
        this.designerCanvas.offDrag(this.onDrag)
        this.designerCanvas.offDragEnd(this.onDragEnd)
        this.connection.offAnchorPositionChanged(this.onAnchorPositionChanged)
    }
}

// Vue组合式API封装
export const useConnectionAdorner = (designerCanvas: DesignerCanvas, connection: Connection) => {
    const adorner = reactive(new ConnectionAdorner(designerCanvas, connection))
    
    onUnmounted(() => {
        adorner.dispose()
    })

    return {
        adorner
    }
}