import { reactive, ref, onMounted, onUnmounted } from 'vue'
import type { DesignerCanvas } from './DesignerCanvas'
import type { DesignerItem } from './DesignerItem'
import type { Connection } from './Connection'

export enum ConnectorOrientation {
    None = 'None',
    Left = 'Left',
    Top = 'Top',
    Right = 'Right',
    Bottom = 'Bottom'
}

export interface ConnectorInfo {
    designerItemLeft: number
    designerItemTop: number
    designerItemSize: { width: number, height: number }
    position: { x: number, y: number }
    orientation: ConnectorOrientation
}

export class Connector {
    private _dragStartPoint: { x: number, y: number } | null = null
    private _orientation: ConnectorOrientation = ConnectorOrientation.None
    private _position = { x: 0, y: 0 }
    private _parentDesignerItem: DesignerItem | null = null
    private _connections: Set<Connection> = new Set()
    private _isActive = false

    get orientation(): ConnectorOrientation {
        return this._orientation
    }
    set orientation(value: ConnectorOrientation) {
        if (this._orientation !== value) {
            this._orientation = value
            this.onPropertyChanged('Orientation')
        }
    }

    get position(): { x: number, y: number } {
        return this._position
    }
    set position(value: { x: number, y: number }) {
        if (this._position.x !== value.x || this._position.y !== value.y) {
            this._position = value
            this.onPropertyChanged('Position')
        }
    }

    get parentDesignerItem(): DesignerItem | null {
        if (!this._parentDesignerItem) {
            // 从Vue上下文获取父设计器项
            this._parentDesignerItem = this.getParentDesignerItem()
        }
        return this._parentDesignerItem
    }

    get connections(): Connection[] {
        return Array.from(this._connections)
    }

    constructor(orientation: ConnectorOrientation = ConnectorOrientation.None) {
        this._orientation = orientation
    }

    // 布局更新时重新计算位置
    public updatePosition() {
        const designer = this.getDesignerCanvas()
        if (designer && this.parentDesignerItem) {
            // 计算相对于设计器画布的中心位置
            const itemPos = designer.getItemPosition(this.parentDesignerItem)
            const itemSize = this.parentDesignerItem.size
            
            let x = itemPos.x
            let y = itemPos.y
            
            // 根据方向计算连接点位置
            switch (this._orientation) {
                case ConnectorOrientation.Left:
                    x = itemPos.x
                    y = itemPos.y + itemSize.height / 2
                    break
                case ConnectorOrientation.Top:
                    x = itemPos.x + itemSize.width / 2
                    y = itemPos.y
                    break
                case ConnectorOrientation.Right:
                    x = itemPos.x + itemSize.width
                    y = itemPos.y + itemSize.height / 2
                    break
                case ConnectorOrientation.Bottom:
                    x = itemPos.x + itemSize.width / 2
                    y = itemPos.y + itemSize.height
                    break
                default: // Center
                    x = itemPos.x + itemSize.width / 2
                    y = itemPos.y + itemSize.height / 2
            }
            
            this.position = { x, y }
        }
    }

    // 鼠标按下事件处理
    public onMouseDown(event: MouseEvent) {
        const canvas = this.getDesignerCanvas()
        if (canvas) {
            this._dragStartPoint = {
                x: event.clientX - canvas.position.x,
                y: event.clientY - canvas.position.y
            }
            this._isActive = true
        }
    }

    // 鼠标移动事件处理
    public onMouseMove(event: MouseEvent) {
        if (!this._isActive || !this._dragStartPoint) return

        const canvas = this.getDesignerCanvas()
        if (canvas) {
            // 创建连接装饰器
            canvas.showConnectorAdorner(this)
        }
    }

    // 鼠标释放事件处理
    public onMouseUp() {
        this._isActive = false
        this._dragStartPoint = null
    }

    // 添加连接
    public addConnection(connection: Connection) {
        this._connections.add(connection)
    }

    // 移除连接
    public removeConnection(connection: Connection) {
        this._connections.delete(connection)
    }

    // 获取连接点信息
    public getInfo(): ConnectorInfo {
        if (!this.parentDesignerItem) {
            throw new Error('Parent designer item not found')
        }

        return {
            designerItemLeft: this.parentDesignerItem.position.x,
            designerItemTop: this.parentDesignerItem.position.y,
            designerItemSize: {
                width: this.parentDesignerItem.size.width,
                height: this.parentDesignerItem.size.height
            },
            position: this.position,
            orientation: this.orientation
        }
    }

    // 获取父设计器画布
    private getDesignerCanvas(): DesignerCanvas | null {
        // 在实际实现中需要遍历DOM树查找
        return null
    }

    // 获取父设计器项
    private getParentDesignerItem(): DesignerItem | null {
        // 在实际实现中需要从Vue上下文中获取
        return null
    }

    // 属性变更通知
    private onPropertyChanged(propertyName: string) {
        // 通知Vue响应式系统
        // 在实际实现中需要触发事件或调用回调
    }
}

// Vue组合式API封装
export const useConnector = (orientation: ConnectorOrientation = ConnectorOrientation.None) => {
    const connector = reactive(new Connector(orientation))
    
    onMounted(() => {
        connector.updatePosition()
    })

    return {
        connector,
        ConnectorOrientation
    }
}