import { reactive, onMounted, onUnmounted } from 'vue'
import type { DesignerCanvas } from './DesignerCanvas'
import type { Connector } from './Connector'
import type { DesignerItem } from './DesignerItem'
import type { Connection } from './Connection'
import { PathFinder } from './PathFinder'

export class ConnectorAdorner {
    private pathGeometry: any = null
    private designerCanvas: DesignerCanvas
    private sourceConnector: Connector
    private hitDesignerItem: DesignerItem | null = null
    private hitConnector: Connector | null = null
    private isActive = false

    constructor(designerCanvas: DesignerCanvas, sourceConnector: Connector) {
        this.designerCanvas = designerCanvas
        this.sourceConnector = sourceConnector
        this.init()
    }

    private init() {
        this.designerCanvas.onMouseMove(this.onMouseMove)
        this.designerCanvas.onMouseUp(this.onMouseUp)
    }

    private onMouseMove = (event: MouseEvent) => {
        if (!this.isActive) return

        const position = {
            x: event.clientX - this.designerCanvas.position.x,
            y: event.clientY - this.designerCanvas.position.y
        }

        this.hitTesting(position)
        this.pathGeometry = this.getPathGeometry(position)
        this.designerCanvas.requestRender()
    }

    private onMouseUp = (event: MouseEvent) => {
        if (!this.isActive) return

        if (this.hitConnector) {
            const newConnection = new Connection(
                this.sourceConnector,
                this.hitConnector
            )
            this.designerCanvas.addConnection(newConnection)
        }

        if (this.hitDesignerItem) {
            this.hitDesignerItem.isDragConnectionOver = false
        }

        this.isActive = false
        this.designerCanvas.hideConnectorAdorner()
    }

    public activate() {
        this.isActive = true
    }

    private hitTesting(position: { x: number, y: number }) {
        let hitConnectorFlag = false
        const hitElement = this.designerCanvas.getElementFromPoint(position)

        let current = hitElement
        while (current) {
            if (current.isConnector) {
                this.hitConnector = current as Connector
                hitConnectorFlag = true
            }

            if (current.isDesignerItem) {
                this.hitDesignerItem = current as DesignerItem
                if (!hitConnectorFlag) {
                    this.hitConnector = null
                }
                return
            }

            current = current.parentElement
        }

        this.hitConnector = null
        this.hitDesignerItem = null
    }

    private getPathGeometry(position: { x: number, y: number }): any {
        const targetOrientation = this.hitConnector 
            ? this.hitConnector.orientation 
            : 'None'

        const pathPoints = PathFinder.getConnectionLine(
            this.sourceConnector.getInfo(),
            position,
            targetOrientation
        )

        if (pathPoints.length > 0) {
            return {
                startPoint: pathPoints[0],
                segments: pathPoints.slice(1)
            }
        }
        return null
    }

    public render(ctx: CanvasRenderingContext2D) {
        if (!this.pathGeometry) return

        ctx.beginPath()
        ctx.moveTo(this.pathGeometry.startPoint.x, this.pathGeometry.startPoint.y)
        
        this.pathGeometry.segments.forEach((point: {x: number, y: number}) => {
            ctx.lineTo(point.x, point.y)
        })

        ctx.strokeStyle = '#778899'
        ctx.lineWidth = 1
        ctx.lineJoin = 'round'
        ctx.stroke()
    }

    public dispose() {
        this.designerCanvas.offMouseMove(this.onMouseMove)
        this.designerCanvas.offMouseUp(this.onMouseUp)
    }
}

// Vue组合式API封装
export const useConnectorAdorner = (designerCanvas: DesignerCanvas, sourceConnector: Connector) => {
    const adorner = reactive(new ConnectorAdorner(designerCanvas, sourceConnector))
    
    onUnmounted(() => {
        adorner.dispose()
    })

    return {
        adorner
    }
}