import { reactive, ref, onMounted, nextTick } from 'vue'
import type { DesignerItem } from './DesignerItem'
import type { Connection } from './Connection'
import type { Connector } from './Connector'
import { DesignerItemTip } from './DesignerItemTip'

interface TxNode {
    code: string
    parentCode: string
    name: string
    content: string
    remark: string
    stat: string
    errInfo: string
    x: number
    y: number
    subCode: string
    conditions: string
}

interface TxEntry {
    startNode: string
    endNode: string
    condition: string
}

export class DesignerCanvas {
    private rubberbandSelectionStartPoint: {x: number, y: number} | null = null
    private selectedItems: DesignerItem[] = []
    private layoutCount = 0
    private readonly START_X = 20
    private readonly START_Y = 10
    private readonly OFFSET_X = 220
    private readonly OFFSET_Y = 120
    private leftestX = Number.MAX_VALUE
    private startNode = "0"
    private endNode = "0"
    private lstNodes: TxNode[] = []
    private nodes: TxNode[] = []
    private lstHeadNodes: TxNode[] = []
    private lstEntrys: TxEntry[] = []
    private canvasRef: HTMLElement | null = null

    constructor(canvasElement: HTMLElement) {
        this.canvasRef = canvasElement
    }

    getSelectedItems(): DesignerItem[] {
        return this.selectedItems
    }

    clearSelection() {
        this.selectedItems.forEach(item => item.isSelected = false)
        this.selectedItems = []
    }

    async loadFlowChat(flowList: TxNode[]) {
        this.lstNodes = []
        this.nodes = []
        this.lstEntrys = []
        
        if (!flowList || flowList.length < 1) return
        
        this.startNode = flowList.reduce((min, item) => 
            item.code < min ? item.code : min, flowList[0].code)
        this.endNode = (flowList.length + 1).toString()
        
        this.nodes.push({
            code: this.startNode,
            parentCode: "",
            name: "开始",
            content: "",
            remark: "",
            stat: "",
            errInfo: "",
            x: 0,
            y: 0,
            subCode: "",
            conditions: ""
        })
        
        this.nodes.push({
            code: this.endNode,
            parentCode: "",
            name: "结束", 
            content: "",
            remark: "",
            stat: "",
            errInfo: "",
            x: 0,
            y: 0,
            subCode: "",
            conditions: ""
        })
        
        this.layout(flowList)
        await this.renderNodes()
        await this.renderConnections()
    }

    private async renderNodes() {
        if (!this.canvasRef) return
        
        for (const node of this.lstNodes) {
            const item = new DesignerItem(node.code)
            item.name = node.name
            item.remark = node.remark
            item.errorInfo = node.errInfo
            
            switch(node.stat.toUpperCase()) {
                case "Z":
                    item.tipSymbol = DesignerItemTip.Error
                    item.tooltip = "结账失败"
                    break
                case "Y":
                    item.tipSymbol = DesignerItemTip.Success
                    item.tooltip = "结账成功"
                    break
                case "N":
                    item.tipSymbol = DesignerItemTip.UnHandled
                    item.tooltip = "未处理"
                    break
                case "S":
                    item.tipSymbol = DesignerItemTip.None
                    item.tooltip = "正在执行"
                    break
                default:
                    item.tipSymbol = DesignerItemTip.None
            }
            
            item.position = {
                x: this.START_X + node.x,
                y: this.START_Y + node.y
            }
            
            this.canvasRef.appendChild(item.element)
            await nextTick()
        }
    }

    private async renderConnections() {
        if (!this.canvasRef) return
        
        for (const entry of this.lstEntrys) {
            const sourceConnector = this.getConnector(entry.startNode, "bottom")
            const sinkConnector = this.getConnector(entry.endNode, "top")
            
            if (sourceConnector && sinkConnector) {
                const connection = new Connection(
                    sourceConnector, 
                    sinkConnector,
                    entry.condition
                )
                this.canvasRef.appendChild(connection.element)
                await nextTick()
            }
        }
    }

    private layout(flowList: TxNode[]) {
        this.layoutCount = 0
        
        // 构建节点和连接关系
        flowList.forEach(item => {
            const subCodes = item.subCode.split(',')
            subCodes.forEach(k => {
                this.lstEntrys.push({
                    startNode: k,
                    endNode: item.code,
                    condition: item.conditions
                })
            })
            this.nodes.push(item)
        })
        
        // 查找结束节点并添加虚拟连接
        const lastNode = this.findLastNode(flowList)
        if (lastNode) {
            this.lstEntrys.push({
                startNode: lastNode.code,
                endNode: this.endNode,
                condition: ""
            })
        }
        
        // 排序节点
        this.nodes.sort((a, b) => parseInt(a.code) - parseInt(b.code))
        
        // 布局起始节点
        const firstNode = this.findFirstNode(this.startNode)
        if (firstNode) {
            firstNode.x = this.START_X
            firstNode.y = this.START_Y
            this.layoutCount++
            this.lstNodes.push(firstNode)
            this.layoutNextNodes(firstNode)
        }
        
        // 布局其他节点
        this.lstHeadNodes = this.getHeadNodes()
        this.nodes.forEach(node => {
            if (this.lstHeadNodes.some(n => n.code === node.code)) {
                node.x = this.START_X + 100
                node.y = this.START_Y
                this.layoutCount++
                this.lstNodes.push(node)
                this.layoutNextNodes(node)
            }
            this.layoutNextNodes(node)
        })
    }

    private layoutNextNodes(node: TxNode) {
        const nextNodes = this.findNextNodes(node)
        if (!nextNodes || nextNodes.length === 0) return
        
        const half = nextNodes.length % 2 === 0 
            ? nextNodes.length / 2 - 1 
            : Math.floor(nextNodes.length / 2)
            
        for (let i = 0; i < nextNodes.length; i++) {
            if (nextNodes[i].x === 0 && nextNodes[i].y === 0) {
                nextNodes[i].y = node.y + this.OFFSET_Y
                
                if (nextNodes[i].code !== this.endNode) {
                    nextNodes[i].x = node.x + (i - half) * this.OFFSET_X
                    
                    if (node.subCode) {
                        const subNodes = node.subCode.split(',')
                        if (subNodes.length > 1) {
                            // 处理多分支情况
                        }
                    }
                } else {
                    nextNodes[i].x = this.START_X
                    const maxY = this.lstNodes.reduce((max, n) => 
                        n.y > max ? n.y : max, 0)
                    nextNodes[i].y = maxY + this.OFFSET_Y
                }
                
                this.lstNodes.push(nextNodes[i])
                this.leftestX = Math.min(this.leftestX, nextNodes[i].x)
                
                if (nextNodes.length === 1) {
                    this.layoutCount++
                    this.layoutNextNodes(nextNodes[i])
                }
            }
        }
        
        if (nextNodes.length > 1) {
            this.layoutCount++
        }
    }

    private findFirstNode(code: string): TxNode | undefined {
        return this.nodes.find(n => n.code === code)
    }

    private findLastNode(flowList: TxNode[]): TxNode | undefined {
        for (const node of flowList) {
            const isLast = !flowList.some(n => 
                n.subCode.split(',').includes(node.code))
            if (isLast) return node
        }
    }

    private findNextNodes(node: TxNode): TxNode[] {
        return this.lstEntrys
            .filter(e => e.startNode === node.code)
            .map(e => this.nodes.find(n => n.code === e.endNode))
            .filter(Boolean) as TxNode[]
    }

    private getHeadNodes(): TxNode[] {
        return this.nodes.filter(n => {
            if (n.code === this.startNode || n.code === this.endNode) {
                return false
            }
            return !this.lstEntrys.some(e => e.endNode === n.code)
        })
    }

    private getConnector(itemId: string, position: string): Connector | null {
        // 实际实现中需要查找DOM中的连接点元素
        return null
    }

    handleMouseDown(e: MouseEvent) {
        if (e.target === this.canvasRef) {
            this.rubberbandSelectionStartPoint = {
                x: e.clientX,
                y: e.clientY
            }
            this.clearSelection()
        }
    }

    handleMouseMove(e: MouseEvent) {
        if (!this.rubberbandSelectionStartPoint) return
        
        // 实现框选逻辑
        // ...
    }

    handleMouseUp() {
        this.rubberbandSelectionStartPoint = null
    }
}

// Vue组合式API封装
export const useDesignerCanvas = (canvasElement: HTMLElement) => {
    const canvas = reactive(new DesignerCanvas(canvasElement))
    
    onMounted(() => {
        canvasElement.addEventListener('mousedown', canvas.handleMouseDown)
        canvasElement.addEventListener('mousemove', canvas.handleMouseMove)
        canvasElement.addEventListener('mouseup', canvas.handleMouseUp)
    })
    
    onUnmounted(() => {
        canvasElement.removeEventListener('mousedown', canvas.handleMouseDown)
        canvasElement.removeEventListener('mousemove', canvas.handleMouseMove)
        canvasElement.removeEventListener('mouseup', canvas.handleMouseUp)
    })
    
    return {
        canvas
    }
}