import { reactive, onMounted, onUnmounted } from 'vue'

export class DesignerCanvasAdorner {
    private adornedElement: HTMLElement | null = null
    private scalingGrip: HTMLElement | null = null
    private isDragging = false
    private startX = 0
    private startY = 0
    private startWidth = 0
    private startHeight = 0

    constructor(element: HTMLElement) {
        this.adornedElement = element
        this.createGrip()
    }

    private createGrip() {
        if (!this.adornedElement) return

        // 创建缩放控制柄
        const grip = document.createElement('div')
        grip.style.position = 'absolute'
        grip.style.width = '10px'
        grip.style.height = '10px'
        grip.style.right = '0'
        grip.style.bottom = '0'
        grip.style.backgroundColor = 'red'
        grip.style.border = '1px solid blue'
        grip.style.cursor = 'nwse-resize'
        
        grip.addEventListener('mousedown', this.onGripMouseDown)
        this.adornedElement.appendChild(grip)
        this.scalingGrip = grip

        // 添加选中边框
        this.adornedElement.style.border = '1px dashed navy'
    }

    private onGripMouseDown = (e: MouseEvent) => {
        if (!this.adornedElement) return

        this.isDragging = true
        this.startX = e.clientX
        this.startY = e.clientY
        this.startWidth = parseInt(document.defaultView?.getComputedStyle(this.adornedElement).width || '0', 10)
        this.startHeight = parseInt(document.defaultView?.getComputedStyle(this.adornedElement).height || '0', 10)
        
        document.addEventListener('mousemove', this.onGripMouseMove)
        document.addEventListener('mouseup', this.onGripMouseUp)
    }

    private onGripMouseMove = (e: MouseEvent) => {
        if (!this.isDragging || !this.adornedElement) return

        const dx = e.clientX - this.startX
        const dy = e.clientY - this.startY
        
        this.adornedElement.style.width = `${Math.max(0, this.startWidth + dx)}px`
        this.adornedElement.style.height = `${Math.max(0, this.startHeight + dy)}px`
    }

    private onGripMouseUp = () => {
        this.isDragging = false
        document.removeEventListener('mousemove', this.onGripMouseMove)
        document.removeEventListener('mouseup', this.onGripMouseUp)
    }

    public dispose() {
        if (this.scalingGrip) {
            this.scalingGrip.removeEventListener('mousedown', this.onGripMouseDown)
            this.scalingGrip.remove()
        }
        
        if (this.adornedElement) {
            this.adornedElement.style.border = 'none'
        }
        
        document.removeEventListener('mousemove', this.onGripMouseMove)
        document.removeEventListener('mouseup', this.onGripMouseUp)
    }
}

// Vue组合式API封装
export const useDesignerCanvasAdorner = (elementRef: Ref<HTMLElement | null>) => {
    const adorner = reactive<DesignerCanvasAdorner | null>(null)

    onMounted(() => {
        if (elementRef.value) {
            adorner = new DesignerCanvasAdorner(elementRef.value)
        }
    })

    onUnmounted(() => {
        adorner?.dispose()
    })

    return {
        adorner
    }
}