import { reactive, ref, computed, onMounted } from 'vue'
import type { DesignerCanvas } from './DesignerCanvas'

export enum DesignerItemTip {
    None = 'None',
    Error = 'Error',
    UnHandled = 'UnHandled',
    Execute = 'Execute',
    Success = 'Success'
}

export class DesignerItem {
    private _isSelected = false
    private _isDragConnectionOver = false
    private _sourceTipSymbol = DesignerItemTip.None
    private _currentSerialNumber: string
    private _remark = ''
    private _errorInfo = ''
    private _element: HTMLElement | null = null
    private _parentCanvas: DesignerCanvas | null = null

    constructor(key: string) {
        this._currentSerialNumber = key
    }

    get isSelected(): boolean {
        return this._isSelected
    }
    set isSelected(value: boolean) {
        if (this._isSelected !== value) {
            this._isSelected = value
            this.updateSelectionStyle()
        }
    }

    get isDragConnectionOver(): boolean {
        return this._isDragConnectionOver
    }
    set isDragConnectionOver(value: boolean) {
        if (this._isDragConnectionOver !== value) {
            this._isDragConnectionOver = value
            this.updateConnectorVisibility()
        }
    }

    get sourceTipSymbol(): DesignerItemTip {
        return this._sourceTipSymbol
    }
    set sourceTipSymbol(value: DesignerItemTip) {
        if (this._sourceTipSymbol !== value) {
            this._sourceTipSymbol = value
            this.updateTipSymbol()
        }
    }

    get currentSerialNumber(): string {
        return this._currentSerialNumber
    }

    get remark(): string {
        return this._remark
    }
    set remark(value: string) {
        this._remark = value
    }

    get errorInfo(): string {
        return this._errorInfo
    }
    set errorInfo(value: string) {
        this._errorInfo = value
    }

    public initialize(element: HTMLElement, parentCanvas: DesignerCanvas) {
        this._element = element
        this._parentCanvas = parentCanvas
        this.setupEventListeners()
        this.render()
    }

    private setupEventListeners() {
        if (!this._element) return

        this._element.addEventListener('mousedown', this.onMouseDown)
        this._element.addEventListener('dblclick', this.onDoubleClick)
    }

    private onMouseDown = (e: MouseEvent) => {
        if (!this._parentCanvas) return

        const isCtrlPressed = e.ctrlKey || e.metaKey
        const isShiftPressed = e.shiftKey

        if (isCtrlPressed || isShiftPressed) {
            if (this._isSelected) {
                this._isSelected = false
                this._parentCanvas.removeSelectedItem(this)
            } else {
                this._isSelected = true
                this._parentCanvas.addSelectedItem(this)
            }
        } else if (!this._isSelected) {
            this._parentCanvas.clearSelection()
            this._isSelected = true
            this._parentCanvas.addSelectedItem(this)
        }

        e.stopPropagation()
    }

    private onDoubleClick = () => {
        this.showRemarkPopup()
    }

    private showRemarkPopup() {
        if (!this._element) return

        const popup = document.createElement('div')
        popup.style.position = 'absolute'
        popup.style.zIndex = '1000'
        popup.style.border = '5px solid darkgreen'
        popup.style.borderRadius = '5px'
        popup.style.backgroundColor = 'white'
        popup.style.width = '400px'
        popup.style.padding = '10px'

        const content = document.createElement('div')
        content.style.width = '360px'
        content.style.fontSize = '15px'
        content.style.wordWrap = 'break-word'
        content.textContent = this._remark

        popup.appendChild(content)
        document.body.appendChild(popup)

        const rect = this._element.getBoundingClientRect()
        popup.style.left = `${rect.left}px`
        popup.style.top = `${rect.top}px`

        const closePopup = () => {
            document.body.removeChild(popup)
            document.removeEventListener('mousedown', closePopup)
        }

        document.addEventListener('mousedown', closePopup)
    }

    private updateSelectionStyle() {
        if (!this._element) return

        if (this._isSelected) {
            this._element.style.boxShadow = '0 0 5px 2px rgba(0, 0, 255, 0.5)'
        } else {
            this._element.style.boxShadow = 'none'
        }
    }

    private updateConnectorVisibility() {
        // 在实际实现中更新连接器可见性
    }

    private updateTipSymbol() {
        if (!this._element) return

        const tipElement = this._element.querySelector('.tip-symbol')
        if (tipElement) {
            tipElement.className = `tip-symbol ${this._sourceTipSymbol.toLowerCase()}`
        }
    }

    private render() {
        if (!this._element) return

        this._element.innerHTML = `
            <div class="designer-item">
                <div class="header">
                    <span class="serial">${this._currentSerialNumber}</span>
                    <span class="tip-symbol ${this._sourceTipSymbol.toLowerCase()}"></span>
                </div>
                <div class="content">
                    <!-- 内容区域 -->
                </div>
                <div class="connectors">
                    <!-- 连接点区域 -->
                </div>
            </div>
        `
    }
}

// Vue组合式API封装
export const useDesignerItem = (key: string) => {
    const item = reactive(new DesignerItem(key))
    
    onMounted(() => {
        const element = document.getElementById(`designer-item-${key}`)
        if (element) {
            item.initialize(element, /* 传入父画布实例 */)
        }
    })

    return {
        item,
        DesignerItemTip
    }
}