import { reactive, onMounted, onUnmounted } from 'vue'
import type { DesignerCanvas } from './DesignerCanvas'
import type { DesignerItem } from './DesignerItem'

export class DragThumb {
    private element: HTMLElement | null = null
    private startX = 0
    private startY = 0
    private isDragging = false
    private designerCanvas: DesignerCanvas | null = null

    constructor(element: HTMLElement, canvas: DesignerCanvas) {
        this.element = element
        this.designerCanvas = canvas
        this.setupDragHandlers()
    }

    private setupDragHandlers() {
        if (!this.element) return

        this.element.addEventListener('mousedown', this.onMouseDown)
        document.addEventListener('mousemove', this.onMouseMove)
        document.addEventListener('mouseup', this.onMouseUp)
    }

    private onMouseDown = (e: MouseEvent) => {
        if (!this.designerCanvas) return

        this.isDragging = true
        this.startX = e.clientX
        this.startY = e.clientY
        e.preventDefault()
    }

    private onMouseMove = (e: MouseEvent) => {
        if (!this.isDragging || !this.designerCanvas) return

        const deltaX = e.clientX - this.startX
        const deltaY = e.clientY - this.startY

        const selectedItems = this.designerCanvas.getSelectedItems()
        if (selectedItems.length === 0) return

        // 计算最小偏移量防止移出画布
        let minLeft = Infinity
        let minTop = Infinity
        
        selectedItems.forEach(item => {
            minLeft = Math.min(minLeft, item.position.x)
            minTop = Math.min(minTop, item.position.y)
        })

        const actualDeltaX = Math.max(-minLeft, deltaX)
        const actualDeltaY = Math.max(-minTop, deltaY)

        // 更新所有选中项位置
        selectedItems.forEach(item => {
            item.position = {
                x: item.position.x + actualDeltaX,
                y: item.position.y + actualDeltaY
            }
        })

        this.startX = e.clientX
        this.startY = e.clientY
    }

    private onMouseUp = () => {
        this.isDragging = false
    }

    public dispose() {
        if (!this.element) return

        this.element.removeEventListener('mousedown', this.onMouseDown)
        document.removeEventListener('mousemove', this.onMouseMove)
        document.removeEventListener('mouseup', this.onMouseUp)
    }
}

// Vue组合式API封装
export const useDragThumb = (elementRef: HTMLElement, canvas: DesignerCanvas) => {
    const dragThumb = reactive(new DragThumb(elementRef, canvas))
    
    onUnmounted(() => {
        dragThumb.dispose()
    })

    return {
        dragThumb
    }
}

export interface DragObject {
    xaml: string
    desiredSize?: { width: number; height: number }
}