// 定义枚举类型
enum ConnectorOrientation {
  Left,
  Top,
  Right,
  Bottom,
  None
}

// 定义ConnectorInfo接口
interface ConnectorInfo {
  DesignerItemLeft: number;
  DesignerItemTop: number;
  DesignerItemSize: { Width: number; Height: number };
  Orientation: ConnectorOrientation;
  Position: { X: number; Y: number };
}

// 定义PathFinder类
class PathFinder {
  private static readonly margin = 20;

  // 获取连接线路径
  public static GetConnectionLine(source: ConnectorInfo, sink: ConnectorInfo, showLastLine: boolean): { X: number; Y: number }[] {
    let linePoints: { X: number; Y: number }[] = [];

    const rectSource = this.GetRectWithMargin(source, this.margin);
    const rectSink = this.GetRectWithMargin(sink, this.margin);

    const startPoint = this.GetOffsetPoint(source, rectSource);
    const endPoint = this.GetOffsetPoint(sink, rectSink);

    linePoints.push(startPoint);
    let currentPoint = startPoint;

    if (!this.isPointInRect(currentPoint, rectSink) && !this.isPointInRect(endPoint, rectSource)) {
      while (true) {
        // 源节点处理
        if (this.IsPointVisible(currentPoint, endPoint, [rectSource, rectSink])) {
          linePoints.push(endPoint);
          currentPoint = endPoint;
          break;
        }

        const neighbour = this.GetNearestVisibleNeighborSink(currentPoint, endPoint, sink, rectSource, rectSink);
        if (!isNaN(neighbour.X)) {
          linePoints.push(neighbour);
          linePoints.push(endPoint);
          currentPoint = endPoint;
          break;
        }

        if (this.isSamePoint(currentPoint, startPoint)) {
          let flag: boolean;
          const n = this.GetNearestNeighborSource(source, endPoint, rectSource, rectSink, flag);
          linePoints.push(n);
          currentPoint = n;

          if (!this.IsRectVisible(currentPoint, rectSink, [rectSource])) {
            let n1: { X: number; Y: number };
            let n2: { X: number; Y: number };
            this.GetOppositeCorners(source.Orientation, rectSource, n1, n2);
            if (flag) {
              linePoints.push(n1);
              currentPoint = n1;
            } else {
              linePoints.push(n2);
              currentPoint = n2;
            }
            if (!this.IsRectVisible(currentPoint, rectSink, [rectSource])) {
              if (flag) {
                linePoints.push(n2);
                currentPoint = n2;
              } else {
                linePoints.push(n1);
                currentPoint = n1;
              }
            }
          }
        }

        // 目标节点处理
        else {
          let n1: { X: number; Y: number };
          let n2: { X: number; Y: number }; // 邻居角点
          let s1: { X: number; Y: number };
          let s2: { X: number; Y: number }; // 对角点
          this.GetNeighborCorners(sink.Orientation, rectSink, n1, n2);
          this.GetOppositeCorners(sink.Orientation, rectSink, s1, s2);

          const n1Visible = this.IsPointVisible(currentPoint, n1, [rectSource, rectSink]);
          const n2Visible = this.IsPointVisible(currentPoint, n2, [rectSource, rectSink]);

          if (n1Visible && n2Visible) {
            if (this.isPointInRect(n1, rectSource)) {
              linePoints.push(n2);
              if (this.isPointInRect(s2, rectSource)) {
                linePoints.push(n1);
                linePoints.push(s1);
              } else {
                linePoints.push(s2);
              }
              linePoints.push(endPoint);
              currentPoint = endPoint;
              break;
            }

            if (this.isPointInRect(n2, rectSource)) {
              linePoints.push(n1);
              if (this.isPointInRect(s1, rectSource)) {
                linePoints.push(n2);
                linePoints.push(s2);
              } else {
                linePoints.push(s1);
              }
              linePoints.push(endPoint);
              currentPoint = endPoint;
              break;
            }

            if (this.Distance(n1, endPoint) <= this.Distance(n2, endPoint)) {
              linePoints.push(n1);
              if (this.isPointInRect(s1, rectSource)) {
                linePoints.push(n2);
                linePoints.push(s2);
              } else {
                linePoints.push(s1);
              }
              linePoints.push(endPoint);
              currentPoint = endPoint;
              break;
            } else {
              linePoints.push(n2);
              if (this.isPointInRect(s2, rectSource)) {
                linePoints.push(n1);
                linePoints.push(s1);
              } else {
                linePoints.push(s2);
              }
              linePoints.push(endPoint);
              currentPoint = endPoint;
              break;
            }
          } else if (n1Visible) {
            linePoints.push(n1);
            if (this.isPointInRect(s1, rectSource)) {
              linePoints.push(n2);
              linePoints.push(s2);
            } else {
              linePoints.push(s1);
            }
            linePoints.push(endPoint);
            currentPoint = endPoint;
            break;
          } else {
            linePoints.push(n2);
            if (this.isPointInRect(s2, rectSource)) {
              linePoints.push(n1);
              linePoints.push(s1);
            } else {
              linePoints.push(s2);
            }
            linePoints.push(endPoint);
            currentPoint = endPoint;
            break;
          }
        }
      }
    } else {
      linePoints.push(endPoint);
    }

    linePoints = this.OptimizeLinePoints(linePoints, [rectSource, rectSink], source.Orientation, sink.Orientation);

    this.CheckPathEnd(source, sink, showLastLine, linePoints);
    return linePoints;
  }

  // 获取连接线路径（另一个重载方法）
  public static GetConnectionLine2(source: ConnectorInfo, sinkPoint: { X: number; Y: number }, preferredOrientation: ConnectorOrientation): { X: number; Y: number }[] {
    let linePoints: { X: number; Y: number }[] = [];
    const rectSource = this.GetRectWithMargin(source, 10);
    const startPoint = this.GetOffsetPoint(source, rectSource);
    const endPoint = sinkPoint;

    linePoints.push(startPoint);
    let currentPoint = startPoint;

    if (!this.isPointInRect(endPoint, rectSource)) {
      while (true) {
        if (this.IsPointVisible(currentPoint, endPoint, [rectSource])) {
          linePoints.push(endPoint);
          break;
        }

        let sideFlag: boolean;
        const n = this.GetNearestNeighborSource(source, endPoint, rectSource, sideFlag);
        linePoints.push(n);
        currentPoint = n;

        if (this.IsPointVisible(currentPoint, endPoint, [rectSource])) {
          linePoints.push(endPoint);
          break;
        } else {
          let n1: { X: number; Y: number };
          let n2: { X: number; Y: number };
          this.GetOppositeCorners(source.Orientation, rectSource, n1, n2);
          if (sideFlag) {
            linePoints.push(n1);
          } else {
            linePoints.push(n2);
          }
          linePoints.push(endPoint);
          break;
        }
      }
    } else {
      linePoints.push(endPoint);
    }

    if (preferredOrientation !== ConnectorOrientation.None) {
      linePoints = this.OptimizeLinePoints(linePoints, [rectSource], source.Orientation, preferredOrientation);
    } else {
      linePoints = this.OptimizeLinePoints(linePoints, [rectSource], source.Orientation, this.GetOpositeOrientation(source.Orientation));
    }

    return linePoints;
  }

  // 优化线路点
  private static OptimizeLinePoints(linePoints: { X: number; Y: number }[], rectangles: { Left: number; Top: number; Width: number; Height: number }[], sourceOrientation: ConnectorOrientation, sinkOrientation: ConnectorOrientation): { X: number; Y: number }[] {
    let points: { X: number; Y: number }[] = [];
    let cut = 0;

    for (let i = 0; i < linePoints.length; i++) {
      if (i >= cut) {
        for (let k = linePoints.length - 1; k > i; k--) {
          if (this.IsPointVisible(linePoints[i], linePoints[k], rectangles)) {
            cut = k;
            break;
          }
        }
        points.push(linePoints[i]);
      }
    }

    for (let j = 0; j < points.length - 1; j++) {
      if (points[j].X !== points[j + 1].X && points[j].Y !== points[j + 1].Y) {
        let orientationFrom: ConnectorOrientation;
        let orientationTo: ConnectorOrientation;

        // 起始点方向
        if (j === 0) {
          orientationFrom = sourceOrientation;
        } else {
          orientationFrom = this.GetOrientation(points[j], points[j - 1]);
        }

        // 目标点方向
        if (j === points.length - 2) {
          orientationTo = sinkOrientation;
        } else {
          orientationTo = this.GetOrientation(points[j + 1], points[j + 2]);
        }

        if ((orientationFrom === ConnectorOrientation.Left || orientationFrom === ConnectorOrientation.Right) &&
          (orientationTo === ConnectorOrientation.Left || orientationTo === ConnectorOrientation.Right)) {
          const centerX = Math.min(points[j].X, points[j + 1].X) + Math.abs(points[j].X - points[j + 1].X) / 2;
          points.splice(j + 1, 0, { X: centerX, Y: points[j].Y });
          points.splice(j + 2, 0, { X: centerX, Y: points[j + 2].Y });
          if (points.length - 1 > j + 3) {
            points.splice(j + 3, 1);
          }
          return points;
        }

        if ((orientationFrom === ConnectorOrientation.Top || orientationFrom === ConnectorOrientation.Bottom) &&
          (orientationTo === ConnectorOrientation.Top || orientationTo === ConnectorOrientation.Bottom)) {
          const centerY = Math.min(points[j].Y, points[j + 1].Y) + Math.abs(points[j].Y - points[j + 1].Y) / 2;
          points.splice(j + 1, 0, { X: points[j].X, Y: centerY });
          points.splice(j + 2, 0, { X: points[j + 2].X, Y: centerY });
          if (points.length - 1 > j + 3) {
            points.splice(j + 3, 1);
          }
          return points;
        }

        if ((orientationFrom === ConnectorOrientation.Left || orientationFrom === ConnectorOrientation.Right) &&
          (orientationTo === ConnectorOrientation.Top || orientationTo === ConnectorOrientation.Bottom)) {
          points.splice(j + 1, 0, { X: points[j + 1].X, Y: points[j].Y });
          return points;
        }

        if ((orientationFrom === ConnectorOrientation.Top || orientationFrom === ConnectorOrientation.Bottom) &&
          (orientationTo === ConnectorOrientation.Left || orientationTo === ConnectorOrientation.Right)) {
          points.splice(j + 1, 0, { X: points[j].X, Y: points[j + 1].Y });
          return points;
        }
      }
    }

    return points;
  }

  // 获取方向
  private static GetOrientation(p1: { X: number; Y: number }, p2: { X: number; Y: number }): ConnectorOrientation {
    if (p1.X === p2.X) {
      if (p1.Y >= p2.Y) {
        return ConnectorOrientation.Bottom;
      } else {
        return ConnectorOrientation.Top;
      }
    } else if (p1.Y === p2.Y) {
      if (p1.X >= p2.X) {
        return ConnectorOrientation.Right;
      } else {
        return ConnectorOrientation.Left;
      }
    }
    throw new Error('Failed to retrieve orientation');
  }

  // 获取方向（另一个重载方法）
  private static GetOrientation2(sourceOrientation: ConnectorOrientation): 'horizontal' | 'vertical' {
    switch (sourceOrientation) {
      case ConnectorOrientation.Left:
        return 'horizontal';
      case ConnectorOrientation.Top:
        return 'vertical';
      case ConnectorOrientation.Right:
        return 'horizontal';
      case ConnectorOrientation.Bottom:
        return 'vertical';
      default:
        throw new Error('Unknown ConnectorOrientation');
    }
  }

  // 获取最近的源邻居点
  private static GetNearestNeighborSource(source: ConnectorInfo, endPoint: { X: number; Y: number }, rectSource: { Left: number; Top: number; Width: number; Height: number }, rectSink: { Left: number; Top: number; Width: number; Height: number }, flag: boolean): { X: number; Y: number } {
    let n1: { X: number; Y: number };
    let n2: { X: number; Y: number }; // 邻居点
    this.GetNeighborCorners(source.Orientation, rectSource, n1, n2);

    if (this.isPointInRect(n1, rectSink)) {
      flag = false;
      return n2;
    }

    if (this.isPointInRect(n2, rectSink)) {
      flag = true;
      return n1;
    }

    if (this.Distance(n1, endPoint) <= this.Distance(n2, endPoint)) {
      flag = true;
      return n1;
    } else {
      flag = false;
      return n2;
    }
  }

  // 获取最近的源邻居点（另一个重载方法）
  private static GetNearestNeighborSource2(source: ConnectorInfo, endPoint: { X: number; Y: number }, rectSource: { Left: number; Top: number; Width: number; Height: number }, flag: boolean): { X: number; Y: number } {
    let n1: { X: number; Y: number };
    let n2: { X: number; Y: number }; // 邻居点
    this.GetNeighborCorners(source.Orientation, rectSource, n1, n2);

    if (this.Distance(n1, endPoint) <= this.Distance(n2, endPoint)) {
      flag = true;
      return n1;
    } else {
      flag = false;
      return n2;
    }
  }

  // 获取最近的可见目标邻居点
  private static GetNearestVisibleNeighborSink(currentPoint: { X: number; Y: number }, endPoint: { X: number; Y: number }, sink: ConnectorInfo, rectSource: { Left: number; Top: number; Width: number; Height: number }, rectSink: { Left: number; Top: number; Width: number; Height: number }): { X: number; Y: number } {
    let s1: { X: number; Y: number };
    let s2: { X: number; Y: number }; // 目标侧邻居点
    this.GetNeighborCorners(sink.Orientation, rectSink, s1, s2);

    const flag1 = this.IsPointVisible(currentPoint, s1, [rectSource, rectSink]);
    const flag2 = this.IsPointVisible(currentPoint, s2, [rectSource, rectSink]);

    if (flag1) { // s1可见
      if (flag2) { // s1和s2都可见
        if (this.isPointInRect(s1, rectSink)) {
          return s2;
        }

        if (this.isPointInRect(s2, rectSink)) {
          return s1;
        }

        if (this.Distance(s1, endPoint) <= this.Distance(s2, endPoint)) {
          return s1;
        } else {
          return s2;
        }
      } else {
        return s1;
      }
    } else { // s1不可见
      if (flag2) { // 只有s2可见
        return s2;
      } else { // s1和s2都不可见
        return { X: NaN, Y: NaN };
      }
    }
  }

  // 判断点是否可见
  private static IsPointVisible(fromPoint: { X: number; Y: number }, targetPoint: { X: number; Y: number }, rectangles: { Left: number; Top: number; Width: number; Height: number }[]): boolean {
    for (const rect of rectangles) {
      if (this.RectangleIntersectsLine(rect, fromPoint, targetPoint)) {
        return false;
      }
    }
    return true;
  }

  // 判断矩形是否可见
  private static IsRectVisible(fromPoint: { X: number; Y: number }, targetRect: { Left: number; Top: number; Width: number; Height: number }, rectangles: { Left: number; Top: number; Width: number; Height: number }[]): boolean {
    if (this.IsPointVisible(fromPoint, { X: targetRect.Left, Y: targetRect.Top }, rectangles)) {
      return true;
    }

    if (this.IsPointVisible(fromPoint, { X: targetRect.Left + targetRect.Width, Y: targetRect.Top }, rectangles)) {
      return true;
    }

    if (this.IsPointVisible(fromPoint, { X: targetRect.Left, Y: targetRect.Top + targetRect.Height }, rectangles)) {
      return true;
    }

    if (this.IsPointVisible(fromPoint, { X: targetRect.Left + targetRect.Width, Y: targetRect.Top + targetRect.Height }, rectangles)) {
      return true;
    }

    return false;
  }

  // 判断矩形是否与线段相交
  private static RectangleIntersectsLine(rect: { Left: number; Top: number; Width: number; Height: number }, startPoint: { X: number; Y: number }, endPoint: { X: number; Y: number }): boolean {
    const newRect = { ...rect };
    newRect.Left -= 1;
    newRect.Top -= 1;
    newRect.Width += 2;
    newRect.Height += 2;
    const lineRect = {
      Left: Math.min(startPoint.X, endPoint.X),
      Top: Math.min(startPoint.Y, endPoint.Y),
      Width: Math.abs(startPoint.X - endPoint.X),
      Height: Math.abs(startPoint.Y - endPoint.Y)
    };
    return this.isRectIntersect(newRect, lineRect);
  }

  // 获取对角点
  private static GetOppositeCorners(orientation: ConnectorOrientation, rect: { Left: number; Top: number; Width: number; Height: number }, n1: { X: number; Y: number }, n2: { X: number; Y: number }): void {
    switch (orientation) {
      case ConnectorOrientation.Left:
        n1 = { X: rect.Left + rect.Width, Y: rect.Top };
        n2 = { X: rect.Left + rect.Width, Y: rect.Top + rect.Height };
        break;
      case ConnectorOrientation.Top:
        n1 = { X: rect.Left, Y: rect.Top + rect.Height };
        n2 = { X: rect.Left + rect.Width, Y: rect.Top + rect.Height };
        break;
      case ConnectorOrientation.Right:
        n1 = { X: rect.Left, Y: rect.Top };
        n2 = { X: rect.Left, Y: rect.Top + rect.Height };
        break;
      case ConnectorOrientation.Bottom:
        n1 = { X: rect.Left, Y: rect.Top };
        n2 = { X: rect.Left + rect.Width, Y: rect.Top };
        break;
      default:
        throw new Error('No opposite corners found!');
    }
  }

  // 获取邻居角点
  private static GetNeighborCorners(orientation: ConnectorOrientation, rect: { Left: number; Top: number; Width: number; Height: number }, n1: { X: number; Y: number }, n2: { X: number; Y: number }): void {
    switch (orientation) {
      case ConnectorOrientation.Left:
        n1 = { X: rect.Left, Y: rect.Top };
        n2 = { X: rect.Left, Y: rect.Top + rect.Height };
        break;
      case ConnectorOrientation.Top:
        n1 = { X: rect.Left, Y: rect.Top };
        n2 = { X: rect.Left + rect.Width, Y: rect.Top };
        break;
      case ConnectorOrientation.Right:
        n1 = { X: rect.Left + rect.Width, Y: rect.Top };
        n2 = { X: rect.Left + rect.Width, Y: rect.Top + rect.Height };
        break;
      case ConnectorOrientation.Bottom:
        n1 = { X: rect.Left, Y: rect.Top + rect.Height };
        n2 = { X: rect.Left + rect.Width, Y: rect.Top + rect.Height };
        break;
      default:
        throw new Error('No neighour corners found!');
    }
  }

  // 计算两点之间的距离
  private static Distance(p1: { X: number; Y: number }, p2: { X: number; Y: number }): number {
    const dx = p1.X - p2.X;
    const dy = p1.Y - p2.Y;
    return Math.sqrt(dx * dx + dy * dy);
  }

  // 获取带有边距的矩形
  private static GetRectWithMargin(connectorThumb: ConnectorInfo, margin: number): { Left: number; Top: number; Width: number; Height: number } {
    const rect = {
      Left: connectorThumb.DesignerItemLeft,
      Top: connectorThumb.DesignerItemTop,
      Width: connectorThumb.DesignerItemSize.Width,
      Height: connectorThumb.DesignerItemSize.Height
    };
    rect.Left -= margin;
    rect.Top -= margin;
    rect.Width += 2 * margin;
    rect.Height += 2 * margin;
    return rect;
  }

  // 获取偏移点
  private static GetOffsetPoint(connector: ConnectorInfo, rect: { Left: number; Top: number; Width: number; Height: number }): { X: number; Y: number } {
    let offsetPoint: { X: number; Y: number };
    switch (connector.Orientation) {
      case ConnectorOrientation.Left:
        offsetPoint = { X: rect.Left, Y: connector.Position.Y };
        break;
      case ConnectorOrientation.Top:
        offsetPoint = { X: connector.Position.X, Y: rect.Top };
        break;
      case ConnectorOrientation.Right:
        offsetPoint = { X: rect.Left + rect.Width, Y: connector.Position.Y };
        break;
      case ConnectorOrientation.Bottom:
        offsetPoint = { X: connector.Position.X, Y: rect.Top + rect.Height };
        break;
      default:
        offsetPoint = { X: 0, Y: 0 };
        break;
    }
    return offsetPoint;
  }

  // 检查路径终点
  private static CheckPathEnd(source: ConnectorInfo, sink: ConnectorInfo, showLastLine: boolean, linePoints: { X: number; Y: number }[]): void {
    if (showLastLine) {
      let startPoint: { X: number; Y: number };
      let endPoint: { X: number; Y: number };
      const marginPath = 15;
      switch (source.Orientation) {
        case ConnectorOrientation.Left:
          startPoint = { X: source.Position.X - marginPath, Y: source.Position.Y };
          break;
        case ConnectorOrientation.Top:
          startPoint = { X: source.Position.X, Y: source.Position.Y - marginPath };
          break;
        case ConnectorOrientation.Right:
          startPoint = { X: source.Position.X + marginPath, Y: source.Position.Y };
          break;
        case ConnectorOrientation.Bottom:
          startPoint = { X: source.Position.X, Y: source.Position.Y + marginPath };
          break;
        default:
          startPoint = { X: 0, Y: 0 };
          break;
      }

      switch (sink.Orientation) {
        case ConnectorOrientation.Left:
          endPoint = { X: sink.Position.X - marginPath, Y: sink.Position.Y };
          break;
        case ConnectorOrientation.Top:
          endPoint = { X: sink.Position.X, Y: sink.Position.Y - marginPath };
          break;
        case ConnectorOrientation.Right:
          endPoint = { X: sink.Position.X + marginPath, Y: sink.Position.Y };
          break;
        case ConnectorOrientation.Bottom:
          endPoint = { X: sink.Position.X, Y: sink.Position.Y + marginPath };
          break;
        default:
          endPoint = { X: 0, Y: 0 };
          break;
      }
      linePoints.unshift(startPoint);
      linePoints.push(endPoint);
    } else {
      linePoints.unshift(source.Position);
      linePoints.push(sink.Position);
    }
  }

  // 获取相反方向
  private static GetOpositeOrientation(connectorOrientation: ConnectorOrientation): ConnectorOrientation {
    switch (connectorOrientation) {
      case ConnectorOrientation.Left:
        return ConnectorOrientation.Right;
      case ConnectorOrientation.Top:
        return ConnectorOrientation.Bottom;
      case ConnectorOrientation.Right:
        return ConnectorOrientation.Left;
      case ConnectorOrientation.Bottom:
        return ConnectorOrientation.Top;
      default:
        return ConnectorOrientation.Top;
    }
  }

  // 判断点是否在矩形内
  private static isPointInRect(point: { X: number; Y: number }, rect: { Left: number; Top: number; Width: number; Height: number }): boolean {
    return point.X >= rect.Left && point.X <= rect.Left + rect.Width && point.Y >= rect.Top && point.Y <= rect.Top + rect.Height;
  }

  // 判断两个点是否相同
  private static isSamePoint(p1: { X: number; Y: number }, p2: { X: number; Y: number }): boolean {
    return p1.X === p2.X && p1.Y === p2.Y;
  }

  // 判断两个矩形是否相交
  private static isRectIntersect(rect1: { Left: number; Top: number; Width: number; Height: number }, rect2: { Left: number; Top: number; Width: number; Height: number }): boolean {
    return !(rect1.Left + rect1.Width < rect2.Left || rect2.Left + rect2.Width < rect1.Left || rect1.Top + rect1.Height < rect2.Top || rect2.Top + rect2.Height < rect1.Top);
  }
}

export { PathFinder, ConnectorOrientation, ConnectorInfo };