import { ref, onBeforeUpdate, watchEffect } from 'vue';

// 定义附加属性的存储对象
const relativePositions = new WeakMap<HTMLElement, { x: number; y: number }>();

// 获取相对位置
export function getRelativePosition(element: HTMLElement): { x: number; y: number } {
  if (!element) {
    throw new Error('element is null');
  }
  return relativePositions.get(element) || { x: 0, y: 0 };
}

// 设置相对位置
export function setRelativePosition(element: HTMLElement, value: { x: number; y: number }): void {
  if (!element) {
    throw new Error('element is null');
  }
  relativePositions.set(element, value);
  // 触发重新布局
  const parent = element.parentElement;
  if (parent) {
    const parentComponent = (parent as any).__vueParentComponent;
    if (parentComponent) {
      parentComponent.forceUpdate();
    }
  }
}

// 自定义相对位置面板组件
export default {
  name: 'RelativePositionPanel',
  setup() {
    const containerRef = ref<HTMLElement | null>(null);

    // 测量子元素大小
    const measureChildren = () => {
      if (containerRef.value) {
        const children = containerRef.value.children;
        for (let i = 0; i < children.length; i++) {
          const child = children[i] as HTMLElement;
          // 这里模拟 Measure 操作，实际中可能需要更复杂的逻辑
          child.style.width = 'auto';
          child.style.height = 'auto';
        }
      }
    };

    // 布局子元素
    const arrangeChildren = () => {
      if (containerRef.value) {
        const containerWidth = containerRef.value.offsetWidth;
        const containerHeight = containerRef.value.offsetHeight;
        const children = containerRef.value.children;
        for (let i = 0; i < children.length; i++) {
          const child = children[i] as HTMLElement;
          const { x, y } = getRelativePosition(child);
          const childWidth = child.offsetWidth;
          const childHeight = child.offsetHeight;
          const left = (containerWidth - childWidth) * x;
          const top = (containerHeight - childHeight) * y;
          child.style.position = 'absolute';
          child.style.left = isNaN(left) ? '0px' : `${left}px`;
          child.style.top = isNaN(top) ? '0px' : `${top}px`;
        }
      }
    };

    // 在更新前测量子元素
    onBeforeUpdate(() => {
      measureChildren();
    });

    // 监听容器大小变化并重新布局
    watchEffect(() => {
      arrangeChildren();
    });

    return {
      containerRef,
    };
  },
  template: `
    <div ref="containerRef">
      <slot></slot>
    </div>
  `,
};