import { ref, onMounted } from 'vue';

// 假设这些类型已经在其他文件中定义
interface DesignerItem {
    IsSelected: boolean;
    ActualHeight: number;
    ActualWidth: number;
    Height: number;
    Width: number;
    MinHeight: number;
    MinWidth: number;
}

interface DesignerCanvas {
    SelectedItems: DesignerItem[];
}

interface ISelectable {
    // 这里可以根据实际情况补充 ISelectable 接口的属性和方法
}

// 模拟 Thumb 组件的功能
class ResizeThumb {
    private verticalAlignment: string;
    private horizontalAlignment: string;
    private dragDeltaHandler: (e: { VerticalChange: number; HorizontalChange: number }) => void;

    constructor(verticalAlignment: string, horizontalAlignment: string, dragDeltaHandler: (e: { VerticalChange: number; HorizontalChange: number }) => void) {
        this.verticalAlignment = verticalAlignment;
        this.horizontalAlignment = horizontalAlignment;
        this.dragDeltaHandler = dragDeltaHandler;
        // 模拟订阅 DragDelta 事件
        onMounted(() => {
            // 这里可以添加实际的拖拽事件监听逻辑
            // 例如：document.addEventListener('mousemove', this.handleDragDelta);
        });
    }

    private handleDragDelta = (e: { VerticalChange: number; HorizontalChange: number }) => {
        this.dragDeltaHandler(e);
    }
}

// 实现 ResizeThumb 的主要逻辑
const useResizeThumb = (verticalAlignment: string, horizontalAlignment: string, designerCanvas: DesignerCanvas) => {
    const resizeThumb = new ResizeThumb(verticalAlignment, horizontalAlignment, (e) => {
        const selectedDesignerItems = designerCanvas.SelectedItems.filter(item => item.IsSelected);
        let minLeft: number, minTop: number, minDeltaHorizontal: number, minDeltaVertical: number;
        CalculateDragLimits(selectedDesignerItems, out minLeft, out minTop, out minDeltaHorizontal, out minDeltaVertical);

        selectedDesignerItems.forEach(item => {
            if (item) {
                let dragDeltaVertical: number, dragDeltaHorizontal: number;

                switch (verticalAlignment) {
                    case 'Bottom':
                        dragDeltaVertical = Math.min(-e.VerticalChange, minDeltaVertical);
                        item.Height = item.ActualHeight - dragDeltaVertical;
                        break;
                    case 'Top':
                        const top = item.Top;
                        dragDeltaVertical = Math.min(Math.max(-minTop, e.VerticalChange), minDeltaVertical);
                        item.Top = top + dragDeltaVertical;
                        item.Height = item.ActualHeight - dragDeltaVertical;
                        break;
                    default:
                        break;
                }

                switch (horizontalAlignment) {
                    case 'Left':
                        const left = item.Left;
                        dragDeltaHorizontal = Math.min(Math.max(-minLeft, e.HorizontalChange), minDeltaHorizontal);
                        item.Left = left + dragDeltaHorizontal;
                        item.Width = item.ActualWidth - dragDeltaHorizontal;
                        break;
                    case 'Right':
                        dragDeltaHorizontal = Math.min(-e.HorizontalChange, minDeltaHorizontal);
                        item.Width = item.ActualWidth - dragDeltaHorizontal;
                        break;
                    default:
                        break;
                }
            }
        });
    });

    return resizeThumb;
};

// 计算拖拽限制的方法
const CalculateDragLimits = (selectedDesignerItems: ISelectable[], out minLeft: number, out minTop: number, out minDeltaHorizontal: number, out minDeltaVertical: number) => {
    minLeft = Number.MAX_VALUE;
    minTop = Number.MAX_VALUE;
    minDeltaHorizontal = Number.MAX_VALUE;
    minDeltaVertical = Number.MAX_VALUE;

    selectedDesignerItems.forEach(item => {
        const designerItem = item as DesignerItem;
        const left = designerItem.Left;
        const top = designerItem.Top;

        minLeft = isNaN(left) ? 0 : Math.min(left, minLeft);
        minTop = isNaN(top) ? 0 : Math.min(top, minTop);

        minDeltaVertical = Math.min(minDeltaVertical, designerItem.ActualHeight - designerItem.MinHeight);
        minDeltaHorizontal = Math.min(minDeltaHorizontal, designerItem.ActualWidth - designerItem.MinWidth);
    });
};

export { useResizeThumb };