import { ref, onMounted, onUnmounted, Ref } from 'vue';

// 假设 ISelectable 接口定义如下
interface ISelectable {
  IsSelected: boolean;
}

// 假设 DesignerCanvas 类定义如下
class DesignerCanvas {
  Children: HTMLElement[] = [];
  SelectedItems: ISelectable[] = [];
}

export default {
  props: {
    designerCanvas: {
      type: Object as () => DesignerCanvas,
      required: true
    },
    dragStartPoint: {
      type: Object as () => { x: number; y: number } | null,
      default: null
    }
  },
  setup(props) {
    const startPoint = ref<{ x: number; y: number } | null>(props.dragStartPoint);
    const endPoint = ref<{ x: number; y: number } | null>(null);
    const rubberbandPen = {
      color: 'LightSlateGray',
      width: 1,
      dashStyle: [2, 2]
    };
    const designerCanvas = props.designerCanvas;
    const containerRef = ref<HTMLElement | null>(null);

    const onMouseMove = (e: MouseEvent) => {
      if (e.buttons === 1) {
        if (!containerRef.value?.hasPointerCapture(e.pointerId)) {
          containerRef.value?.setPointerCapture(e.pointerId);
        }
        endPoint.value = { x: e.offsetX, y: e.offsetY };
        updateSelection();
      } else {
        if (containerRef.value?.hasPointerCapture(e.pointerId)) {
          containerRef.value?.releasePointerCapture(e.pointerId);
        }
      }
      e.preventDefault();
    };

    const onMouseUp = (e: MouseEvent) => {
      if (containerRef.value?.hasPointerCapture(e.pointerId)) {
        containerRef.value?.releasePointerCapture(e.pointerId);
      }
      // 移除这个“装饰器”逻辑，在 Vue 中可以通过 v-if 等方式移除元素
      endPoint.value = null;
      e.preventDefault();
    };

    const updateSelection = () => {
      designerCanvas.SelectedItems.forEach((item) => {
        item.IsSelected = false;
      });
      designerCanvas.SelectedItems = [];

      if (startPoint.value && endPoint.value) {
        const rubberBand = {
          left: Math.min(startPoint.value.x, endPoint.value.x),
          top: Math.min(startPoint.value.y, endPoint.value.y),
          width: Math.abs(startPoint.value.x - endPoint.value.x),
          height: Math.abs(startPoint.value.y - endPoint.value.y)
        };

        designerCanvas.Children.forEach((item) => {
          const rect = item.getBoundingClientRect();
          const itemBounds = {
            left: rect.left,
            top: rect.top,
            width: rect.width,
            height: rect.height
          };

          if (
            rubberBand.left <= itemBounds.left &&
            rubberBand.top <= itemBounds.top &&
            rubberBand.left + rubberBand.width >= itemBounds.left + itemBounds.width &&
            rubberBand.top + rubberBand.height >= itemBounds.top + itemBounds.height &&
            'IsSelected' in item
          ) {
            (item as unknown as ISelectable).IsSelected = true;
            designerCanvas.SelectedItems.push(item as unknown as ISelectable);
          }
        });
      }
    };

    onMounted(() => {
      if (containerRef.value) {
        containerRef.value.addEventListener('mousemove', onMouseMove);
        containerRef.value.addEventListener('mouseup', onMouseUp);
      }
    });

    onUnmounted(() => {
      if (containerRef.value) {
        containerRef.value.removeEventListener('mousemove', onMouseMove);
        containerRef.value.removeEventListener('mouseup', onMouseUp);
      }
    });

    return {
      startPoint,
      endPoint,
      rubberbandPen,
      containerRef
    };
  },
  template: `
    <div ref="containerRef" style="position: relative; width: 100%; height: 100%;">
      <div
        v-if="startPoint && endPoint"
        :style="{
          position: 'absolute',
          left: Math.min(startPoint.x, endPoint.x) + 'px',
          top: Math.min(startPoint.y, endPoint.y) + 'px',
          width: Math.abs(startPoint.x - endPoint.x) + 'px',
          height: Math.abs(startPoint.y - endPoint.y) + 'px',
          border: \`\${rubberbandPen.width}px dashed \${rubberbandPen.color}\`
        }"
      ></div>
    </div>
  `
};