﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections;
using System.Xml.Linq;
using Platform.Presentation.FocusManagers;
using Platform.Common.BasicFunctions;

namespace TellerSystem.Controls.Ext.Controls
{
    public partial class MultiSelectComboBox : UserControl
    {
        public MultiSelectComboBox()
        {
            InitializeComponent();
            this.PreviewKeyDown += new System.Windows.Input.KeyEventHandler(MultiSelectComboBox_PreviewKeyDown);
            this.Loaded += new RoutedEventHandler(MultiSelectComboBox_Loaded);
        }

        #region 字段
        StringBuilder _sb = new StringBuilder();
        StringBuilder _sb1 = new StringBuilder();
        private Platform.Presentation.FocusManagers.FocusManager _focus;
        #endregion

        #region 属性
        public string SelectedValueList
        {
            get { return (string)GetValue(SelectedValueListProperty); }
            private set { SetValue(SelectedValueListProperty, value); }
        }

        public static readonly DependencyProperty SelectedValueListProperty =
            DependencyProperty.Register("SelectedValueList", typeof(string), typeof(MultiSelectComboBox), new UIPropertyMetadata(string.Empty));

        public string SelectedValue
        {
            get
            {
                return _sb1.Length == 0 ? string.Empty : _sb1.ToString(1, _sb1.Length - 1);
            }
            set
            {
                ClearSelectedValue();
                var select = (value ?? string.Empty).Split(',');
                if (select.Length == 0) return;
                var nodeList = tree.ItemsSource as IEnumerable<MultiSelectComboBoxNode>;
                if (nodeList == null) return;
                foreach (var item in select)
                {
                    nodeList.ToList().ForEach(x =>
                    {
                        SetSelectedValue(x, item);

                    });
                }
                Refresh();
            }
        }
        #endregion

        #region 方法

        /// <summary>
        /// 为该控件指定数据源
        /// </summary>
        /// <param name="data">可迭代的数据源</param>
        public void SetItems(IEnumerable data)
        {
            //组装数据源
            var source = new List<MultiSelectComboBoxNode>();
            var items = data.Cast<KeyValuePair<string, string>>();
            foreach (var item in items)
            {
                source.Add(new MultiSelectComboBoxNode { Title = item.Key, Value = item.Value, IsSelected = false });
            }
            tree.ItemsSource = source;
        }

        /// <summary>
        /// 为该控件指定形如XML的数据源
        /// </summary>
        /// <param name="xmlData">可转化为XML的数据</param>
        /// <param name="nodeName">一级结点标签名</param>
        /// <param name="titleName">结点标签对应的Title属性名称</param>
        /// <param name="valueName">结点标签对应的Value属性名称</param>
        public void SetItems(string xmlData, string nodeName = "org", string titleName = "name", string valueName = "id")
        {
            XDocument doc = XDocument.Parse(xmlData);
            XElement root = doc.Root;
            var elements = root.Elements(nodeName);
            var source = new List<MultiSelectComboBoxNode>();
            foreach (var element in elements)
            {
                source.Add(CreateTopLevelNode(element, nodeName, titleName, valueName));
            }
            tree.ItemsSource = source;
        }

        /// <summary>
        /// 设置选中结点的值
        /// </summary>
        /// <param name="value">选中结点的值</param>
        public void Setvalue(string value)
        {
            this.SelectedValue = value;
        }

        /// <summary>
        /// 根据给定的value值对相应的叶子结点进行勾选
        /// </summary>
        /// <param name="node">一级结点</param>
        /// <param name="value">给定的Value值</param>
        private void SetSelectedValue(MultiSelectComboBoxNode node, string value)
        {

            if (node.Children.Count > 0)
            {
                foreach (var item in node.Children)
                {
                    SetSelectedValue(item, value);
                }
            }
            else//叶子节点
            {
                if (node.Value == value)
                {
                    node.IsSelected = true;
                    SetParentState(node);
                }
            }

        }

        void MultiSelectComboBox_Loaded(object sender, RoutedEventArgs e)
        {
            if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this)) return;
            _focus = Platform.Presentation.FocusManagers.FocusManager.GetManagerInstance((DependencyObject)this) as Platform.Presentation.FocusManagers.FocusManager;
            //if (_focus == null) return;
            combobox.SetBinding(ComboBox.ToolTipProperty, new Binding("SelectedValueList") { Source = this, Mode = BindingMode.OneWay });
            var child = VisualTreeHelper.GetChild(combobox, 0);
            var children=  child.FindVisualTreeChildren((x) => { return (x is ContentPresenter); });
            if (children != null)
            {
                var cp = children[0] as ContentPresenter;
                if (cp != null)
                {
                    cp.ContentTemplate = null;
                    cp.SetBinding(ContentPresenter.ContentProperty, new Binding("SelectedValueList") { Source = this, Mode = BindingMode.OneWay });
                }
            }
           
        }

        private void checkbox_Click(object sender, RoutedEventArgs e)
        {
            var node = (sender as CheckBox).DataContext as MultiSelectComboBoxNode;
            SelectCascade(node);
            SetParentState(node);
            Refresh();
        }

        void MultiSelectComboBox_PreviewKeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {

            if (e.Key == Key.Up)
            {
                _focus.MoveFocus(FocusNavigation.Backward);
            }
            else if (e.Key == Key.Down || e.Key == Key.Tab || e.Key == Key.Enter)
            {

                _focus.MoveFocus(FocusNavigation.Forward);
            }
            else if (e.Key == Key.Left)
            {

                _focus.MoveFocus(FocusNavigation.Backward);
            }
            else if (e.Key == Key.Right)
            {

                _focus.MoveFocus(FocusNavigation.Forward);
            }
        }

        /// <summary>
        /// 用递归算法根据XML元素生成第一级节点（含子节点）
        /// </summary>
        /// <param name="ele">指定的XML元素</param>
        /// <returns></returns>
        private MultiSelectComboBoxNode CreateTopLevelNode(XElement ele, string nodeName, string titleName, string valueName)
        {

            if (ele.Elements(nodeName).Count() > 0)
            {
                MultiSelectComboBoxNode node = new MultiSelectComboBoxNode() { Title = ele.Attribute(titleName).Value, Value = ele.Attribute(valueName).Value, IsSelected = false };
                foreach (var item in ele.Elements(nodeName))
                {
                    var childNode = CreateTopLevelNode(item, nodeName, titleName, valueName);
                    childNode.Parent = node;
                    node.Children.Add(childNode);
                }
                return node;
            }
            else
            {
                MultiSelectComboBoxNode node = new MultiSelectComboBoxNode() { Title = ele.Attribute(titleName).Value, Value = ele.Attribute(valueName).Value, IsSelected = false };
                return node;
            }
        }

        /// <summary>
        /// 根据指点结点寻找所有选中的叶子节点
        /// </summary>
        /// <param name="node"></param>
        private void FindSelectedLeaf(MultiSelectComboBoxNode node)
        {
            if (node.Children.Count > 0)
            {
                foreach (var item in node.Children)
                {
                    FindSelectedLeaf(item);
                }
            }
            else//叶子节点
            {
                if (node.IsSelected == true)
                {
                    _sb.Append("," + node.Title);
                    _sb1.Append("," + node.Value);
                }
            }
        }

        /// <summary>
        /// 撤消或选中指点结点本身及其所有子结点
        /// </summary>
        /// <param name="node">指定的结点</param>
        private void SelectCascade(MultiSelectComboBoxNode node)
        {
            if (node.Children.Count > 0)
            {
                node.Children.ForEach(x =>
                {
                    x.IsSelected = node.IsSelected;
                    SelectCascade(x);
                });
            }
        }

        private void SetParentState(MultiSelectComboBoxNode node)
        {
            if (node.Parent != null)
            {
                MultiSelectComboBoxNode parent = node.Parent;
                if (parent.Children.All(x => { return x.IsSelected == true; }))
                {
                    parent.IsSelected = true;
                }

                else if (parent.Children.All(x => { return x.IsSelected == false; }))
                {
                    parent.IsSelected = false;
                }

                else
                {
                    parent.IsSelected = null;
                }
                SetParentState(parent);
            }
        }

        private void Refresh()
        {
            _sb.Clear();
            _sb1.Clear();
            SelectedValueList = string.Empty;
            var nodeList = tree.ItemsSource as IEnumerable<MultiSelectComboBoxNode>;
            if (nodeList != null)
            {
                nodeList.ToList().ForEach(x =>
                {

                    FindSelectedLeaf(x);
                });
                SelectedValueList = _sb.Length == 0 ? string.Empty : _sb.ToString(1, _sb.Length - 1);
            }
        }

        /// <summary>
        /// 通过SelectedValue属性设置选中项时，要将数据源中的所有选项的IsSelected属性置为false
        /// </summary>
        private void ClearSelectedValue()
        {
            var nodeList = tree.ItemsSource as IEnumerable<MultiSelectComboBoxNode>;
            if (nodeList != null)
            {
                foreach (var node in nodeList)
                {
                    node.IsSelected = false;
                    SelectCascade(node);
                }
                Refresh();
            }
        }
        #endregion

    }

    public class MultiSelectComboBoxNode : INotifyPropertyChanged
    {
        public MultiSelectComboBoxNode()
        {
            Children = new List<MultiSelectComboBoxNode>();

        }
        #region 属性

        private string title;

        public string Title
        {
            get { return title; }
            set
            {
                title = value;
                NotifyPropertyChanged("Title");
            }
        }

        private string value;

        public string Value
        {
            get { return value; }
            set
            {
                this.value = value;
                NotifyPropertyChanged("Value");
            }
        }



        private bool? _isSelected;

        public bool? IsSelected
        {
            get { return _isSelected; }
            set
            {
                _isSelected = value;
                NotifyPropertyChanged("IsSelected");
            }
        }

        private MultiSelectComboBoxNode _parent;

        public MultiSelectComboBoxNode Parent
        {
            get { return _parent; }
            set { _parent = value; }
        }

        public List<MultiSelectComboBoxNode> Children { get; private set; }
        #endregion

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

    }
}
