import { ref, onMounted, onUnmounted } from 'vue';
import { FocusManager } from '@/composables/FocusManager'; // 假设 FocusManager 有对应的 TypeScript 实现
import { TellerSystemServiceProxyExt } from '@/services/TellerSystemServiceProxyExt'; // 假设服务代理有对应的 TypeScript 实现
import { PlatformControlsHelper } from '@/helpers/PlatformControlsHelper'; // 假设辅助工具类有对应的 TypeScript 实现

// 模拟 DependencyProperty
class DependencyProperty {
  static registerAttached(name: string, type: any, ownerType: any, metadata: any) {
    return { name, type, ownerType, metadata };
  }
}

// 模拟 UIPropertyMetadata
class UIPropertyMetadata {
  constructor(defaultValue: any, propertyChangedCallback?: (d: any, e: any) => void) {
    this.defaultValue = defaultValue;
    this.propertyChangedCallback = propertyChangedCallback;
  }
  defaultValue: any;
  propertyChangedCallback?: (d: any, e: any) => void;
}

// 模拟 PropertyChangedCallback
type PropertyChangedCallback = (d: any, e: any) => void;

// 模拟 RoutedEventArgs
class RoutedEventArgs {
  constructor() {}
  handled: boolean = false;
}

// 模拟 MouseButtonEventArgs
class MouseButtonEventArgs extends RoutedEventArgs {
  constructor() {
    super();
  }
}

// 模拟 KeyEventArgs
class KeyEventArgs extends RoutedEventArgs {
  constructor(public key: any) {
    super();
  }
}

// 模拟 Button 类型
class Button {
  previewMouseLeftButtonUp: ((sender: any, e: MouseButtonEventArgs) => void)[] = [];
  previewKeyDown: ((sender: any, e: KeyEventArgs) => void)[] = [];
  addPreviewMouseLeftButtonUpHandler(handler: (sender: any, e: MouseButtonEventArgs) => void) {
    this.previewMouseLeftButtonUp.push(handler);
  }
  removePreviewMouseLeftButtonUpHandler(handler: (sender: any, e: MouseButtonEventArgs) => void) {
    const index = this.previewMouseLeftButtonUp.indexOf(handler);
    if (index!== -1) {
      this.previewMouseLeftButtonUp.splice(index, 1);
    }
  }
  addPreviewKeyDownHandler(handler: (sender: any, e: KeyEventArgs) => void) {
    this.previewKeyDown.push(handler);
  }
  removePreviewKeyDownHandler(handler: (sender: any, e: KeyEventArgs) => void) {
    const index = this.previewKeyDown.indexOf(handler);
    if (index!== -1) {
      this.previewKeyDown.splice(index, 1);
    }
  }
  removeTip() {}
  showTip(message: string) {}
  setValue(property: any, value: any) {}
  getValue(property: any) {
    return null;
  }
}

// 模拟 TextBox 类型
class TextBox {
  text: string = '';
}

// 模拟 DateTime
class DateTime {
  static now() {
    return new Date();
  }
  static fromValue(value: Date) {
    const dt = new DateTime();
    dt.value = value;
    return dt;
  }
  value: Date;
  get totalMilliseconds() {
    return this.value.getTime();
  }
}

// 模拟 ServiceSettings
class ServiceSettings {
  static getConfig(key: string, defaultValue: number) {
    return defaultValue;
  }
}

// 模拟 FocusManager 中的相关方法
class CustomFocusManager {
  static getManagerInstance(control: any) {
    return {
      _focusList: [control], // 简单模拟有该控件在焦点列表中
    };
  }
}

// 模拟 TellerSystem.Communication.Message
class TellerSystemMessage {
  static FmtCEName(text: string) {
    return text; // 简单返回原文本，可根据实际逻辑修改
  }
}

/**
 * 申明控件附加属性
 */
export class ControlsAttributes {
  // 文本框水印文本
  static WaterMarkTextProperty = DependencyProperty.registerAttached(
    'WaterMarkText',
    String,
    ControlsAttributes,
    new UIPropertyMetadata('')
  );

  static getWaterMarkText(obj: any) {
    return obj.getValue(this.WaterMarkTextProperty);
  }

  static setWaterMarkText(obj: any, value: string) {
    obj.setValue(this.WaterMarkTextProperty, value);
  }

  // 姓名标准点转换
  static IsFmtCENameProperty = DependencyProperty.registerAttached(
    'IsFmtCEName',
    Boolean,
    ControlsAttributes,
    new UIPropertyMetadata(false, (d, e) => {
      const textBox = d as TextBox;
      if (textBox) {
        if (e.newValue) {
          CustomFocusManager.getManagerInstance(textBox)?.addPreviewLostFocusHandler(
            ControlsAttributes.textChangedToFmtCEName
          );
        } else {
          CustomFocusManager.getManagerInstance(textBox)?.removePreviewLostFocusHandler(
            ControlsAttributes.textChangedToFmtCEName
          );
        }
      }
    })
  );

  static getIsFmtCEName(obj: any) {
    return obj.getValue(this.IsFmtCENameProperty);
  }

  static setIsFmtCEName(obj: any, value: boolean) {
    obj.setValue(this.IsFmtCENameProperty, value);
  }

  static textChangedToFmtCEName(sender: any, e: RoutedEventArgs) {
    const textBox = sender as TextBox;
    if (textBox) {
      textBox.text = TellerSystemMessage.FmtCEName(textBox.text);
    }
  }

  // 按钮点击间隔控制
  static ButtonClickDelayProperty = DependencyProperty.registerAttached(
    'ButtonClickDelay',
    Number,
    ControlsAttributes,
    new UIPropertyMetadata(0, (d, e) => {
      const btn = d as Button;
      if (!btn) return;
      btn.removePreviewMouseLeftButtonUpHandler(ControlsAttributes.btn_PreviewMouseLeftButtonUp);
      btn.removePreviewKeyDownHandler(ControlsAttributes.btn_PreviewKeyDown);
      if (e.newValue!== 0) {
        btn.addPreviewMouseLeftButtonUpHandler(ControlsAttributes.btn_PreviewMouseLeftButtonUp);
        btn.addPreviewKeyDownHandler(ControlsAttributes.btn_PreviewKeyDown);
      }
    })
  );

  static ClickTimestampProperty = DependencyProperty.registerAttached(
    'ClickTimestamp',
    DateTime,
    ControlsAttributes,
    new UIPropertyMetadata(null)
  );

  static getButtonClickDelay(obj: any) {
    return obj.getValue(this.ButtonClickDelayProperty);
  }

  static setButtonClickDelay(obj: any, value: number) {
    obj.setValue(this.ButtonClickDelayProperty, value);
  }

  static btn_PreviewMouseLeftButtonUp(sender: any, e: MouseButtonEventArgs) {
    ControlsAttributes.checkButtonClick(sender as Button, e);
  }

  static btn_PreviewKeyDown(sender: any, e: KeyEventArgs) {
    if (e.key === 'Enter') {
      ControlsAttributes.checkButtonClick(sender as Button, e);
    }
  }

  static checkButtonClick(btn: Button, e: RoutedEventArgs) {
    if (!btn) return;
    const fm = CustomFocusManager.getManagerInstance(btn);
    if (!fm ||!fm._focusList ||!fm._focusList.includes(btn)) return;
    const delay = ControlsAttributes.getButtonClickDelay(btn);
    if (delay < 0) {
      delay = ServiceSettings.getConfig('ButtonClickDelay', 3000);
    }
    const timestamp = btn.getValue(ControlsAttributes.ClickTimestampProperty) as DateTime;
    const timeSpan = DateTime.now().totalMilliseconds - (timestamp?.totalMilliseconds || 0);
    if (!timestamp || timeSpan > delay) {
      btn.removeTip();
      btn.setValue(ControlsAttributes.ClickTimestampProperty, DateTime.now());
      return;
    }
    btn.showTip('请勿频繁点击，稍后再试...');
    e.handled = true;
  }
}