﻿/*  
 * CustComboBox implements combobox with popup DataGrid control.
 * Bahrudin Hrnjica, bhrnjica@hotmail.com
 * First Release Oct, 2009
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Windows.Markup;
using System.Collections.ObjectModel;
using System.Windows.Controls.Primitives;
using Platform.Controls.ControlsHelper;
using System.Timers;
using System.Data;

namespace TellerSystem.Controls.Ext.Controls
{/// <summary>
    /// CustComboBox implements combobox with popup DataGrid control.
    /// Usage:
    ///     <local:CustComboBox Margin="121,0,251,159" VerticalAlignment="Bottom" Height="29" 
    ///	                        ItemsSource="{Binding Collection}" DisplayMemberPath="Property1" IsEditable="True" >
    ///	                        
    ///		                <toolKit:DataGridTextColumn Header="HeaderTitle1" Binding="{Binding Property1, Mode=Default}" />
    ///		                <toolKit:DataGridTextColumn Header="HeaderTitle2" Binding="{Binding Property2, Mode=Default}"/>
    ///		                <toolKit:DataGridTextColumn Header="HeaderTitle3" Binding="{Binding Property3, Mode=Default}"/>
    ///		                
    ///	    </local:CustComboBox>
    ///
    /// </summary>
    [DefaultProperty("Columns")]
    [ContentProperty("Columns")]
    public class ComboBoxDataGrid : ComboBox
    {
        const string partPopupDataGrid = "PART_PopupDataGrid";
        //Columns of DataGrid
        private ObservableCollection<DataGridTextColumn> columns;
        //Attached DataGrid control
        private DataGrid popupDataGrid;

        static ComboBoxDataGrid()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(ComboBoxDataGrid), new FrameworkPropertyMetadata(typeof(ComboBoxDataGrid)));
        }

        //The property is default and Content property for CustComboBox
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public ObservableCollection<DataGridTextColumn> Columns
        {
            get
            {
                if (this.columns == null)
                {
                    this.columns = new ObservableCollection<DataGridTextColumn>();
                }
                return this.columns;
            }
        }

        //Apply theme and attach columns to DataGrid popupo control
        public override void OnApplyTemplate()
        {
            if (popupDataGrid == null)
            {
                popupDataGrid = this.Template.FindName(partPopupDataGrid, this) as DataGrid;
                if (popupDataGrid != null && columns != null)
                {
                    //Add columns to DataGrid columns
                    for (int i = 0; i < columns.Count; i++)
                        popupDataGrid.Columns.Add(columns[i]);

                    //Add event handler for DataGrid popup
                    popupDataGrid.MouseDown += new MouseButtonEventHandler(popupDataGrid_MouseDown);
                    popupDataGrid.SelectionChanged += new SelectionChangedEventHandler(popupDataGrid_SelectionChanged);

                }
            }
            //Call base class method
            base.OnApplyTemplate();
        }

        //Synchronize selection between Combo and DataGrid popup
        void popupDataGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            //When open in Blend prevent raising exception 
            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                DataGrid dg = sender as DataGrid;
                if (dg != null)
                {
                    this.SelectedItem = dg.SelectedItem;
                    this.SelectedValue = dg.SelectedValue;
                    this.SelectedIndex = dg.SelectedIndex;
                    this.SelectedValuePath = dg.SelectedValuePath;

                }
            }
        }
        //Event for DataGrid popup MouseDown
        void popupDataGrid_MouseDown(object sender, MouseButtonEventArgs e)
        {
            DataGrid dg = sender as DataGrid;
            if (dg != null)
            {
                DependencyObject dep = (DependencyObject)e.OriginalSource;

                // iteratively traverse the visual tree and stop when dep is one of ..
                while ((dep != null) &&
                        !(dep is DataGridCell) &&
                        !(dep is DataGridColumnHeader))
                {
                    dep = VisualTreeHelper.GetParent(dep);
                }

                if (dep == null)
                    return;

                if (dep is DataGridColumnHeader)
                {
                    // do something
                }
                //When user clicks to DataGrid cell, popup have to be closed
                if (dep is DataGridCell)
                {
                    this.IsDropDownOpen = false;
                }
            }
        }

        //When selection changed in combobox (pressing  arrow key down or up) must be synchronized with opened DataGrid popup
        protected override void OnSelectionChanged(SelectionChangedEventArgs e)
        {
            base.OnSelectionChanged(e);
            if (popupDataGrid == null)
                return;

            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                if (IsDropDownOpen)
                {
                    popupDataGrid.SelectedItem = this.SelectedItem;
                    if (popupDataGrid.SelectedItem != null)
                        popupDataGrid.ScrollIntoView(popupDataGrid.SelectedItem);
                }
            }
        }
        protected override void OnDropDownOpened(EventArgs e)
        {
            popupDataGrid.SelectedItem = this.SelectedItem;

            base.OnDropDownOpened(e);

            if (popupDataGrid.SelectedItem != null)
                popupDataGrid.ScrollIntoView(popupDataGrid.SelectedItem);
        }

        #region AutoComplete

        bool isKeyEvent = false;
        TextBox editTextBox;
        Timer interval = new Timer(400);
        int oldSelStart;
        int oldSelLength;
        string oldText = string.Empty;
        #endregion

        #region Public
        private bool _isAutoComplete;

        public bool IsAutoComplete
        {
            get { return _isAutoComplete; }
            set
            {
                _isAutoComplete = value;

                if (_isAutoComplete)
                {
                    this.IsEditable = true;
                    RoutedEventHandler handler = null;
                    handler = (sender, e) =>
                    {
                        this.Loaded -= handler;

                        interval.AutoReset = true;
                        interval.Elapsed += interval_Elapsed;

                        editTextBox = this.Template.FindName(ComboBoxHelper.PartEditableTextBox, this) as TextBox;
                        editTextBox.TextChanged += editTextBox_TextChanged;
                        editTextBox.PreviewKeyDown += editTextBox_PreviewKeyDown;

                        this.GotMouseCapture += ComboBoxDataGrid_GotMouseCapture;
                        this.DropDownOpened += ComboBoxDataGrid_DropDownOpened;

                        this.GotFocus += ComboBoxDataGrid_GotFocus;
                        this.SelectionChanged += ComboBoxDataGrid_SelectionChanged;
                        this.LostFocus += ComboBoxDataGrid_LostFocus;
                    };

                    this.Loaded += handler;
                }
                else
                {
                    interval.Elapsed -= interval_Elapsed;

                    if (editTextBox != null)
                    {
                        editTextBox.TextChanged -= editTextBox_TextChanged;
                        editTextBox.PreviewKeyDown -= editTextBox_PreviewKeyDown;
                    }

                    this.GotMouseCapture -= ComboBoxDataGrid_GotMouseCapture;
                    this.DropDownOpened -= ComboBoxDataGrid_DropDownOpened;

                    this.GotFocus -= ComboBoxDataGrid_GotFocus;
                    this.SelectionChanged -= ComboBoxDataGrid_SelectionChanged;
                    this.LostFocus -= ComboBoxDataGrid_LostFocus;
                }
            }
        }

        void ComboBoxDataGrid_LostFocus(object sender, RoutedEventArgs e)
        {
            isKeyEvent = false;
            this.IsDropDownOpen = false;

            if (this.SelectedIndex == -1)
            {
                editTextBox.Clear();
            }
            else
            {
            }

            interval.Close();

            try
            {
                editTextBox.CaretIndex = 0;
            }
            catch (Exception)
            {
            }
        }

        void ComboBoxDataGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            interval.Stop();
        }

        void ComboBoxDataGrid_GotFocus(object sender, RoutedEventArgs e)
        {
            editTextBox.Focus();
        }

        void ComboBoxDataGrid_DropDownOpened(object sender, EventArgs e)
        {
            editTextBox.Text = oldText;
            editTextBox.SelectionStart = oldSelStart;
            editTextBox.SelectionLength = oldSelLength;
        }

        void ComboBoxDataGrid_GotMouseCapture(object sender, MouseEventArgs e)
        {
            oldSelStart = editTextBox.SelectionStart;
            oldSelLength = editTextBox.SelectionLength;
            oldText = editTextBox.Text;
        }

        void editTextBox_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            isKeyEvent = true;
        }

        void editTextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (string.IsNullOrEmpty(editTextBox.Text))
            {
                this.IsDropDownOpen = false;
            }
            else if (isKeyEvent)
            {
                this.SelectedItem = null;
                interval.Stop();
                interval.Start();
            }
        }

        void interval_Elapsed(object sender, ElapsedEventArgs e)
        {
            interval.Stop();
            isKeyEvent = false;

            this.Dispatcher.BeginInvoke(new Action(() =>
            {
                this.IsDropDownOpen = true;

                DataView dv = this.ItemsSource as DataView;
                if (dv == null) return;

                string text = editTextBox.Text;

                if (!dv.Filter(text)) editTextBox.Clear(); //筛选出错清空筛选框

                if (this.SelectedItem != null)
                {
                    editTextBox.Text = this.SelectedItem.GetType().GetProperty(this.DisplayMemberPath).GetValue(this.SelectedItem, null).ToString();
                }
            }));
        }

        #endregion
    }
}
