import { ref, onMounted } from 'vue';

// 定义一些辅助函数所需的类型
type UIElement = HTMLElement;
type Point3D = { x: number; y: number; z: number };
type Vector3D = { x: number; y: number; z: number };

// 模拟 Storyboard 和 TimelineCollection
class TimelineCollection {
    children: any[] = [];
    add(child: any) {
        this.children.push(child);
    }
}

class Storyboard {
    children: TimelineCollection = new TimelineCollection();
    begin() {
        // 这里可以实现动画开始的逻辑
    }
}

// 模拟 DoubleAnimation
class DoubleAnimation {
    constructor(
        public to: number,
        public duration: number,
        public from?: number,
        public accelerationRatio?: number,
        public decelerationRatio?: number,
        public beginTime?: number
    ) {}
}

// 模拟 Point3DAnimation
class Point3DAnimation {
    constructor(
        public from: Point3D,
        public to: Point3D,
        public duration: number,
        public autoReverse: boolean = false,
        public beginTime?: number,
        public decelerationRatio?: number
    ) {}
}

// 模拟 ObjectAnimationUsingKeyFrames
class ObjectAnimationUsingKeyFrames {
    keyFrames: any[] = [];
    addKeyFrame(value: any, time: number) {
        this.keyFrames.push({ value, time });
    }
}

// 模拟 Border
class Border {
    child: UIElement;
    opacity: number = 1;
    visibility: 'visible' | 'hidden' = 'visible';
    horizontalAlignment: 'stretch' | 'left' | 'right' | 'center' = 'stretch';
    verticalAlignment: 'stretch' | 'top' | 'bottom' | 'center' = 'stretch';
    background: string = 'transparent';

    constructor(child: UIElement) {
        this.child = child;
    }
}

// 模拟 Viewport3D
class Viewport3D {
    camera: { position: Point3D };
    visibility: 'visible' | 'hidden' = 'visible';
    children: any[] = [];

    constructor() {
        this.camera = { position: { x: 0, y: 0, z: 1 } };
    }
}

// 模拟 ModelVisual3D
class ModelVisual3D {
    content: any;
    constructor(content: any) {
        this.content = content;
    }
}

// 模拟 MeshGeometry3D
class MeshGeometry3D {
    triangleIndices: number[] = [];
    textureCoordinates: { x: number; y: number }[] = [];
    positions: Point3D[] = [];
}

// 模拟 DiffuseMaterial
class DiffuseMaterial {
    brush: any;
    constructor(brush: any) {
        this.brush = brush;
    }
}

// 模拟 VisualBrush
class VisualBrush {
    stretch: 'uniform' | 'fill' | 'none' | 'uniformToFill' = 'uniform';
    relativeTransform: any;
    constructor(child: UIElement) {}
}

// 模拟 AxisAngleRotation3D
class AxisAngleRotation3D {
    axis: Vector3D;
    angle: number;
    constructor(axis: Vector3D, angle: number) {
        this.axis = axis;
        this.angle = angle;
    }
}

// 模拟 RotateTransform3D
class RotateTransform3D {
    rotation: AxisAngleRotation3D;
    constructor(rotation: AxisAngleRotation3D) {
        this.rotation = rotation;
    }
}

// 模拟 GeometryModel3D
class GeometryModel3D {
    geometry: MeshGeometry3D;
    material: DiffuseMaterial;
    backMaterial: DiffuseMaterial;
    transform: RotateTransform3D;
    constructor(
        geometry: MeshGeometry3D,
        material: DiffuseMaterial,
        backMaterial: DiffuseMaterial,
        transform: RotateTransform3D
    ) {
        this.geometry = geometry;
        this.material = material;
        this.backMaterial = backMaterial;
        this.transform = transform;
    }
}

// 模拟 DirectionalLight 和 AmbientLight
class DirectionalLight {
    color: string;
    direction: Vector3D;
    constructor(color: string, direction: Vector3D) {
        this.color = color;
        this.direction = direction;
    }
}

class AmbientLight {
    color: string;
    constructor(color: string) {
        this.color = color;
    }
}

// 模拟 Model3DGroup
class Model3DGroup {
    children: any[] = [];
    add(child: any) {
        this.children.push(child);
    }
}

// 模拟 Rotate3DContainer
export default function useRotate3DContainer() {
    const _back2FrontStory = ref<Storyboard | null>(null);
    const _backWarpper = ref<Border | null>(null);
    const _front2BackStory = ref<Storyboard | null>(null);
    const _frontWarpper = ref<Border | null>(null);
    const CurrentIndex = ref(0);

    const Init = () => {
        const children = document.querySelectorAll('.rotate-3d-container > *');
        if (children.length === 2) {
            const array: UIElement[] = Array.from(children);
            // 这里需要清空容器，实际操作需要根据 Vue 模板来处理
            // 假设这里有一个容器元素
            const container = document.querySelector('.rotate-3d-container');
            if (container) {
                container.innerHTML = '';
            }

            _frontWarpper.value = new Border(array[0]);
            _backWarpper.value = new Border(array[1]);
            _backWarpper.value.opacity = 0;
            _backWarpper.value.visibility = 'hidden';

            const viewport = Get3DView();
            viewport.horizontalAlignment = 'stretch';
            viewport.verticalAlignment = 'stretch';

            _front2BackStory.value = new Storyboard();
            _front2BackStory.value.children.add(GetShowHideAnimation(viewport, 0, 800));
            _front2BackStory.value.children.add(GetShowHideAnimation(_backWarpper.value, 800, -1));
            _front2BackStory.value.children.add(GetShowHideAnimation(_frontWarpper.value, -1, 50));
            _front2BackStory.value.children.add(GetFadeAnimation(_frontWarpper.value, 0, -1, 50));
            _front2BackStory.value.children.add(GetFadeAnimation(_backWarpper.value, 1, 750, 50));
            _front2BackStory.value.children.add(GetCameraMoveAnimation(0, 0, 1, 0, 0, 1.4, 0, 400, viewport));
            _front2BackStory.value.children.add(GetRotateAnimation(0, -180, 0.3, 0.3, 50, 750));

            _back2FrontStory.value = new Storyboard();
            _back2FrontStory.value.children.add(GetShowHideAnimation(viewport, 0, 800));
            _back2FrontStory.value.children.add(GetShowHideAnimation(_frontWarpper.value, 800, -1));
            _back2FrontStory.value.children.add(GetShowHideAnimation(_backWarpper.value, -1, 50));
            _back2FrontStory.value.children.add(GetFadeAnimation(_backWarpper.value, 0, -1, 50));
            _back2FrontStory.value.children.add(GetFadeAnimation(_frontWarpper.value, 1, 750, 50));
            _back2FrontStory.value.children.add(GetCameraMoveAnimation(0, 0, 1, 0, 0, 1.4, 0, 400, viewport));
            _back2FrontStory.value.children.add(GetRotateAnimation(180, 360, 0.3, 0.3, 50, 750));

            // 这里需要将元素添加到容器中，实际操作需要根据 Vue 模板来处理
            if (container) {
                // 假设这里有一个方法将 Border 元素添加到容器中
                container.appendChild(_frontWarpper.value.child);
                container.appendChild(_backWarpper.value.child);
                // 假设这里有一个方法将 Viewport3D 元素添加到容器中
                // 由于 Viewport3D 是模拟的，实际需要根据具体实现处理
            }
        }
    };

    const GetFadeAnimation = (target: UIElement, toOpacity: number, beginTime: number, duration: number) => {
        const result = new DoubleAnimation(toOpacity, duration);
        if (beginTime >= 0) {
            result.beginTime = beginTime;
        }
        // 这里需要设置目标和属性路径，实际需要根据 Vue 动画系统处理
        return result;
    };

    const GetCameraMoveAnimation = (
        x1: number,
        y1: number,
        z1: number,
        x2: number,
        y2: number,
        z2: number,
        beginTime: number,
        duration: number,
        view: Viewport3D
    ) => {
        const result = new Point3DAnimation(
            { x: x1, y: y1, z: z1 },
            { x: x2, y: y2, z: z2 },
            duration,
            true,
            beginTime,
            0.3
        );
        // 这里需要设置目标和属性路径，实际需要根据 Vue 动画系统处理
        return result;
    };

    const GetRotateAnimation = (
        fromDegree: number,
        toDegree: number,
        accelerationRatio: number,
        decelerationRatio: number,
        beginTime: number,
        duration: number
    ) => {
        const result = new DoubleAnimation(fromDegree, duration, toDegree, accelerationRatio, decelerationRatio, beginTime);
        // 这里需要设置目标和属性路径，实际需要根据 Vue 动画系统处理
        return result;
    };

    const Get3DView = () => {
        const viewport = new Viewport3D();
        viewport.camera = { position: { x: 0, y: 0, z: 1 } };
        viewport.visibility = 'hidden';
        viewport.children.push(GetLightVisual3D());
        viewport.children.push(GetSceneVisual3D(_frontWarpper.value!, _backWarpper.value!));
        return viewport;
    };

    const GetLightVisual3D = () => {
        const group = new Model3DGroup();
        group.add(new DirectionalLight('#444444', { x: 0, y: 0, z: -1 }));
        group.add(new AmbientLight('#BBBBBB'));
        return new ModelVisual3D(group);
    };

    const GetSceneVisual3D = (frontElement: Border, backElement: Border) => {
        let w = 1.0;
        let h = 0;
        const element = frontElement.child as HTMLElement;
        if (element) {
            h = element.offsetHeight / element.offsetWidth;
        }

        const meshgmod = new MeshGeometry3D();
        meshgmod.triangleIndices = [0, 1, 2, 2, 3, 0];
        meshgmod.textureCoordinates = [
            { x: 0, y: h },
            { x: w, y: h },
            { x: w, y: 0 },
            { x: 0, y: 0 },
        ];
        meshgmod.positions = [
            { x: -w / 2, y: -h / 2, z: 0 },
            { x: w / 2, y: -h / 2, z: 0 },
            { x: w / 2, y: h / 2, z: 0 },
            { x: -w / 2, y: h / 2, z: 0 },
        ];

        const frontBrush = new VisualBrush(frontElement.child);
        const backBrush = new VisualBrush(backElement.child);
        backBrush.relativeTransform = { scaleX: -1, scaleY: 1, centerX: 0.5, centerY: 0 };

        const rotate = new AxisAngleRotation3D({ x: 0, y: 1, z: 0 }, 0);
        // 这里需要注册名称，实际需要根据 Vue 系统处理
        const gmode3D = new GeometryModel3D(meshgmod, new DiffuseMaterial(frontBrush), new DiffuseMaterial(backBrush), new RotateTransform3D(rotate));
        return new ModelVisual3D(gmode3D);
    };

    const GetShowHideAnimation = (element: UIElement, showTime: number, hideTime: number) => {
        const frame = new ObjectAnimationUsingKeyFrames();
        // 这里需要设置目标和属性路径，实际需要根据 Vue 动画系统处理
        if (showTime >= 0) {
            frame.addKeyFrame('visible', showTime);
        }
        if (hideTime >= 0) {
            frame.addKeyFrame('hidden', hideTime);
        }
        return frame;
    };

    const Turn = (isReverse = true) => {
        let target: Storyboard | null = null;
        let fromAngle = 0;
        let step = 180;

        if (_frontWarpper.value && _frontWarpper.value.opacity > 0) {
            target = _front2BackStory.value;
            CurrentIndex.value = 1;
        } else {
            fromAngle = 180;
            target = _back2FrontStory.value;
            CurrentIndex.value = 0;
        }

        const direction = target?.children.children[6] as DoubleAnimation;
        if (!isReverse) {
            step = -180;
        }
        if (direction) {
            direction.from = fromAngle;
            direction.to = fromAngle + step;
        }

        if (target) {
            target.begin();
        }
    };

    onMounted(() => {
        Init();
    });

    return {
        CurrentIndex,
        Turn,
    };
}