import { ref, reactive, onMounted } from 'vue';

// 模拟依赖属性
class DependencyProperty {
    static register(name: string, type: any, owner: any, metadata: any) {
        // 这里只是简单模拟，实际需要更复杂的逻辑来处理依赖属性
        return { name, type, owner, metadata };
    }
}

// 模拟 Thickness 类
class Thickness {
    constructor(public left: number, public top: number, public right: number, public bottom: number) {}
}

// 模拟 FormattedText 类
class FormattedText {
    constructor(
        public text: string,
        public culture: any,
        public flowDirection: any,
        public typeface: any,
        public fontSize: number,
        public foreground: any
    ) {}
    get height() {
        // 简单模拟高度，实际需要根据字体等计算
        return 20; 
    }
}

// 模拟 SolidColorBrush 类
class SolidColorBrush {
    constructor(public color: any) {}
}

// 模拟 Color 类
class Color {
    static fromArgb(a: number, r: number, g: number, b: number) {
        return { a, r, g, b };
    }
}

// 模拟 Line 类
class Line {
    public x1: number = 0;
    public x2: number = 0;
    public y1: number = 0;
    public y2: number = 0;
}

// 模拟 UIElement 类
class UIElement {
    public visibility: string = 'Visible';
    public desiredSize: { width: number; height: number } = { width: 0, height: 0 };
    public arrange(rect: { x: number; y: number; width: number; height: number }) {
        // 这里可以添加实际的布局逻辑
    }
    public measure(size: { width: number; height: number }) {
        // 这里可以添加实际的测量逻辑
    }
}

// 模拟 FrameworkElement 类
class FrameworkElement extends UIElement {
    public width: number | 'NaN' = 'NaN';
}

// 模拟 DrawingContext 类
class DrawingContext {
    drawLine(pen: any, point1: { x: number; y: number }, point2: { x: number; y: number }) {
        // 这里可以添加实际的绘制线条逻辑
    }
    drawText(text: FormattedText, point: { x: number; y: number }) {
        // 这里可以添加实际的绘制文本逻辑
    }
    drawGeometry(brush: any, pen: any, geometry: any) {
        // 这里可以添加实际的绘制几何图形逻辑
    }
}

// 模拟 Brushes 类
class Brushes {
    static get RoyalBlue() {
        return new SolidColorBrush(Color.fromArgb(0xFF, 0x41, 0x69, 0xE1));
    }
}

// 模拟 Pen 类
class Pen {
    constructor(public brush: any, public thickness: number) {}
}

// 模拟 RectangleGeometry 类
class RectangleGeometry {
    constructor(public rect: { x: number; y: number; width: number; height: number }, public radiusX: number, public radiusY: number) {}
}

// 模拟 Rect 类
class Rect {
    constructor(public x: number, public y: number, public width: number, public height: number) {}
}

// 模拟 Action 类型
type Action<T> = (arg: T) => void;

export class TradePanel {
    private _marginKV: number = 10;
    private evenColumn: number = 0.35;
    private oddColumn: number = 0.65;
    private _marginTop: number = 8;
    private _columnSize: number[] = [];
    private lineList: Line[] = [];
    private rowDetail: { [key: number]: number } = {};
    private _columns: number = 4;
    private ft: FormattedText | null = null;

    public margin: Thickness = new Thickness(0, 5, 0, 0);
    public children: FrameworkElement[] = [];

    // 依赖属性
    public static IsShowLineProperty = DependencyProperty.register('IsShowLine', Boolean, TradePanel, {
        affectsRender: true
    });
    public static HeaderProperty = DependencyProperty.register('Header', String, TradePanel, {
        affectsRender: true
    });
    public static ColumnSpanProperty = DependencyProperty.register('ColumnSpan', Number, TradePanel, {
        affectsParentArrange: true
    });
    public static IsCentralAloneProperty = DependencyProperty.register('IsCentralAlone', Boolean, TradePanel, {
        affectsParentArrange: true
    });
    public static GroupNameProperty = DependencyProperty.register('GroupName', String, TradePanel, {});

    // 属性访问器
    public get IsShowLine(): boolean {
        return this[TradePanel.IsShowLineProperty.name] as boolean;
    }
    public set IsShowLine(value: boolean) {
        this[TradePanel.IsShowLineProperty.name] = value;
    }
    public get Header(): string {
        return this[TradePanel.HeaderProperty.name] as string;
    }
    public set Header(value: string) {
        this[TradePanel.HeaderProperty.name] = value;
    }
    public get Columns(): number {
        return this._columns;
    }
    public set Columns(value: number) {
        this._columns = value;
    }

    // 跨列属性方法
    public static getColumnSpan(obj: any): number {
        return obj[TradePanel.ColumnSpanProperty.name] as number;
    }
    public static setColumnSpan(obj: any, value: number) {
        obj[TradePanel.ColumnSpanProperty.name] = value;
    }

    // 单独居中属性方法
    public static getIsCentralAlone(obj: any): boolean {
        return obj[TradePanel.IsCentralAloneProperty.name] as boolean;
    }
    public static setIsCentralAlone(obj: any, value: boolean) {
        obj[TradePanel.IsCentralAloneProperty.name] = value;
    }

    // 子控件分组属性方法
    public static getGroupName(obj: any): string {
        return obj[TradePanel.GroupNameProperty.name] as string;
    }
    public static setGroupName(obj: any, value: string) {
        obj[TradePanel.GroupNameProperty.name] = value;
    }

    // 相关的处理方法
    public raiseGroupAction(groupName: string, action: Action<UIElement>) {
        this.children.forEach((item) => {
            const element = item as UIElement;
            if (element && TradePanel.getGroupName(element) === groupName) {
                action(element);
            }
        });
    }

    // 根据指定元素所在的列数，获取其横坐标
    private getX(col: number, ele: FrameworkElement): number {
        let re = 0;
        if (TradePanel.getIsCentralAlone(ele)) {
            let sum = 0;
            for (let i = 0; i < this._columnSize.length; i++) {
                sum += this._columnSize[i];
            }
            if (ele.width === 'NaN') {
                re = this._marginKV;
            } else {
                re = (sum - ele.desiredSize.width) / 2;
            }
            return re;
        } else {
            let span = TradePanel.getColumnSpan(ele);
            if (span + col > this._columnSize.length) {
                span = this._columnSize.length - col;
            }
            let totalWidth = 0;
            if (col === 0) {
                for (let i = 0; i < span; i++) {
                    totalWidth += this._columnSize[i];
                }
                re = totalWidth - ele.desiredSize.width - this._marginKV;
                return re;
            } else {
                for (let i = 0; i < col; i++) {
                    re += this._columnSize[i];
                }
                if (col % 2 === 0) {
                    for (let i = 0; i < span; i++) {
                        totalWidth += this._columnSize[col + i];
                    }
                    re += totalWidth - ele.desiredSize.width - this._marginKV;
                }
                return re;
            }
        }
    }

    // 布局重写方法
    protected arrangeOverride(finalSize: { width: number; height: number }): { width: number; height: number } {
        let fontHeight = 0;
        if (this.Header) {
            fontHeight = this.ft?.height || 0;
        }
        let currentColumn = 0;
        let lineHight = 0;
        let currentY = this._marginTop + fontHeight;
        const totalColumn = this._columnSize.length;

        for (let i = 0; i < this.children.length; i++) {
            const ele = this.children[i];
            if (ele && ele.visibility!== 'Collapsed') {
                if (TradePanel.getIsCentralAlone(ele)) {
                    currentColumn = 0;
                    if (lineHight === 0) {
                        currentY += lineHight;
                    } else {
                        currentY += this._marginTop + lineHight;
                    }
                    if (ele.width === 'NaN') {
                        ele.arrange({
                            x: this.getX(currentColumn, ele),
                            y: currentY + this.getY(i, ele, this.rowDetail),
                            width: finalSize.width - this._marginKV * 2,
                            height: ele.desiredSize.height
                        });
                    } else {
                        ele.arrange({
                            x: this.getX(currentColumn, ele),
                            y: currentY + this.getY(i, ele, this.rowDetail),
                            width: ele.desiredSize.width,
                            height: ele.desiredSize.height
                        });
                    }
                    lineHight = ele.desiredSize.height;
                    if (lineHight === 0) {
                        currentY += lineHight;
                    } else {
                        currentY += this._marginTop + lineHight;
                    }
                    lineHight = 0;
                } else {
                    let cs = TradePanel.getColumnSpan(ele);
                    if (cs === 0) {
                        cs = 1;
                        TradePanel.setColumnSpan(ele, 1);
                    }
                    if (cs + currentColumn > totalColumn) {
                        currentColumn = 0;
                        if (lineHight === 0) {
                            currentY += lineHight;
                        } else {
                            currentY += this._marginTop + lineHight;
                        }
                        lineHight = 0;
                    }
                    if (ele.desiredSize.height > lineHight) {
                        lineHight = ele.desiredSize.height;
                    }
                    ele.arrange({
                        x: this.getX(currentColumn, ele),
                        y: currentY + this.getY(i, ele, this.rowDetail),
                        width: ele.desiredSize.width,
                        height: ele.desiredSize.height
                    });
                    currentColumn += cs;
                }
            }
        }
        return finalSize;
    }

    // 测量重写方法
    protected measureOverride(availableSize: { width: number; height: number }): { width: number; height: number } {
        let fontHeight = 0;
        if (this.Header) {
            const family = { name: '宋体' };
            const headerFace = { family, style: 'Normal', weight: 'Bold', stretch: 'Normal' };
            this.ft = new FormattedText(
                this.Header,
                null,
                'LeftToRight',
                headerFace,
                14,
                new SolidColorBrush(Color.fromArgb(0xFF, 0x00, 0x71, 0xC0))
            );
            fontHeight = this.ft.height;
        }
        for (let i = 0; i < this.children.length; i++) {
            const ele = this.children[i];
            if (ele && ele.visibility!== 'Collapsed') {
                ele.measure(availableSize);
            }
        }
        if (isNaN(availableSize.width)) {
            availableSize.width = 780;
        }
        this._columnSize = [];
        let total = 0;
        if (this.Columns % 2!== 0) {
            this.Columns++;
            for (let i = 0; i < this.Columns - 2; i++) {
                let columnWidth = 0;
                if (i % 2 === 0) {
                    columnWidth = availableSize.width * (2 / this.Columns) * this.evenColumn;
                } else {
                    columnWidth = availableSize.width * (2 / this.Columns) * this.oddColumn;
                }
                total += columnWidth;
                this._columnSize.push(columnWidth);
            }
            this._columnSize.push(availableSize.width - total);
            this.Columns--;
        } else {
            for (let i = 0; i < this.Columns; i++) {
                let columnWidth = 0;
                if (i % 2 === 0) {
                    columnWidth = availableSize.width * (2 / this.Columns) * this.evenColumn;
                } else {
                    columnWidth = availableSize.width * (2 / this.Columns) * this.oddColumn;
                }
                this._columnSize.push(columnWidth);
            }
        }
        this.setRowDetail(availableSize);
        if (isNaN(availableSize.height)) {
            const totalActualHeight = this.getTotalHeight(this.rowDetail);
            if (totalActualHeight === 0) {
                if (!this.Header) {
                    availableSize.height = 0;
                } else {
                    availableSize.height = fontHeight;
                }
            } else {
                if (!this.Header) {
                    availableSize.height = totalActualHeight + this._marginTop * 2;
                } else {
                    availableSize.height = totalActualHeight + this._marginTop * 2 + fontHeight;
                }
            }
        }
        let sum = 0;
        this.lineList = [];
        for (let j = 0; j < this.Columns - 1; j++) {
            sum += this._columnSize[j];
            const line = new Line();
            line.x1 = sum;
            line.x2 = sum;
            line.y1 = 0;
            line.y2 = availableSize.height;
            this.lineList.push(line);
        }
        return availableSize;
    }

    // 渲染方法
    protected onRender(dc: DrawingContext) {
        if (this.IsShowLine) {
            this.lineList.forEach((line) => {
                dc.drawLine(new Pen(Brushes.RoyalBlue, 1), { x: line.x1, y: line.y1 }, { x: line.x2, y: line.y2 });
            });
        }
        if (this.Header) {
            const family = { name: '宋体' };
            const headerFace = { family, style: 'Normal', weight: 'Bold', stretch: 'Normal' };
            this.ft = new FormattedText(
                this.Header,
                null,
                'LeftToRight',
                headerFace,
                14,
                new SolidColorBrush(Color.fromArgb(0xFF, 0x00, 0x71, 0xC0))
            );
            const p = new Pen(Brushes.RoyalBlue, 0.4);
            const rect = new Rect(0, 0, this.actualWidth, this.actualHeight);
            const rg = new RectangleGeometry(rect, 0, 0);
            const rect2 = new Rect(1, 1, this.actualWidth - 2, this.ft.height);
            const rg2 = new RectangleGeometry(rect2, 0, 0);
            dc.drawGeometry(null, p, rg);
            dc.drawGeometry(
                new SolidColorBrush(Color.fromArgb(0xcA, 0xeb, 0xeb, 0xeb)),
                null,
                rg2
            );
            dc.drawText(this.ft, { x: 9, y: 1 });
        }
    }

    // 设置行详情
    private setRowDetail(size: { width: number; height: number }) {
        this.rowDetail = {};
        let lineHight = 0;
        let lineeleNumber = 0;
        const totalColumn = this._columnSize.length;
        let currentColumn = 0;
        for (let i = 0; i < this.children.length; i++) {
            const ele = this.children[i];
            if (ele && ele.visibility!== 'Collapsed') {
                if (TradePanel.getIsCentralAlone(ele)) {
                    if (currentColumn === 0) {
                        lineeleNumber = i;
                        lineHight = ele.desiredSize.height;
                        this.rowDetail[lineeleNumber] = lineHight;
                        lineHight = 0;
                    } else {
                        currentColumn = 0;
                        lineeleNumber = i - 1;
                        this.rowDetail[lineeleNumber] = lineHight;
                        lineHight = ele.desiredSize.height;
                        lineeleNumber = i;
                        this.rowDetail[lineeleNumber] = lineHight;
                        lineHight = 0;
                    }
                } else {
                    let cs = TradePanel.getColumnSpan(ele);
                    if (cs === 0) {
                        cs = 1;
                        TradePanel.setColumnSpan(ele, 1);
                    }
                    if (cs + currentColumn > totalColumn) {
                        currentColumn = 0;
                        lineeleNumber = i - 1;
                        this.rowDetail[lineeleNumber] = lineHight;
                        lineHight = 0;
                    }
                    if (ele.desiredSize.height > lineHight) {
                        lineHight = ele.desiredSize.height;
                    }
                    currentColumn += cs;
                }
            }
        }
        const lastIndex = this.children.length - 1;
        if (!this.rowDetail[lastIndex]) {
            this.rowDetail[lastIndex] = lineHight;
        }
    }

    // 获取 Y 坐标
    private getY(index: number, ele: FrameworkElement, dic: { [key: number]: number }): number {
        let y = 0;
        const height = ele.desiredSize.height;
        for (const [key, value] of Object.entries(dic)) {
            if (index <= parseInt(key)) {
                y = (value - height) / 2;
                break;
            }
        }
        return y;
    }

    // 获取子元素的实际高度总和(包括间隙)
    private getTotalHeight(dic: { [key: number]: number }): number {
        let lines = 0;
        let height = 0;
        for (const [, value] of Object.entries(dic)) {
            if (value > 0) {
                height += value;
                lines++;
            }
        }
        if (lines > 1) {
            height += (lines - 1) * this._marginTop;
        }
        return height;
    }
}