// 引入必要的类型定义
import { Visibility, RoutedUICommand, CommandBinding, Action, Dispatcher, DispatcherPriority, DependencyObject, FrameworkElement, ScrollViewer, Button, PageBase, ISubmitContract, IAuthorizeContract, IRecheckContract, IPaymentFeeContract, IEvaluationContract, Message, SubmitEntry, AuthorizeEntry, RecheckEntry, PaymentFeeArguments, FeeViewer, TradeAPIs, ContractManager, TradeInfo, AuthManager, PlatformLogger } from './types';

// 定义SubmitPage类
export class SubmitPage extends PageBase implements ISubmitContract, IAuthorizeContract, IRecheckContract, IPaymentFeeContract, IEvaluationContract {
    // 静态构造函数
    static {
        // 重写默认样式键属性
        // 注意：TypeScript中没有直接对应的方式，这里只是模拟逻辑
        // 可以在前端框架中通过样式类等方式实现类似功能
    }

    // 命令定义
    static readonly SubmitCommand: RoutedUICommand = new RoutedUICommand("提交", "SubmitCommand", SubmitPage);
    static readonly CancelCommand: RoutedUICommand = new RoutedUICommand("取消", "CancelButton", SubmitPage);

    // 操作按钮控制
    private _buttonViewer: FrameworkElement | null = null;
    private _submitButton: FrameworkElement | null = null;
    private _cancelButton: FrameworkElement | null = null;

    // 重写OnApplyTemplate方法
    public override OnApplyTemplate(): void {
        super.OnApplyTemplate();
        this._buttonViewer = this.GetTemplateChild("ButtonViewerPanel") as FrameworkElement;
        this._submitButton = this.GetTemplateChild("SubmitButton") as FrameworkElement;
        this._cancelButton = this.GetTemplateChild("CancelButton") as FrameworkElement;

        // 控件滚动条定位
        const scrollViewer = this.GetTemplateChild("TradeScrollViewer") as ScrollViewer;
        if (scrollViewer && !DesignerProperties.GetIsInDesignMode(this)) {
            (scrollViewer.Content as FrameworkElement).HorizontalAlignment = HorizontalAlignment.Left;
            scrollViewer.RequestBringIntoView = (sender, e) => {
                const scroll = sender as ScrollViewer;
                if (scroll) {
                    if (!isNaN(e.TargetRect.Y) && e.TargetRect.Y > scroll.ActualHeight * 2 / 3) {
                        scroll.ScrollToVerticalOffset(scroll.VerticalOffset + e.TargetRect.Y - scroll.ActualHeight * 2 / 3);
                    }
                }
            };
        }
    }

    // 设置操作区域的可见性
    protected ShowBottonViewer(visibility: Visibility): void {
        if (this._buttonViewer) {
            this._buttonViewer.Visibility = visibility;
        }
    }

    // 在操作区域添加按钮
    protected AddOperateButton(buttonMsg: string, clickEventHandler: (sender: any, e: any) => void, index: number = 0): Button {
        let button: Button | null = null;
        const panel = this._buttonViewer as Panel;
        if (panel) {
            button = new Button({ Content: buttonMsg, Margin: new Thickness(0, 0, 10, 0) });
            button.Click = clickEventHandler;
            if (index > panel.Children.length) {
                index = panel.Children.length;
            }
            panel.Children.Insert(index, button);
        }
        return button;
    }

    // 提交按钮属性
    public get SubmitButton(): Button {
        return this._submitButton as Button;
    }

    // 取消按钮属性
    public get CancelButton(): Button {
        return this._cancelButton as Button;
    }

    // 构造函数
    constructor() {
        super();

        // 绑定命令
        this.CommandBindings.Add(new CommandBinding(SubmitCommand,
            (sender, e) => {
                this.ShowBottonViewer(Visibility.Collapsed);
                ContractManager.RaiseSubmit(this);
                this.ShowBottonViewer(Visibility.Visible);
            },
            (sender, e) => {
                e.CanExecute = true;
            }
        ));

        this.CommandBindings.Add(new CommandBinding(CancelCommand,
            (sender, e) => {
                this.Close();
            },
            (sender, e) => {
                e.CanExecute = true;
            }
        ));

        // 构建授权实例
        this.AuthorizeEntries = [];
        const authorizeEntry = new AuthorizeEntry("0");
        authorizeEntry.PreviewAuthorize = (sender, e) => {
            this.PreviewAuthorize(e);
        };
        authorizeEntry.RemoteAuthorized = (sender, e) => {
            const entry = sender as AuthorizeEntry;
            this.SubmitEntry.LoadData(entry.PageData, entry.MessageData);
            this.SubmitEntry.RaiseSubmit();
        };
        this.AuthorizeEntries.push(authorizeEntry);

        // 构建复核实例
        this.RecheckEntries = [];
        const recheckEntry = new RecheckEntry("0");
        recheckEntry.PreviewRecheck = (sender, e) => {
            this.PreviewRecheck(e);
        };
        this.RecheckEntries.push(recheckEntry);

        this.PagePrinted = (s1, e1) => {
            ContractManager.RaiseEvaluation(this);
        };

        Dispatcher.CurrentDispatcher.BeginInvoke(new Action<PageBase>((s1) => {
            if (!TradeAPIs.CheckWorkState(true)) {
                this.Close();
                return;
            }
            if (!TradeAPIs.CheckDFState(true)) {
                this.Close();
                return;
            }
            const flag = Platform.Common.RunningParameters.ConfigManager.GetInstance().GetConfigValue("QBDJ", Platform.Common.RunningParameters.ConfigType.Client);
            if (flag !== "true") return;
            const list = Platform.Common.BasicFunctions.DependencyObjectExtensions.FindVisualTreeChildren(s1 as DependencyObject, x => x instanceof Platform.Controls.NumericContainer.CurrencyEditor);
            list.forEach((item) => {
                Platform.Presentation.FocusManagers.FocusManager.AddPreviewLostFocusHandler(item, (sender, e) => {
                    const element = sender as Platform.Controls.NumericContainer.CurrencyEditor;
                    if (!element) return;
                    const money = new TellerSystem.Controls.Ext.Controls.TBC_MoneyInfo();
                    money.Init(element.Text);
                    const d = new Platform.Controls.TradeExtension.PageDialog();
                    d.Open(this, money, "券别登记");
                    if (!money.IsOk) {
                        Platform.Controls.ControlsHelper.ControlHelper.SetFocus(this, element);
                    }
                });
            });
        }), DispatcherPriority.Background, this);
    }

    // ISubmitContract接口实现
    public PreviewSubmitting(out message: Message): boolean {
        message = new Message();
        return false;
    }

    public SubmitEntry: SubmitEntry;

    // IAuthorizeContract接口实现
    public AuthorizeEntries: AuthorizeEntry[];

    public PreviewAuthorize(args: AuthorizeEntry.PreviewAuthorizeEventArgs): void {
        return;
    }

    // IRecheckContract接口实现
    public RecheckEntries: RecheckEntry[];

    public PreviewRecheck(args: RecheckEntry.PreviewRecheckEventArgs): void {
        return;
    }

    // IPaymentFeeContract接口实现
    public CheckFee(args: PaymentFeeArguments, out isPaymentFee: boolean): boolean {
        isPaymentFee = false;
        const _msg = args.Message;
        let fd16 = _msg.Fd16;
        let fd122_I = "";

        args.Metadata.SetDictionaryItem("DefaultPaymentFeeType", _msg.Fd9_3);

        this.TradeAPIs.CommSend(_msg);
        _msg.Fd16 = "9986";
        _msg.Fd12 = fd16;
        this.TradeAPIs.CallServer(_msg);
        _msg.Fd16 = fd16;

        if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") === "true") {
            fd122_I = _msg.Fd122_I;
        }

        if (_msg.Fd122_F === "1") {
            _msg.ClearFd(122);
            _msg.Fd122_F = "1";
        } else {
            _msg.ClearFd(122);
        }

        if (TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("EncrypteGM") === "true") {
            _msg.Fd122_I = fd122_I;
        }

        if (_msg.Fd12 === "0000") {
            if (_msg.Fd9_1 && _msg.Fd9_2 && _msg.Fd9_3) {
                _msg.Fd9_3 = " ";
            }
            isPaymentFee = false;
        } else if (_msg.Fd12 === "P047") {
            isPaymentFee = true;
            return true;
        } else {
            return false;
        }
        return true;
    }

    public RaisePaymentFee(args: PaymentFeeArguments): boolean {
        const fd9_3 = args.Metadata.GetDictionaryItem<string>("DefaultPaymentFeeType");
        const _msg = args.Message;

        if (fd9_3 && !_msg.Fd9_3) {
            _msg.Fd9_3 = fd9_3;
        }
        const fee = new FeeViewer(_msg);
        fee.Is_Wy = false;
        if (_msg.FileData.indexOf("存款证明手续费") !== -1) {
            fee._is_Cunkuan = true;
        }
        this.ShowPage(fee, "收费界面");
        if (!fee.IsFinishFee) {
            this.TradeAPIs.ShowErrorMessageBox("收费失败！");
            return false;
        }

        const FLAG = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("2ea3a7e6cfa04f89a335bf1e363295de");
        if (FLAG === "V11" && (_msg.Fd16 === "2405" || _msg.Fd16 === "2105" || _msg.Fd16 === "3522" || _msg.Fd16 === "5105")) {
            if (TradeInfo.TradeScreens.has("fee")) {
                TradeInfo.TradeScreens.delete("fee");
                TradeInfo.TradeScreens.set("fee", AuthManager.GetPageImages(fee));
            } else {
                TradeInfo.TradeScreens.set("fee", AuthManager.GetPageImages(fee));
            }
        }

        if (FLAG === "V21" && (_msg.Fd16 === "2405" || _msg.Fd16 === "5112" || _msg.Fd16 === "3522" || _msg.Fd16 === "5105" || _msg.Fd16 === "2410")) {
            if (TradeInfo.TradeScreens.has("fee")) {
                TradeInfo.TradeScreens.delete("fee");
                TradeInfo.TradeScreens.set("fee", AuthManager.GetPageImages(fee));
            } else {
                TradeInfo.TradeScreens.set("fee", AuthManager.GetPageImages(fee));
            }
        }

        if (this.SubmitEntry && fee.PrintFeeGuest) {
            if (FLAG === "V11" && _msg.Fd16 === "5105") {
                // 不做处理
            } else if ((FLAG === "V11" && _msg.Fd16 !== "5105") || FLAG === "V21" || FLAG === "V22") {
                this.SubmitEntry.MorePrintList = this.SubmitEntry.MorePrintList.filter(x => !(x instanceof Print_Fee));
                this.SubmitEntry.MorePrintList.push(fee.PrintFee);
                this.SubmitEntry.MorePrintList = this.SubmitEntry.MorePrintList.filter(x => !(x instanceof Print_FeeGuest));
                this.SubmitEntry.MorePrintList.push(fee.PrintFeeGuest);
            } else {
                this.SubmitEntry.MorePrintList = this.SubmitEntry.MorePrintList.filter(x => !(x instanceof Print_FeeGuest));
                this.SubmitEntry.MorePrintList.push(fee.PrintFeeGuest);
            }
        } else {
            this.SubmitEntry.MorePrintList = this.SubmitEntry.MorePrintList.filter(x => !(x instanceof Print_Fee));
            this.SubmitEntry.MorePrintList = this.SubmitEntry.MorePrintList.filter(x => !(x instanceof Print_FeeGuest));
        }
        return true;
    }

    // IEvaluationContract接口实现
    public PreviewEvaluation(record: EvaluationRecord): boolean {
        return false;
    }
}