// 定义 Action 类型，模拟 C# 中的 Action 委托
type Action<T> = (arg: T) => void;

export interface IMessenger {
    /**
     * Registers a recipient for a type of message TMessage. The action
     * parameter will be executed when a corresponding message is sent.
     * Registering a recipient does not create a hard reference to it,
     * so if this recipient is deleted, no memory leak is caused.
     * 
     * @template TMessage - The type of message that the recipient registers for.
     * @param recipient - The recipient that will receive the messages.
     * @param action - The action that will be executed when a message of type TMessage is sent.
     */
    Register<TMessage>(recipient: any, action: Action<TMessage>): void;

    /**
     * Registers a recipient for a type of message TMessage.
     * The action parameter will be executed when a corresponding 
     * message is sent. See the receiveDerivedMessagesToo parameter
     * for details on how messages deriving from TMessage (or, if TMessage is an interface,
     * messages implementing TMessage) can be received too.
     * Registering a recipient does not create a hard reference to it,
     * so if this recipient is deleted, no memory leak is caused.
     * 
     * @template TMessage - The type of message that the recipient registers for.
     * @param recipient - The recipient that will receive the messages.
     * @param token - A token for a messaging channel. If a recipient registers
     * using a token, and a sender sends a message using the same token, then this
     * message will be delivered to the recipient. Other recipients who did not
     * use a token when registering (or who used a different token) will not
     * get the message. Similarly, messages sent without any token, or with a different
     * token, will not be delivered to that recipient.
     * @param action - The action that will be executed when a message of type TMessage is sent.
     */
    Register<TMessage>(recipient: any, token: any, action: Action<TMessage>): void;

    /**
     * Registers a recipient for a type of message TMessage.
     * The action parameter will be executed when a corresponding 
     * message is sent. See the receiveDerivedMessagesToo parameter
     * for details on how messages deriving from TMessage (or, if TMessage is an interface,
     * messages implementing TMessage) can be received too.
     * Registering a recipient does not create a hard reference to it,
     * so if this recipient is deleted, no memory leak is caused.
     * 
     * @template TMessage - The type of message that the recipient registers for.
     * @param recipient - The recipient that will receive the messages.
     * @param token - A token for a messaging channel. If a recipient registers
     * using a token, and a sender sends a message using the same token, then this
     * message will be delivered to the recipient. Other recipients who did not
     * use a token when registering (or who used a different token) will not
     * get the message. Similarly, messages sent without any token, or with a different
     * token, will not be delivered to that recipient.
     * @param receiveDerivedMessagesToo - If true, message types deriving from
     * TMessage will also be transmitted to the recipient. For example, if a SendOrderMessage
     * and an ExecuteOrderMessage derive from OrderMessage, registering for OrderMessage
     * and setting receiveDerivedMessagesToo to true will send SendOrderMessage
     * and ExecuteOrderMessage to the recipient that registered.
     * Also, if TMessage is an interface, message types implementing TMessage will also be
     * transmitted to the recipient. For example, if a SendOrderMessage
     * and an ExecuteOrderMessage implement IOrderMessage, registering for IOrderMessage
     * and setting receiveDerivedMessagesToo to true will send SendOrderMessage
     * and ExecuteOrderMessage to the recipient that registered.
     * @param action - The action that will be executed when a message of type TMessage is sent.
     */
    Register<TMessage>(recipient: any, token: any, receiveDerivedMessagesToo: boolean, action: Action<TMessage>): void;

    /**
     * Registers a recipient for a type of message TMessage.
     * The action parameter will be executed when a corresponding 
     * message is sent. See the receiveDerivedMessagesToo parameter
     * for details on how messages deriving from TMessage (or, if TMessage is an interface,
     * messages implementing TMessage) can be received too.
     * Registering a recipient does not create a hard reference to it,
     * so if this recipient is deleted, no memory leak is caused.
     * 
     * @template TMessage - The type of message that the recipient registers for.
     * @param recipient - The recipient that will receive the messages.
     * @param receiveDerivedMessagesToo - If true, message types deriving from
     * TMessage will also be transmitted to the recipient. For example, if a SendOrderMessage
     * and an ExecuteOrderMessage derive from OrderMessage, registering for OrderMessage
     * and setting receiveDerivedMessagesToo to true will send SendOrderMessage
     * and ExecuteOrderMessage to the recipient that registered.
     * Also, if TMessage is an interface, message types implementing TMessage will also be
     * transmitted to the recipient. For example, if a SendOrderMessage
     * and an ExecuteOrderMessage implement IOrderMessage, registering for IOrderMessage
     * and setting receiveDerivedMessagesToo to true will send SendOrderMessage
     * and ExecuteOrderMessage to the recipient that registered.
     * @param action - The action that will be executed when a message of type TMessage is sent.
     */
    Register<TMessage>(recipient: any, receiveDerivedMessagesToo: boolean, action: Action<TMessage>): void;

    /**
     * Sends a message to registered recipients. The message will
     * reach all recipients that registered for this message type
     * using one of the Register methods.
     * 
     * @template TMessage - The type of message that will be sent.
     * @param message - The message to send to registered recipients.
     */
    Send<TMessage>(message: TMessage): void;

    /**
     * Sends a message to registered recipients. The message will
     * reach only recipients that registered for this message type
     * using one of the Register methods, and that are
     * of the targetType.
     * 
     * @template TMessage - The type of message that will be sent.
     * @template TTarget - The type of recipients that will receive
     * the message. The message won't be sent to recipients of another type.
     * @param message - The message to send to registered recipients.
     */
    Send<TMessage, TTarget>(message: TMessage): void;

    /**
     * Sends a message to registered recipients. The message will
     * reach only recipients that registered for this message type
     * using one of the Register methods, and that are
     * of the targetType.
     * 
     * @template TMessage - The type of message that will be sent.
     * @param message - The message to send to registered recipients.
     * @param token - A token for a messaging channel. If a recipient registers
     * using a token, and a sender sends a message using the same token, then this
     * message will be delivered to the recipient. Other recipients who did not
     * use a token when registering (or who used a different token) will not
     * get the message. Similarly, messages sent without any token, or with a different
     * token, will not be delivered to that recipient.
     */
    Send<TMessage>(message: TMessage, token: any): void;

    /**
     * Unregisters a messager recipient completely. After this method
     * is executed, the recipient will not receive any messages anymore.
     * 
     * @param recipient - The recipient that must be unregistered.
     */
    Unregister(recipient: any): void;

    /**
     * Unregisters a message recipient for a given type of messages only. 
     * After this method is executed, the recipient will not receive messages
     * of type TMessage anymore, but will still receive other message types (if it
     * registered for them previously).
     * 
     * @template TMessage - The type of messages that the recipient wants
     * to unregister from.
     * @param recipient - The recipient that must be unregistered.
     */
    Unregister<TMessage>(recipient: any): void;

    /**
     * Unregisters a message recipient for a given type of messages only and for a given token. 
     * After this method is executed, the recipient will not receive messages
     * of type TMessage anymore with the given token, but will still receive other message types
     * or messages with other tokens (if it registered for them previously).
     * 
     * @template TMessage - The type of messages that the recipient wants
     * to unregister from.
     * @param recipient - The recipient that must be unregistered.
     * @param token - The token for which the recipient must be unregistered.
     */
    Unregister<TMessage>(recipient: any, token: any): void;

    /**
     * Unregisters a message recipient for a given type of messages and for
     * a given action. Other message types will still be transmitted to the
     * recipient (if it registered for them previously). Other actions that have
     * been registered for the message type TMessage and for the given recipient (if
     * available) will also remain available.
     * 
     * @template TMessage - The type of messages that the recipient wants
     * to unregister from.
     * @param recipient - The recipient that must be unregistered.
     * @param action - The action that must be unregistered for
     * the recipient and for the message type TMessage.
     */
    Unregister<TMessage>(recipient: any, action: Action<TMessage>): void;

    /**
     * Unregisters a message recipient for a given type of messages, for
     * a given action and a given token. Other message types will still be transmitted to the
     * recipient (if it registered for them previously). Other actions that have
     * been registered for the message type TMessage, for the given recipient and other tokens (if
     * available) will also remain available.
     * 
     * @template TMessage - The type of messages that the recipient wants
     * to unregister from.
     * @param recipient - The recipient that must be unregistered.
     * @param token - The token for which the recipient must be unregistered.
     * @param action - The action that must be unregistered for
     * the recipient and for the message type TMessage.
     */
    Unregister<TMessage>(recipient: any, token: any, action: Action<TMessage>): void;
}