// 定义 WeakAction 接口，模拟 C# 中的 WeakAction 功能
interface WeakAction {
    IsAlive: boolean;
    Target: any;
    MethodName?: string;
    MarkForDeletion(): void;
    ExecuteWithObject(message: any): void;
}

// 定义 WeakActionAndToken 结构
interface WeakActionAndToken {
    Action: WeakAction;
    Token: any;
}

// 定义 IMessenger 接口
interface IMessenger {
    Register<TMessage>(recipient: any, action: (message: TMessage) => void): void;
    Register<TMessage>(recipient: any, receiveDerivedMessagesToo: boolean, action: (message: TMessage) => void): void;
    Register<TMessage>(recipient: any, token: any, action: (message: TMessage) => void): void;
    Register<TMessage>(recipient: any, token: any, receiveDerivedMessagesToo: boolean, action: (message: TMessage) => void): void;
    Send<TMessage>(message: TMessage): void;
    Send<TMessage, TTarget>(message: TMessage): void;
    Send<TMessage>(message: TMessage, token: any): void;
    Unregister(recipient: any): void;
    Unregister<TMessage>(recipient: any): void;
    Unregister<TMessage>(recipient: any, token: any): void;
    Unregister<TMessage>(recipient: any, action: (message: TMessage) => void): void;
    Unregister<TMessage>(recipient: any, token: any, action: (message: TMessage) => void): void;
    OverrideDefault(newMessenger: IMessenger): void;
    Reset(): void;
    ResetAll(): void;
    RequestCleanup(): void;
    Cleanup(): void;
}

// 实现 Messenger 类
class Messenger implements IMessenger {
    private static CreationLock: object = {};
    private static _defaultInstance: IMessenger | null = null;
    private _registerLock: object = {};
    private _recipientsOfSubclassesAction: Map<Function, WeakActionAndToken[]> | null = null;
    private _recipientsStrictAction: Map<Function, WeakActionAndToken[]> | null = null;
    private _isCleanupRegistered: boolean = false;

    public static get Default(): IMessenger {
        if (Messenger._defaultInstance === null) {
            // 使用闭包模拟锁机制
            (() => {
                if (Messenger._defaultInstance === null) {
                    Messenger._defaultInstance = new Messenger();
                }
            })();
        }
        return Messenger._defaultInstance;
    }

    public Register<TMessage>(recipient: any, param1: any, param2?: any, param3?: any): void {
        let token: any = null;
        let receiveDerivedMessagesToo: boolean = false;
        let action: (message: TMessage) => void;

        if (typeof param1 === 'boolean') {
            receiveDerivedMessagesToo = param1;
            action = param2 as (message: TMessage) => void;
        } else if (typeof param1 === 'function') {
            action = param1;
        } else {
            token = param1;
            if (typeof param2 === 'boolean') {
                receiveDerivedMessagesToo = param2;
                action = param3 as (message: TMessage) => void;
            } else {
                action = param2 as (message: TMessage) => void;
            }
        }

        const messageType = this.getMessageType<TMessage>();
        let recipients: Map<Function, WeakActionAndToken[]>;

        if (receiveDerivedMessagesToo) {
            if (this._recipientsOfSubclassesAction === null) {
                this._recipientsOfSubclassesAction = new Map<Function, WeakActionAndToken[]>();
            }
            recipients = this._recipientsOfSubclassesAction;
        } else {
            if (this._recipientsStrictAction === null) {
                this._recipientsStrictAction = new Map<Function, WeakActionAndToken[]>();
            }
            recipients = this._recipientsStrictAction;
        }

        // 模拟锁机制
        (() => {
            let list: WeakActionAndToken[];
            if (!recipients.has(messageType)) {
                list = [];
                recipients.set(messageType, list);
            } else {
                list = recipients.get(messageType)!;
            }

            // 这里简单模拟 WeakAction，实际使用中可能需要更完善的实现
            const weakAction: WeakAction = {
                IsAlive: true,
                Target: recipient,
                MethodName: action.name,
                MarkForDeletion() {
                    this.IsAlive = false;
                },
                ExecuteWithObject(message: any) {
                    action(message);
                }
            };

            const item: WeakActionAndToken = {
                Action: weakAction,
                Token: token
            };

            list.push(item);
        })();

        this.RequestCleanup();
    }

    public Send<TMessage>(message: TMessage, param1?: any, param2?: any): void {
        const messageType = this.getMessageType<TMessage>();
        const messageTargetType = param1 as Function | null;
        const token = param2;

        if (this._recipientsOfSubclassesAction) {
            const listClone = Array.from(this._recipientsOfSubclassesAction.keys());
            listClone.forEach((type) => {
                let list: WeakActionAndToken[] | null = null;
                if (messageType === type || messageType.prototype instanceof type || type.prototype.isPrototypeOf(messageType.prototype)) {
                    list = this._recipientsOfSubclassesAction!.get(type)?.slice() || null;
                }
                this.SendToList(message, list, messageTargetType, token);
            });
        }

        if (this._recipientsStrictAction) {
            let list: WeakActionAndToken[] | null = null;
            if (this._recipientsStrictAction.has(messageType)) {
                list = this._recipientsStrictAction.get(messageType)?.slice() || null;
            }
            if (list) {
                this.SendToList(message, list, messageTargetType, token);
            }
        }

        this.RequestCleanup();
    }

    public Unregister(recipient: any, param1?: any, param2?: any, param3?: any): void {
        if (typeof param1 === 'undefined') {
            this.UnregisterFromLists(recipient, this._recipientsOfSubclassesAction);
            this.UnregisterFromLists(recipient, this._recipientsStrictAction);
        } else {
            const messageType = this.getMessageType<any>();
            this.UnregisterFromLists(recipient, param1, param2, this._recipientsStrictAction);
            this.UnregisterFromLists(recipient, param1, param2, this._recipientsOfSubclassesAction);
            this.RequestCleanup();
        }
    }

    public static OverrideDefault(newMessenger: IMessenger): void {
        Messenger._defaultInstance = newMessenger;
    }

    public static Reset(): void {
        Messenger._defaultInstance = null;
    }

    public ResetAll(): void {
        Messenger.Reset();
    }

    private CleanupList(lists: Map<Function, WeakActionAndToken[]> | null): void {
        if (!lists) {
            return;
        }
        // 模拟锁机制
        (() => {
            const listsToRemove: Function[] = [];
            lists.forEach((list, type) => {
                const recipientsToRemove = list.filter(item => !item.Action.IsAlive);
                recipientsToRemove.forEach(recipient => {
                    const index = list.indexOf(recipient);
                    if (index !== -1) {
                        list.splice(index, 1);
                    }
                });
                if (list.length === 0) {
                    listsToRemove.push(type);
                }
            });
            listsToRemove.forEach(key => {
                lists.delete(key);
            });
        })();
    }

    private SendToList<TMessage>(message: TMessage, weakActionsAndTokens: WeakActionAndToken[] | null, messageTargetType: Function | null, token: any): void {
        if (weakActionsAndTokens) {
            const listClone = weakActionsAndTokens.slice();
            listClone.forEach((item) => {
                if (item.Action.IsAlive && item.Action.Target &&
                    (messageTargetType === null || item.Action.Target.constructor === messageTargetType || messageTargetType.prototype.isPrototypeOf(item.Action.Target.constructor.prototype)) &&
                    ((item.Token === null && token === null) || (item.Token !== null && item.Token === token))) {
                    item.Action.ExecuteWithObject(message);
                }
            });
        }
    }

    private UnregisterFromLists(recipient: any, lists: Map<Function, WeakActionAndToken[]> | null): void {
        if (!recipient || !lists || lists.size === 0) {
            return;
        }
        // 模拟锁机制
        (() => {
            lists.forEach((list) => {
                list.forEach((item) => {
                    if (item.Action.Target === recipient) {
                        item.Action.MarkForDeletion();
                    }
                });
            });
        })();
    }

    private UnregisterFromLists<TMessage>(recipient: any, token: any, action: ((message: TMessage) => void) | null, lists: Map<Function, WeakActionAndToken[]> | null): void {
        const messageType = this.getMessageType<TMessage>();
        if (!recipient || !lists || lists.size === 0 || !lists.has(messageType)) {
            return;
        }
        // 模拟锁机制
        (() => {
            const list = lists.get(messageType)!;
            list.forEach((item) => {
                const weakActionCasted = item.Action;
                if (weakActionCasted.Target === recipient &&
                    (action === null || action.name === weakActionCasted.MethodName) &&
                    (token === null || token === item.Token)) {
                    weakActionCasted.MarkForDeletion();
                }
            });
        })();
    }

    public RequestCleanup(): void {
        if (!this._isCleanupRegistered) {
            // 模拟 Dispatcher 延迟执行
            setTimeout(() => {
                this.Cleanup();
            }, 0);
            this._isCleanupRegistered = true;
        }
    }

    public Cleanup(): void {
        this.CleanupList(this._recipientsOfSubclassesAction);
        this.CleanupList(this._recipientsStrictAction);
        this._isCleanupRegistered = false;
    }

    private getMessageType<TMessage>(): Function {
        // 简单模拟获取类型，实际可能需要更复杂的处理
        return ({} as TMessage).constructor;
    }
}

export { Messenger, IMessenger };