﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.Reflection;

namespace TellerSystem.Library.Ext.TradeExtension
{
    public class WeakAction
    {
        private Action _staticAction;

        /// <summary>
        ///     构造函数
        /// </summary>
        protected WeakAction()
        {
        }

        /// <summary>
        ///     构造函数
        /// </summary>
        /// <param name="action">需要关联的Action</param>
        public WeakAction(Action action)
            : this(action == null ? null : action.Target, action)
        {
        }

        /// <summary>
        ///     Initializes a new instance of the <see cref="WeakAction" /> class.
        /// </summary>
        /// <param name="target">The action's owner.</param>
        /// <param name="action">The action that will be associated to this instance.</param>
        [SuppressMessage(
            "Microsoft.Design",
            "CA1062:Validate arguments of public methods",
            MessageId = "1",
            Justification = "Method should fail with an exception if action is null.")]
        public WeakAction(object target, Action action)
        {
            if (action.Method.IsStatic)
            {
                _staticAction = action;

                if (target != null)
                {
                    // Keep a reference to the target to control the
                    // WeakAction's lifetime.
                    Reference = new WeakReference(target);
                }

                return;
            }

            Method = action.Method;
            ActionReference = new WeakReference(action.Target);
            Reference = new WeakReference(target);
        }

        /// <summary>
        ///     引用的取方法信息
        /// </summary>
        protected MethodInfo Method { get; set; }

        /// <summary>
        ///     引用的方法名称
        /// </summary>
        public virtual string MethodName
        {
            get
            {
                if (_staticAction != null)
                {
                    return _staticAction.Method.Name;
                }

                return Method.Name;
            }
        }

        /// <summary>
        ///     行为目标实体的弱引用
        /// </summary>
        protected WeakReference ActionReference { get; set; }

        /// <summary>
        ///     行为对象实体的弱引用
        /// </summary>
        protected WeakReference Reference { get; set; }

        /// <summary>
        ///     静态方法标志
        /// </summary>
        public bool IsStatic
        {
            get { return _staticAction != null; }
        }

        /// <summary>
        ///     实体 活动标志
        /// </summary>
        public virtual bool IsAlive
        {
            get
            {
                if (_staticAction == null
                    && Reference == null)
                {
                    return false;
                }

                if (_staticAction != null)
                {
                    if (Reference != null)
                    {
                        return Reference.IsAlive;
                    }

                    return true;
                }

                return Reference.IsAlive;
            }
        }

        /// <summary>
        ///     Action的拥有者
        /// </summary>
        public object Target
        {
            get
            {
                if (Reference == null)
                {
                    return null;
                }

                return Reference.Target;
            }
        }

        /// <summary>
        ///     Action触发对象
        /// </summary>
        protected object ActionTarget
        {
            get
            {
                if (ActionReference == null)
                {
                    return null;
                }

                return ActionReference.Target;
            }
        }

        /// <summary>
        ///     执行活动中的Action
        /// </summary>
        public void Execute()
        {
            if (_staticAction != null)
            {
                _staticAction();
                return;
            }

            object actionTarget = ActionTarget;

            if (IsAlive)
            {
                if (Method != null
                    && ActionReference != null
                    && actionTarget != null)
                {
                    Method.Invoke(actionTarget, null);
                    return;
                }
            }
        }

        /// <summary>
        ///     清空操作
        /// </summary>
        public void MarkForDeletion()
        {
            Reference = null;
            ActionReference = null;
            Method = null;
            _staticAction = null;
        }
    }
}