﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Xml;
using Platform.Common.LogSystem;
using Platform.Common.RunningParameters;
using Platform.Library;
using TellerSystem.Communication.SocketListener;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.Library.Ext.TradeInterfaces;
using Platform.Presentation.Interfaces;

namespace TellerSystem.Library.Ext.TradeExtension
{
    /// <summary>
    ///     消息推送管理
    /// </summary>
    public static class NotificationManager
    {
        #region Field

        /// <summary>
        /// Socket监听对象
        /// </summary>
        private static SocketManager _socketEntry;

        /// <summary>
        /// 消息类型列表
        /// </summary>
        private static IEnumerable<Type> _messageTypeList;

        #endregion

        #region Method

        /// <summary>
        ///     启动监听模块
        /// </summary>
        /// <param name="port"></param>
        public static void Startup(int port)
        {
            // NotificationMessageBox.IsReceiveMessage = true;
            //开始监听
            _socketEntry.StartListenning(port);
        }

        /// <summary>
        ///     关闭监听模块
        /// </summary>
        public static void Shutdown()
        {
            _socketEntry.Dispose();
            _socketEntry = null;
        }

        /// <summary>
        ///  初始化消息推送模块
        /// </summary>
        public static void Init(int port)
        {
            // 初始化Socket对象
            _socketEntry = new SocketManager();

            //监听错误处理
            _socketEntry.ErrorHappened += (sender, e) =>
                {
                    PlatformLogger.SystemErrorInfo(string.Format("开启客户端监听模块失败！Operation[{0}]", e.ErrorOperation),
                                                   e.SocketException);
                    //MsgHelper.ShowErrorMessageBox(null, "消息监听模块启用失败！");
                };

            //监听开始处理
            _socketEntry.ServerStart +=
                (sender, e) => PlatformLogger.SystemInfo(string.Format("开启客户端监听模块！Port[{0}]...", e.Port));

            //监听停止处理
            _socketEntry.ServerStop +=
                (sender, e) => PlatformLogger.SystemInfo(string.Format("关闭客户端监听模块！Port[{0}]...", e.Port));

            //接收数据处理
            _socketEntry.DataReceived += (sender, e) =>
                {
                    if (!e.IsCompleted || e.Data == null) return;
                    var data = new ReceivedData(e.Data) { Handle = false };
                    Application.Current.Dispatcher.Invoke(new Action<ReceivedData>(x =>
                        {
                            //分析报文类型
                            dynamic message = AnalyzeData(x.Data);

                            if (message != null)
                            {
                                //发送对应类型报文消息                                
                                Messenger.Default.Send(message);

                            }
                        }), data);
                };

            try
            {

                //初始化所有消息窗体(开启对应类型报文监听)
                Assembly.GetExecutingAssembly()
                        .GetTypes()
                        .Where(c => c.GetInterface("INotification") != null)
                        .ToList()
                        .ForEach(item => item.GetMethod("Init").Invoke(null, null));
            }
            catch
            {

            }
            //开启监听服务
            Startup(port);
        }
        private static MessageBase AnalyzeData(byte[] data)
        {
            MessageBase message = null;

            if (data != null)
            {
                try
                {
                    var md = new MessageData(GetXmlNode(data));

                    if (md.Type == "UploadLog")
                    {
                        try
                        {
                            string filePath = "Logs/" + md.Trade + "/" + md.Sender + "/" + md.Other;
                            if (System.IO.File.Exists(filePath))
                            {
                                string fileName = md.Other;                               
                                string upFilePath = "UpLoadLogs" + "/" + md.Trade + "/" + md.Sender + "/";
                                byte[] filedata=System.IO.File.ReadAllBytes(filePath);
                                TradeHandle.FtpUpLoad(fileName, upFilePath, filedata);
                            }
                        }
                        catch { }
                    }
                    if (md.Type == "5")
                    {
                        try
                        {
                            var page = MainPageManager.GetMainPage() as IPage;
                            if (page == null)
                                MessageBox.Show("对不起，您因故被柜员“" + md.Sender + "”强制下线。", "下线提醒");
                            else
                                page.ShowMessageBox("对不起，您因故被柜员“" + md.Sender + "”强制下线。", "下线提醒");
                            Application.Current.Dispatcher.InvokeShutdown();
                        }
                        catch { }
                    }                  
                    if (_messageTypeList == null)
                    {
                        //获取报文类型列表
                        _messageTypeList = Assembly.GetExecutingAssembly()
                                                   .GetTypes()
                                                   .Where(c => c.BaseType == typeof(MessageBase));
                    }

                    //获取对应报文类型
                    Type type = _messageTypeList.SingleOrDefault(c =>
                        {
                            object[] atts = c.GetCustomAttributes(typeof(NotificationAttribute), false);

                            if (atts.Count() == 1)
                            {
                                var att = atts[0] as NotificationAttribute;
                                if (att != null && att.NotificationFlag == md.Type)
                                {
                                    return true;
                                }
                            }
                            return false;
                        });

                    //生成对应报文实例
                    if (type != null)
                    {
                        object item = Assembly.Load("TellerSystem.Library.Ext.TradeExtension")
                                              .CreateInstance(type.FullName, true, BindingFlags.Default, null, new Object[]
                                                  {
                                                      md
                                                  }, null, null);

                        message = item as MessageBase;
                    }
                }
                catch (Exception e)
                {
                    PlatformLogger.SystemErrorInfo("无效的通知报文！", e);
                }
            }
            return message;
        }

        /// <summary>
        /// XML数据转换
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        private static XmlNode GetXmlNode(byte[] data)
        {
            string sData = PlatformSettings.Encoding.GetString(data);
            var xd = new XmlDocument();
            XmlDeclaration dec = xd.CreateXmlDeclaration("1.0", "GB2312", null);
            xd.AppendChild(dec);
            xd.LoadXml(sData);
            XmlNode xnRoot = xd.SelectSingleNode("root");
            return xnRoot;
        }

        #endregion

        #region Class

        /// <summary>
        ///  接收数据
        /// </summary>
        public class ReceivedData
        {
            /// <summary>
            /// </summary>
            /// <param name="data"></param>
            public ReceivedData(byte[] data)
            {
                Data = data;
            }

            /// <summary>
            ///     数据是否已处理
            /// </summary>
            public bool Handle { get; set; }

            /// <summary>
            ///     收到的数据对象
            /// </summary>
            public byte[] Data { get; private set; }
        }

        #endregion
    }
}