// 定义一个泛型函数类型 Action
type Action = () => void;

// 定义 WeakReference 类来模拟 C# 中的弱引用
class WeakReference<T extends object> {
    private ref: WeakRef<T> | null;

    constructor(target: T | null) {
        this.ref = target ? new WeakRef(target) : null;
    }

    get IsAlive(): boolean {
        return this.ref !== null && this.ref.deref() !== undefined;
    }

    get Target(): T | null {
        return this.ref ? this.ref.deref() || null : null;
    }
}

// 定义 WeakAction 类
export class WeakAction {
    private _staticAction: Action | null = null;
    protected Method: PropertyDescriptor | null = null;
    protected ActionReference: WeakReference<object> | null = null;
    protected Reference: WeakReference<object> | null = null;

    // 构造函数
    protected constructor() { }

    // 构造函数
    constructor(action: Action | null);
    constructor(target: object | null, action: Action);
    constructor(arg1?: object | Action | null, arg2?: Action) {
        if (typeof arg1 === 'function') {
            const action = arg1;
            this.constructor(action ? action.Target : null, action);
        } else if (arg1 !== undefined && arg2 !== undefined) {
            const target = arg1;
            const action = arg2;
            const method = Object.getOwnPropertyDescriptor(action, 'prototype')?.value;
            if (method && typeof method === 'function' && method.hasOwnProperty('name')) {
                this._staticAction = action;
                if (target) {
                    this.Reference = new WeakReference(target);
                }
                return;
            }

            this.Method = Object.getOwnPropertyDescriptor(action, 'prototype');
            this.ActionReference = new WeakReference(action.Target || {});
            this.Reference = new WeakReference(target);
        }
    }

    // 引用的方法名称
    get MethodName(): string {
        if (this._staticAction) {
            return this._staticAction.name;
        }
        return this.Method?.value?.name || '';
    }

    // 静态方法标志
    get IsStatic(): boolean {
        return this._staticAction !== null;
    }

    // 实体 活动标志
    get IsAlive(): boolean {
        if (!this._staticAction && !this.Reference) {
            return false;
        }

        if (this._staticAction) {
            if (this.Reference) {
                return this.Reference.IsAlive;
            }
            return true;
        }

        return this.Reference?.IsAlive || false;
    }

    // Action 的拥有者
    get Target(): object | null {
        return this.Reference?.Target || null;
    }

    // Action 触发对象
    protected get ActionTarget(): object | null {
        return this.ActionReference?.Target || null;
    }

    // 执行活动中的 Action
    Execute(): void {
        if (this._staticAction) {
            this._staticAction();
            return;
        }

        const actionTarget = this.ActionTarget;

        if (this.IsAlive) {
            if (this.Method && this.ActionReference && actionTarget) {
                const methodFunc = this.Method.value as Function;
                methodFunc.apply(actionTarget);
            }
        }
    }

    // 清空操作
    MarkForDeletion(): void {
        this.Reference = null;
        this.ActionReference = null;
        this.Method = null;
        this._staticAction = null;
    }
}