// 模拟 DependencyObject 类
class DependencyObject {
    private properties: { [key: string]: any } = {};

    getValue(key: string): any {
        return this.properties[key];
    }

    setValue(key: string, value: any): void {
        this.properties[key] = value;
    }
}

// 模拟 UIPropertyMetadata 类
class UIPropertyMetadata {
    constructor(public defaultValue: any, public propertyChangedCallback?: (d: DependencyObject, e: any) => void) {}
}

// 模拟 DependencyProperty 类
class DependencyProperty {
    constructor(
        public name: string,
        public type: any,
        public ownerType: any,
        public metadata: UIPropertyMetadata
    ) {}

    static registerAttached(name: string, type: any, ownerType: any, metadata: UIPropertyMetadata): DependencyProperty {
        return new DependencyProperty(name, type, ownerType, metadata);
    }
}

// 模拟 RegexValidationRules 类
class RegexValidationRules {
    private static _instance: RegexValidationRules;
    private rules: { [key: string]: { pattern: string; message: string } } = {};

    private constructor() {}

    static get Instance(): RegexValidationRules {
        if (!this._instance) {
            this._instance = new RegexValidationRules();
        }
        return this._instance;
    }

    AddRule(key: string, pattern: string, message: string): void {
        this.rules[key] = { pattern, message };
    }
}

// 模拟 TextBoxHelper 类
class TextBoxHelper {
    static SetInputRule(d: DependencyObject, value: string): void {
        // 这里可以实现具体的输入规则设置逻辑
        console.log(`Setting input rule: ${value}`);
    }
}

// 定义 InputRules 枚举
enum InputRules {
    None,
    Telephone,
    MobilePhone,
    Fax,
    NumberStr,
    PaymentSystem,
    NumLetter,
    NameLetter,
    RateLetter,
    NoChinese,
    TelephoneOrMobilePhone,
    DoubleNumBer,
    DateZM
}

// 定义 RegexRuleExtension 类
class RegexRuleExtension {
    private static flag = false;

    static readonly InputRuleProperty: DependencyProperty = DependencyProperty.registerAttached(
        "InputRule",
        InputRules,
        RegexRuleExtension,
        new UIPropertyMetadata(InputRules.None, RegexRuleExtension.OnInputRuleChanged)
    );

    static {
        this.LoadData();
    }

    static LoadData(): void {
        if (this.flag) return;
        this.flag = true;

        RegexValidationRules.Instance.AddRule(InputRules.None.toString(), "", "");
        RegexValidationRules.Instance.AddRule(InputRules.Telephone.toString(), @"^([0-9]{3,4}\-[0-9]{6,8}$)|(^\([0-9]{3,4}\)[0-9]{6,8}$)", "请输入正确的固定电话号码,电话加区号！");
        RegexValidationRules.Instance.AddRule(InputRules.Fax.toString(), @"^\s*$|(\(\d{3,4}\)|\d{3,4}-|\s)?\d{7}", "请输入正确的传真号码！");
        RegexValidationRules.Instance.AddRule(InputRules.MobilePhone.toString(), @"^1[0-9]{10}$", "请输入正确的手机号码，11位！");
        RegexValidationRules.Instance.AddRule(InputRules.NumberStr.toString(), @"^\s*$|^[0-9]*$", "请输入数字串！");
        RegexValidationRules.Instance.AddRule(InputRules.PaymentSystem.toString(), "^[^:%'\"]*$", "不允许使用支付系统规定的保留字！");
        RegexValidationRules.Instance.AddRule(InputRules.NumLetter.toString(), @"^\s*$|^[A-Za-z0-9]*$", "只允许输入数字和字母！");
        RegexValidationRules.Instance.AddRule(InputRules.NameLetter.toString(), @"^\s*$|^[a-zA-Z0-9\u4e00-\u9fa5]*$", "只允许输入数字、字母、中文！");
        RegexValidationRules.Instance.AddRule(InputRules[InputRules.DateZM], @"(^\d{4}(0[1-9]|1[0-2])$)|(^99999999$)|(^(?:(?:(?:(?:(?:1[6-9]|[2-9]\d)(?:0[48]|[2468][048]|[13579][26])|(?:(?:16|[2468][048]|[3579][26])00)))(?:0229))|(?:(?:(?:1[6-9]|[2-9]\d)\d{2})(?:(?:(?:0[13578]|1[02])31)|(?:(?:0[13-9]|1[0-2])(?:29|30))|(?:(?:0[1-9])|(?:1[0-2]))(?:0[1-9]|1\d|2[0-8]))))$)", "日期格式20120101或者201201，请重新输入。");
        RegexValidationRules.Instance.AddRule(InputRules.RateLetter.toString(), @"^(-?\d+)(\.\d+)?{0,6}$", "只允许输入浮点数,且小数点后6位");
        RegexValidationRules.Instance.AddRule(InputRules.NoChinese.toString(), @"^\s*$|[^\u4e00-\u9fa5]", "不允许输入中文！");
        RegexValidationRules.Instance.AddRule(InputRules.TelephoneOrMobilePhone.toString(), @"^([0-9]{3,4}\-[0-9]{6,8}$)|(^1[0-9]{10}$)|(^\([0-9]{3,4}\)[0-9]{6,8}$)", "手机号11位，电话需要增加区号！");
        RegexValidationRules.Instance.AddRule(InputRules.DoubleNumBer.toString(), @"^\d+(\.\d+)?$", "只允许输入整数或小数");
    }

    static GetInputType(d: DependencyObject): InputRules {
        return d.getValue(this.InputRuleProperty.name) as InputRules;
    }

    static SetInputType(d: DependencyObject, value: InputRules): void {
        d.setValue(this.InputRuleProperty.name, value);
    }

    static OnInputRuleChanged(d: DependencyObject, e: any): void {
        TextBoxHelper.SetInputRule(d, e.newValue.toString());
    }
}

export { RegexRuleExtension, InputRules };