// 由于 TypeScript 没有像 C# 那样直接的静态类概念，这里使用命名空间来模拟
namespace TellerSystem.Library.Ext.Variables {
    // 模拟 Encoding.Default 的功能
    const defaultEncoding = {
        getBytes(str: string): Uint8Array {
            return new TextEncoder().encode(str);
        },
        getString(bytes: Uint8Array, start: number, length: number): string {
            return new TextDecoder().decode(bytes.slice(start, start + length));
        }
    };

    // 模拟 string 扩展方法 ToDecimal
    (String.prototype as any).ToDecimal = function () {
        return parseFloat(this as string);
    };

    // 模拟 8583 报文相关的静态方法集合
    export class Feild8583 {
        /**
         * 将字符串拆成 byte[] 后，按字节截取，按照域的要求左对齐，空格补全
         * @param str 字符串
         * @param leng 字节长度
         * @returns 处理后的字符串
         */
        public static GetFdStrBytes(str: string, leng: number): string {
            // 格式为 leng 个 Unicode 字符，左对齐的字符串
            const strFd = str.padEnd(leng);
            // 转换为字节数组
            const strBytes = defaultEncoding.getBytes(strFd);
            // 截取 leng 个字节
            return defaultEncoding.getString(strBytes, 0, leng);
        }

        /**
         * 将字符串拆成 byte[] 后，按字节截取
         * 返回长度为 str.Length，count 中的小者
         * @param str 字符串
         * @param index 字节起始位置
         * @param count 字节长度
         * @returns 处理后的字符串
         */
        public static GetStrBytes(str: string, index: number, count: number): string {
            // 转换为字节数组
            const strBytes = defaultEncoding.getBytes(str);
            // 获取较小的长度
            const leng = Math.min(count, strBytes.length);
            // 截取 leng 个字节
            return defaultEncoding.getString(strBytes, index, leng);
        }

        /**
         * 将 Message.Fd101_ 赋值给 FD101 对象
         * @param target IPage 实例
         * @param msg Message 实例
         * @returns FD101 对象
         */
        public static GetFd101(target: IPage, msg: Message): FD101 {
            const fd101 = new FD101();
            fd101.ACTNO24 = msg.Fd101_1;
            fd101.ACTSEQ = msg.Fd101_2;
            fd101.RTXAMT = msg.Fd101_3;
            fd101.VOCTYPE = msg.Fd101_4;
            fd101.VOCNUM = msg.Fd101_5;
            fd101.DESC10 = msg.Fd101_6;
            fd101.NBFLAG = msg.Fd101_7;
            fd101.CUSTNAME = msg.Fd101_8;
            fd101.AVBAL = msg.Fd101_9;
            fd101.CURCD = msg.Fd101_A;
            fd101.CASHFLAG = msg.Fd101_B;
            fd101.DSCPT = msg.Fd101_C;
            fd101.ACTTYPE = msg.Fd101_D;
            fd101.ACTSEQ_2 = msg.Fd101_E;
            fd101.PRODNAME = msg.Fd101_F;
            fd101.MDMTYPE = msg.Fd101_G;
            return fd101;
        }

        /**
         * 为 Message.Fd101_ 赋值
         * @param target IPage 实例
         * @param msg Message 实例
         * @param fd101 FD101 对象
         */
        public static SetFD101(target: IPage, msg: Message, fd101: FD101): void {
            msg.Fd101_1 = fd101.ACTNO24;
            msg.Fd101_2 = fd101.ACTSEQ;
            msg.Fd101_3 = fd101.RTXAMT;
            msg.Fd101_4 = fd101.VOCTYPE;
            msg.Fd101_5 = fd101.VOCNUM;
            msg.Fd101_6 = fd101.DESC10;
            msg.Fd101_7 = fd101.NBFLAG;
            msg.Fd101_8 = fd101.CUSTNAME;
            msg.Fd101_9 = fd101.AVBAL;
            msg.Fd101_A = fd101.CURCD;
            msg.Fd101_B = fd101.CASHFLAG;
            msg.Fd101_C = fd101.DSCPT;
            msg.Fd101_D = fd101.ACTTYPE;
            msg.Fd101_E = fd101.ACTSEQ_2;
            msg.Fd101_F = fd101.PRODNAME;
            msg.Fd101_G = fd101.MDMTYPE;
        }

        /**
         * 为 Message.Fd102_ 赋值
         * @param target IPage 实例
         * @param msg Message 实例
         * @param fd102 FD102 对象
         */
        public static SetFD102(target: IPage, msg: Message, fd102: FD102): void {
            msg.Fd102_1 = fd102.ACTNO24;
            msg.Fd102_2 = fd102.ACTSEQ;
            msg.Fd102_3 = fd102.VOCTYPE;
            msg.Fd102_4 = fd102.VOCNUM;
            msg.Fd102_5 = fd102.FETCHFLAG;
            msg.Fd102_6 = fd102.PWDFLAG;
            msg.Fd102_7 = fd102.QUE_PWD;
            msg.Fd102_8 = fd102.PAY_PWD;
            msg.Fd102_9 = fd102.CERTFLAG;
            msg.Fd102_A = fd102.ID_NO;
            msg.Fd102_B = fd102.STAMPFLAG;
            msg.Fd102_C = fd102.PWDFLAG2;
            msg.Fd102_D = fd102.PWD16;
            msg.Fd102_E = fd102.PRINTDATE;
            msg.Fd102_F = fd102.TXAMT15;
            msg.Fd102_G = fd102.CERTTYPE;
            msg.Fd102_H = fd102.CUSTNAME;
            msg.Fd102_I = fd102.AVBAL;
            msg.Fd102_J = fd102.CURCD;
            msg.Fd102_K = fd102.CT_IND;
            msg.Fd102_L = fd102.DSCPT;
            msg.Fd102_M = fd102.ACTTYPE;
            msg.Fd102_N = fd102.ACTSEQ_2;
            msg.Fd102_O = fd102.ACCOUNTFLAG;
            msg.Fd102_P = fd102.INTEREST1;
            msg.Fd102_Q = fd102.INTEREST2;
            msg.Fd102_R = fd102.INTEREST3;
            msg.Fd102_S = fd102.PAYAMOUNT;
            msg.Fd102_T = fd102.NEWVOCNUM;
            msg.Fd102_U = fd102.SPEFLAG;
            msg.Fd102_V = fd102.PRODNAME;
            msg.Fd102_W = fd102.ACTNO_24;
            msg.Fd102_X = fd102.EDUCREDIT;
            msg.Fd102_Y = fd102.DESC10;
            msg.Fd102_Z = fd102.MDMCODE;
        }

        /**
         * 将 Message.Fd102_ 赋值给 FD102 对象
         * @param target IPage 实例
         * @param msg Message 实例
         * @returns FD102 对象
         */
        public static GetFD102(target: IPage, msg: Message): FD102 {
            const fd102 = new FD102();
            fd102.ACTNO24 = msg.Fd102_1;
            fd102.ACTSEQ = msg.Fd102_2;
            fd102.VOCTYPE = msg.Fd102_3;
            fd102.VOCNUM = msg.Fd102_4;
            fd102.FETCHFLAG = msg.Fd102_5;
            fd102.PWDFLAG = msg.Fd102_6;
            fd102.QUE_PWD = msg.Fd102_7;
            fd102.PAY_PWD = msg.Fd102_8;
            fd102.CERTFLAG = msg.Fd102_9;
            fd102.ID_NO = msg.Fd102_A;
            fd102.STAMPFLAG = msg.Fd102_B;
            fd102.PWDFLAG2 = msg.Fd102_C;
            fd102.PWD16 = msg.Fd102_D;
            fd102.PRINTDATE = msg.Fd102_E;
            fd102.TXAMT15 = msg.Fd102_F;
            fd102.CERTTYPE = msg.Fd102_G;
            fd102.CUSTNAME = msg.Fd102_H;
            fd102.AVBAL = msg.Fd102_I;
            fd102.CURCD = msg.Fd102_J;
            fd102.CT_IND = msg.Fd102_K;
            fd102.DSCPT = msg.Fd102_L;
            fd102.ACTTYPE = msg.Fd102_M;
            fd102.ACTSEQ_2 = msg.Fd102_N;
            fd102.ACCOUNTFLAG = msg.Fd102_O;
            fd102.INTEREST1 = msg.Fd102_P;
            fd102.INTEREST2 = msg.Fd102_Q;
            fd102.INTEREST3 = msg.Fd102_R;
            fd102.PAYAMOUNT = msg.Fd102_S;
            fd102.NEWVOCNUM = msg.Fd102_T;
            fd102.SPEFLAG = msg.Fd102_U;
            fd102.PRODNAME = msg.Fd102_V;
            fd102.ACTNO_24 = msg.Fd102_W;
            fd102.EDUCREDIT = msg.Fd102_X;
            fd102.DESC10 = msg.Fd102_Y;
            fd102.MDMCODE = msg.Fd102_Z;

            return fd102;
        }

        /**
         * 获取 FD102 域字符串
         * (未公开 Message 前的笨方法 add slzh)
         * @param target IPage 实例
         * @param msg Message 实例
         * @returns FD102 域字符串
         */
        public static GetFd102(target: IPage, msg: Message): string {
            /* string.Substring 截取的是 四个字节 unicode ，Fd102_H 截取出来的是 63 个字节 */
            // Encoding.Default.GetString(Encoding.Default.GetBytes(strLineConten), 0, 20);

            return (
                this.GetFdStrBytes(msg.Fd102_1, 24) +
                this.GetFdStrBytes(msg.Fd102_2, 6) +
                this.GetFdStrBytes(msg.Fd102_3, 3) +
                this.GetFdStrBytes(msg.Fd102_4, 16) +
                this.GetFdStrBytes(msg.Fd102_5, 1) +
                this.GetFdStrBytes(msg.Fd102_6, 1) +
                this.GetFdStrBytes(msg.Fd102_7, 16) +
                this.GetFdStrBytes(msg.Fd102_8, 16) +
                this.GetFdStrBytes(msg.Fd102_9, 1) +
                this.GetFdStrBytes(msg.Fd102_A, 20) +
                this.GetFdStrBytes(msg.Fd102_B, 1) +
                this.GetFdStrBytes(msg.Fd102_C, 1) +
                this.GetFdStrBytes(msg.Fd102_D, 16) +
                this.GetFdStrBytes(msg.Fd102_E, 8) +
                this.GetFdStrBytes((msg.Fd102_F.ToDecimal() * 100).toString(), 16) +
                this.GetFdStrBytes(msg.Fd102_G, 1) +
                // + this.GetFdStrBytes(msg.Fd102_H, 60)
                this.GetFdStrBytes(msg.Fd102_H, 120) +
                this.GetFdStrBytes((msg.Fd102_I.ToDecimal() * 100).toString(), 16) +
                this.GetFdStrBytes(msg.Fd102_J, 2) +
                this.GetFdStrBytes(msg.Fd102_K, 1) +
                this.GetFdStrBytes(msg.Fd102_L, 3) +
                this.GetFdStrBytes(msg.Fd102_M, 1) +
                this.GetFdStrBytes(msg.Fd102_N, 6) +
                this.GetFdStrBytes(msg.Fd102_O, 1) +
                this.GetFdStrBytes((msg.Fd102_P.ToDecimal() * 100).toString(), 16) +
                this.GetFdStrBytes((msg.Fd102_Q.ToDecimal() * 100).toString(), 16) +
                this.GetFdStrBytes((msg.Fd102_R.ToDecimal() * 100).toString(), 16) +
                this.GetFdStrBytes((msg.Fd102_S.ToDecimal() * 100).toString(), 16) +
                this.GetFdStrBytes(msg.Fd102_T, 16) +
                this.GetFdStrBytes(msg.Fd102_U, 20) +
                this.GetFdStrBytes(msg.Fd102_V, 30) +
                this.GetFdStrBytes(msg.Fd102_W, 24) +
                this.GetFdStrBytes(msg.Fd102_X, 1) +
                this.GetFdStrBytes(msg.Fd102_Y, 20)
                // + this.GetFdStrBytes(msg.Fd102_Z, 4)
            );
        }

        /**
         * 组 123 域  大额
         * @param target IPage 实例
         * @param fd123 hvFd123 对象
         * @returns 123 域字符串
         */
        public static GetHvFd123(target: IPage, fd123: hvFd123): string {
            return (
                this.GetFdStrBytes(fd123.CMTNO, 3) +
                this.GetFdStrBytes(fd123.O_CMTNO, 3) +
                this.GetFdStrBytes(fd123.PAY_QS_NO, 12) +
                this.GetFdStrBytes(fd123.OR_BR_NO, 12) +
                this.GetFdStrBytes(fd123.SENDCO, 4) +
                this.GetFdStrBytes(fd123.PAY_OPN_BR_NO, 12) +
                this.GetFdStrBytes(fd123.PAY_AC_NO, 32) +
                this.GetFdStrBytes(fd123.PAY_NAME, 60) +
                this.GetFdStrBytes(fd123.PAY_ADDR, 60) +
                this.GetFdStrBytes(fd123.CASH_QS_NO, 12) +
                this.GetFdStrBytes(fd123.AC_BR_NO, 12) +
                this.GetFdStrBytes(fd123.RECECO, 4) +
                this.GetFdStrBytes(fd123.CASH_OPN_BR_NO, 12) +
                this.GetFdStrBytes(fd123.CASH_AC_NO, 32) +
                this.GetFdStrBytes(fd123.CASH_NAME, 60) +
                this.GetFdStrBytes(fd123.CASH_ADDR, 60) +
                this.GetFdStrBytes(fd123.YW_TYPE, 2) +
                this.GetFdStrBytes(fd123.ORDERNO, 8) +
                this.GetFdStrBytes(fd123.CONTENT, 60) +
                this.GetFdStrBytes(((fd123.RATE.ToDecimal() * 100000)).toString(), 7) +
                this.GetFdStrBytes(fd123.RATEDATE, 5) +
                this.GetFdStrBytes(fd123.OORDERNO, 8) +
                this.GetFdStrBytes(fd123.PROCODE, 8) +
                this.GetFdStrBytes(fd123.CR_DATE, 8) +
                this.GetFdStrBytes(fd123.CR_BR_NO, 12) +
                this.GetFdStrBytes(fd123.CR_TX_NUM, 8) +
                this.GetFdStrBytes(fd123.QR_BR_NO, 12) +
                this.GetFdStrBytes(fd123.QR_TX_NUM, 8) +
                this.GetFdStrBytes(fd123.TX_TYPE, 1) +
                this.GetFdStrBytes(fd123.NOTPAY_CONTENT, 255) +
                this.GetFdStrBytes(fd123.NOTPAY_ORDERNO, 3) +
                this.GetFdStrBytes(fd123.O_NOTPAY_ORDERNO, 3) +
                this.GetFdStrBytes(fd123.RESP1, 1) +
                this.GetFdStrBytes(fd123.OPERLEVEL, 1) +
                this.GetFdStrBytes(fd123.PROC_STS, 1) +
                this.GetFdStrBytes(fd123.OPCD, 2) +
                this.GetFdStrBytes(fd123.LW_IND, 1) +
                this.GetFdStrBytes(fd123.HP_ADD_PWD, 1) +
                this.GetFdStrBytes((fd123.PACKID || '').padStart(8, '0'), 8) +
                this.GetFdStrBytes((fd123.O_PACKID || '').padStart(8, '0'), 8) +
                this.GetFdStrBytes((fd123.PACK_DATE || '').padStart(8, '0'), 8) +
                this.GetFdStrBytes((fd123.O_PACK_DATE || '').padStart(8, '0'), 8) +
                this.GetFdStrBytes((fd123.TXNUM || '').padStart(5, '0'), 5) +
                this.GetFdStrBytes((fd123.O_TXNUM || '').padStart(5, '0'), 5) +
                this.GetFdStrBytes(fd123.LV_YW_IND, 12) +
                this.GetFdStrBytes((fd123.RESP_DATE || '').padStart(8, '0'), 8) +
                this.GetFdStrBytes((fd123.RCP_STS || '').padStart(2, '0'), 2) +
                this.GetFdStrBytes(fd123.RESP2, 8) +
                this.GetFdStrBytes((fd123.RETCODE || '').padStart(2, '0'), 2) +
                this.GetFdStrBytes((fd123.YW_DTL_ID || '').padStart(8, '0'), 8) +
                this.GetFdStrBytes((fd123.ACC_IND || '').padStart(1, '0'), 1) +
                this.GetFdStrBytes((fd123.REQ_TYPE || '').padStart(1, '0'), 1) +
                this.GetFdStrBytes(((fd123.DF_FREEAMT.ToDecimal() * 100)).toString(), 16) +
                this.GetFdStrBytes(fd123.PROTNO, 60) +
                this.GetFdStrBytes((fd123.PO_NO || '').padStart(8, '0'), 8) +
                this.GetFdStrBytes((fd123.PO_TYPE || '').padStart(1, '0'), 1) +
                this.GetFdStrBytes((fd123.OTYPE || '').padStart(1, '0'), 1) +
                this.GetFdStrBytes(fd123.CR_TX_DATE, 8) +
                this.GetFdStrBytes(fd123.CR_TX_TLNO, 6) +
                this.GetFdStrBytes(fd123.CR_TX_TRCE, 8)
            );
        }

        /**
         * 拆 123 域  大额
         * @param target IPage 实例
         * @param FD123 123 域字符串
         * @returns hvFd123 对象
         */
        public static SetHvFd123(target: IPage, FD123: string): hvFd123 {
            const fd123_tmp = new hvFd123();
            FD123 = this.GetFdStrBytes(FD123, 999); // 补齐 999 位
            fd123_tmp.CMTNO = this.GetStrBytes(FD123, 0, 3);
            fd123_tmp.O_CMTNO = this.GetStrBytes(FD123, 3, 3);
            fd123_tmp.PAY_QS_NO = this.GetStrBytes(FD123, 6, 12);
            fd123_tmp.OR_BR_NO = this.GetStrBytes(FD123, 18, 12);
            fd123_tmp.SENDCO = this.GetStrBytes(FD123, 30, 4);
            fd123_tmp.PAY_OPN_BR_NO = this.GetStrBytes(FD123, 34, 12);
            fd123_tmp.PAY_AC_NO = this.GetStrBytes(FD123, 46, 32);
            fd123_tmp.PAY_NAME = this.GetStrBytes(FD123, 78, 60);
            fd123_tmp.PAY_ADDR = this.GetStrBytes(FD123, 138, 60);
            fd123_tmp.CASH_QS_NO = this.GetStrBytes(FD123, 198, 12);
            fd123_tmp.AC_BR_NO = this.GetStrBytes(FD123, 210, 12);
            fd123_tmp.RECECO = this.GetStrBytes(FD123, 222, 4);
            fd123_tmp.CASH_OPN_BR_NO = this.GetStrBytes(FD123, 226, 12);
            fd123_tmp.CASH_AC_NO = this.GetStrBytes(FD123, 238, 32);
            fd123_tmp.CASH_NAME = this.GetStrBytes(FD123, 270, 60);
            fd123_tmp.CASH_ADDR = this.GetStrBytes(FD123, 330, 60);
            fd123_tmp.YW_TYPE = this.GetStrBytes(FD123, 390, 2);
            fd123_tmp.ORDERNO = this.GetStrBytes(FD123, 392, 8);
            fd123_tmp.CONTENT = this.GetStrBytes(FD123, 400, 60);
            fd123_tmp.RATE = (this.GetStrBytes(FD123, 460, 7).ToDecimal() * 0.00001).toString();
            fd123_tmp.RATEDATE = this.GetStrBytes(FD123, 467, 5);
            fd123_tmp.OORDERNO = this.GetStrBytes(FD123, 472, 8);
            fd123_tmp.PROCODE = this.GetStrBytes(FD123, 480, 8);
            fd123_tmp.CR_DATE = this.GetStrBytes(FD123, 488, 8);
            fd123_tmp.CR_BR_NO = this.GetStrBytes(FD123, 496, 12);
            fd123_tmp.CR_TX_NUM = this.GetStrBytes(FD123, 508, 8);
            fd123_tmp.QR_BR_NO = this.GetStrBytes(FD123, 516, 12);
            fd123_tmp.QR_TX_NUM = this.GetStrBytes(FD123, 528, 8);
            fd123_tmp.TX_TYPE = this.GetStrBytes(FD123, 536, 1);
            fd123_tmp.NOTPAY_CONTENT = this.GetStrBytes(FD123, 537, 255);
            fd123_tmp.NOTPAY_ORDERNO = this.GetStrBytes(FD123, 792, 3);
            fd123_tmp.O_NOTPAY_ORDERNO = this.GetStrBytes(FD123, 795, 3);
            fd123_tmp.RESP1 = this.GetStrBytes(FD123, 798, 1);
            fd123_tmp.OPERLEVEL = this.GetStrBytes(FD123, 799, 1);
            fd123_tmp.PROC_STS = this.GetStrBytes(FD123, 800, 1);
            fd123_tmp.OPCD = this.GetStrBytes(FD123, 801, 2);
            fd123_tmp.LW_IND = this.GetStrBytes(FD123, 803, 1);
            fd123_tmp.HP_ADD_PWD = this.GetStrBytes(FD123, 804, 1);
            fd123_tmp.PACKID = this.GetStrBytes(FD123, 805, 8);
            fd123_tmp.O_PACKID = this.GetStrBytes(FD123, 813, 8);
            fd123_tmp.PACK_DATE = this.GetStrBytes(FD123, 821, 8);
            fd123_tmp.O_PACK_DATE = this.GetStrBytes(FD123, 829, 8);
            fd123_tmp.TXNUM = this.GetStrBytes(FD123, 837, 5);
            fd123_tmp.O_TXNUM = this.GetStrBytes(FD123, 842, 5);
            fd123_tmp.LV_YW_IND = this.GetStrBytes(FD123, 847, 12);
            fd123_tmp.RESP_DATE = this.GetStrBytes(FD123, 859, 8);
            fd123_tmp.RCP_STS = this.GetStrBytes(FD123, 867, 2);
            fd123_tmp.RESP2 = this.GetStrBytes(FD123, 869, 8);
            fd123_tmp.RETCODE = this.GetStrBytes(FD123, 877, 2);
            fd123_tmp.YW_DTL_ID = this.GetStrBytes(FD123, 879, 8);
            fd123_tmp.ACC_IND = this.GetStrBytes(FD123, 887, 1);
            fd123_tmp.REQ_TYPE = this.GetStrBytes(FD123, 888, 1);
            fd123_tmp.DF_FREEAMT = (this.GetStrBytes(FD123, 889, 16).ToDecimal() * 0.01).toString();
            fd123_tmp.PROTNO = this.GetStrBytes(FD123, 905, 60);
            fd123_tmp.PO_NO = this.GetStrBytes(FD123, 965, 8);
            fd123_tmp.PO_TYPE = this.GetStrBytes(FD123, 973, 1);
            fd123_tmp.OTYPE = this.GetStrBytes(FD123, 974, 1);
            fd123_tmp.CR_TX_DATE = this.GetStrBytes(FD123, 975, 8);
            fd123_tmp.CR_TX_TLNO = this.GetStrBytes(FD123, 983, 6);
            fd123_tmp.CR_TX_TRCE = this.GetStrBytes(FD123, 989, 8);

            return fd123_tmp;
        }

        /**
         * 组 123 域  小额
         * @param target IPage 实例
         * @param fd123 lvFd123 对象
         * @returns 123 域字符串
         */
        public static GetLvFd123(target: IPage, fd123: lvFd123): string {
            return (
                this.GetFdStrBytes(fd123.CRDB, 1) +
                this.GetFdStrBytes((fd123.OPCD || '').padStart(2, '0'), 2) +
                this.GetFdStrBytes((fd123.PKGNO || '').padStart(3, '0'), 3) +
                this.GetFdStrBytes((fd123.ORDERNO || '').padStart(8, '0'), 8) +
                this.GetFdStrBytes(fd123.TXNUM, 5) +
                this.GetFdStrBytes(fd123.OR_BR_NO, 12) +
                this.GetFdStrBytes(fd123.AC_BR_NO, 12) +
                this.GetFdStrBytes(((fd123.TX_AMT.ToDecimal() * 100)).toString(), 15) +
                this.GetFdStrBytes(fd123.PAY_OPN_BR_NO, 12) +
                this.GetFdStrBytes(fd123.PAY_QS_NO, 12) +
                this.GetFdStrBytes(fd123.PAY_AC_NO, 32) +
                this.GetFdStrBytes(fd123.PAY_NAME, 60) +
                this.GetFdStrBytes(fd123.PAY_ADDR, 60) +
                this.GetFdStrBytes(fd123.PWD, 8) +
                this.GetFdStrBytes(fd123.CASH_QS_NO, 12) +
                this.GetFdStrBytes(fd123.CASH_OPN_BR_NO, 12) +
                this.GetFdStrBytes(fd123.CASH_AC_NO, 32) +
                this.GetFdStrBytes(fd123.CASH_NAME, 60) +
                this.GetFdStrBytes(fd123.CASH_ADDR, 60) +
                this.GetFdStrBytes(fd123.YW_TYPE, 12) +
                this.GetFdStrBytes(fd123.BRF, 60) +
                this.GetFdStrBytes(fd123.WT_DATE, 8) +
                this.GetFdStrBytes(fd123.OPERLEVEL, 1) +
                this.GetFdStrBytes(fd123.AUTH_TEL, 6) +
                this.GetFdStrBytes(fd123.AUTH_PWD, 8) +
                this.GetFdStrBytes(fd123.NOTE_TYPE, 2) +
                this.GetFdStrBytes(fd123.NOTE_NO, 12) +
                this.GetFdStrBytes(fd123.PACK_DATE, 8) +
                this.GetFdStrBytes(fd123.PACK_ID, 8) +
                this.GetFdStrBytes(fd123.CHRG_IND, 1) +
                this.GetFdStrBytes(((fd123.CHRG_AMT.ToDecimal() * 100)).toString(), 15) +
                this.GetFdStrBytes(fd123.RESP_DAY, 8) +
                this.GetFdStrBytes(fd123.PO_TYPE, 2) +
                this.GetFdStrBytes(fd123.PO_DATE, 8) +
                this.GetFdStrBytes(fd123.PO_NUM, 8) +
                this.GetFdStrBytes(((fd123.PC_TX_AMT.ToDecimal() * 100)).toString(), 15) +
                this.GetFdStrBytes(((fd123.JF_TX_AMT.ToDecimal() * 100)).toString(), 15) +
                this.GetFdStrBytes(fd123.O_WT_DATE, 8) +
                this.GetFdStrBytes(fd123.O_TXNUM, 5) +
                this.GetFdStrBytes(fd123.O_ORDERNO, 8) +
                this.GetFdStrBytes(fd123.RET_CODE, 2) +
                this.GetFdStrBytes(fd123.RCP_STS, 2) +
                this.GetFdStrBytes(fd123.RESP1, 8) +
                this.GetFdStrBytes(fd123.RESP2, 8) +
                this.GetFdStrBytes(fd123.PACK_END, 1) +
                this.GetFdStrBytes(fd123.LV_STS, 1) +
                this.GetFdStrBytes(fd123.LW_IND, 1) +
                this.GetFdStrBytes(fd123.DFXYNO, 60) +
                this.GetFdStrBytes(fd123.OORBRNO, 12) +
                this.GetFdStrBytes(fd123.OACBRNO, 12) +
                this.GetFdStrBytes(fd123.PAY_AC_NO1, 32) +
                this.GetFdStrBytes((fd123.CR_TX_DATE || '').padStart(8, '0'), 8) +
                this.GetFdStrBytes(fd123.CR_TX_TLNO, 6) +
                this.GetFdStrBytes((fd123.CR_TX_TRCE || '').padStart(8, '0'), 8)
            );
        }

        /**
         * 拆 123 域  小额
         * @param target IPage 实例
         * @param FD123 123 域字符串
         * @returns lvFd123 对象
         */
        public static SetLvFd123(target: IPage, FD123: string): lvFd123 {
            const fd123_tmp = new lvFd123();
            FD123 = this.GetFdStrBytes(FD123, 999); // 补齐 999 位
            fd123_tmp.CRDB = this.GetStrBytes(FD123, 0, 1);
            fd123_tmp.OPCD = this.GetStrBytes(FD123, 1, 2);
            fd123_tmp.PKGNO = this.GetStrBytes(FD123, 3, 3);
            fd123_tmp.ORDERNO = this.GetStrBytes(FD123, 6, 8);
            fd123_tmp.TXNUM = this.GetStrBytes(FD123, 14, 5);
            fd123_tmp.OR_BR_NO = this.GetStrBytes(FD123, 19, 12);
            fd123_tmp.AC_BR_NO = this.GetStrBytes(FD123, 31, 12);
            fd123_tmp.TX_AMT = (this.GetStrBytes(FD123, 43, 15).ToDecimal() * 0.01).toString();
            fd123_tmp.PAY_OPN_BR_NO = this.GetStrBytes(FD123, 58, 12);
            fd123_tmp.PAY_QS_NO = this.GetStrBytes(FD123, 70, 12);
            fd123_tmp.PAY_AC_NO = this.GetStrBytes(FD123, 82, 32);
            fd123_tmp.PAY_NAME = this.GetStrBytes(FD123, 114, 60);
            fd123_tmp.PAY_ADDR = this.GetStrBytes(FD123, 174, 60);
            fd123_tmp.PWD = this.GetStrBytes(FD123, 234, 8);
            fd123_tmp.CASH_QS_NO = this.GetStrBytes(FD123, 242, 12);
            fd123_tmp.CASH_OPN_BR_NO = this.GetStrBytes(FD123, 254, 12);
            fd123_tmp.CASH_AC_NO = this.GetStrBytes(FD123, 266, 32);
            fd123_tmp.CASH_NAME = this.GetStrBytes(FD123, 298, 60);
            fd123_tmp.CASH_ADDR = this.GetStrBytes(FD123, 358, 60);
            fd123_tmp.YW_TYPE = this.GetStrBytes(FD123, 418, 12);
            fd123_tmp.BRF = this.GetStrBytes(FD123, 430, 60);
            fd123_tmp.WT_DATE = this.GetStrBytes(FD123, 490, 8);
            fd123_tmp.OPERLEVEL = this.GetStrBytes(FD123, 498, 1);
            fd123_tmp.AUTH_TEL = this.GetStrBytes(FD123, 499, 6);
            fd123_tmp.AUTH_PWD = this.GetStrBytes(FD123, 505, 8);
            fd123_tmp.NOTE_TYPE = this.GetStrBytes(FD123, 513, 2);
            fd123_tmp.NOTE_NO = this.GetStrBytes(FD123, 515, 12);
            fd123_tmp.PACK_DATE = this.GetStrBytes(FD123, 527, 8);
            fd123_tmp.PACK_ID = this.GetStrBytes(FD123, 535, 8);
            fd123_tmp.CHRG_IND = this.GetStrBytes(FD123, 543, 1);
            fd123_tmp.CHRG_AMT = (this.GetStrBytes(FD123, 544, 15).ToDecimal() * 0.01).toString();
            fd123_tmp.RESP_DAY = this.GetStrBytes(FD123, 559, 8);
            fd123_tmp.PO_TYPE = this.GetStrBytes(FD123, 567, 2);
            fd123_tmp.PO_DATE = this.GetStrBytes(FD123, 569, 8);
            fd123_tmp.PO_NUM = this.GetStrBytes(FD123, 577, 8);
            fd123_tmp.PC_TX_AMT = (this.GetStrBytes(FD123, 585, 15).ToDecimal() * 0.01).toString();
            fd123_tmp.JF_TX_AMT = (this.GetStrBytes(FD123, 600, 15).ToDecimal() * 0.01).toString();
            fd123_tmp.O_WT_DATE = this.GetStrBytes(FD123, 615, 8);
            fd123_tmp.O_TXNUM = this.GetStrBytes(FD123, 623, 5);
            fd123_tmp.O_ORDERNO = this.GetStrBytes(FD123, 628, 8);
            fd123_tmp.RET_CODE = this.GetStrBytes(FD123, 636, 2);
            fd123_tmp.RCP_STS = this.GetStrBytes(FD123, 638, 2);
            fd123_tmp.RESP1 = this.GetStrBytes(FD123, 640, 8);
            fd123_tmp.RESP2 = this.GetStrBytes(FD123, 648, 8);
            fd123_tmp.PACK_END = this.GetStrBytes(FD123, 656, 1);
            fd123_tmp.LV_STS = this.GetStrBytes(FD123, 657, 1);
            fd123_tmp.LW_IND = this.GetStrBytes(FD123, 658, 1);
            fd123_tmp.DFXYNO = this.GetStrBytes(FD123, 659, 60);
            fd123_tmp.OORBRNO = this.GetStrBytes(FD123, 719, 12);
            fd123_tmp.OACBRNO = this.GetStrBytes(FD123, 731, 12);
            fd123_tmp.PAY_AC_NO1 = this.GetStrBytes(FD123, 743, 32);
            fd123_tmp.CR_TX_DATE = this.GetStrBytes(FD123, 775, 8);
            fd123_tmp.CR_TX_TLNO = this.GetStrBytes(FD123, 783, 6);
            fd123_tmp.CR_TX_TRCE = this.GetStrBytes(FD123, 789, 8);

            return fd123_tmp;
        }

        /**
         * 组 125 域  授权
         * @param strFd125 125 域字符串
         * @returns AuthFd125 对象
         */
        public static SetAuthFd125(strFd125: string): AuthFd125 {
            const authFd125 = new AuthFd125();
            authFd125.DeAccount = this.GetStrBytes(strFd125, 0, 24);
            authFd125.CrAccount = this.GetStrBytes(strFd125, 24, 24);
            authFd125.CTSigns = this.GetStrBytes(strFd125, 48, 1);
            authFd125.DeAccountType = this.GetStrBytes(strFd125, 49, 1);
            authFd125.CrAccountType = this.GetStrBytes(strFd125, 50, 1);
            authFd125.DeAccountPro = this.GetStrBytes(strFd125, 51, 1);
            authFd125.CrAccountPro = this.GetStrBytes(strFd125, 52, 1);
            authFd125.DeIsOwn = this.GetStrBytes(strFd125, 53, 1);
            authFd125.CrIsOwn = this.GetStrBytes(strFd125, 54, 1);
            authFd125.DCIsSameCus = this.GetStrBytes(strFd125, 55, 1);
            authFd125.SinAmt = (this.GetStrBytes(strFd125, 56, 16).ToDecimal() * 0.01).toString();
            authFd125.DeCaAddAmt = (this.GetStrBytes(strFd125, 72, 16).ToDecimal() * 0.01).toString();
            authFd125.DeTrAddAmt = (this.GetStrBytes(strFd125, 88, 16).ToDecimal() * 0.01).toString();
            authFd125.LossMarked = this.GetStrBytes(strFd125, 152, 1);
            authFd125.DocumentType = this.GetStrBytes(strFd125, 154, 3);
            authFd125.DeductMarked = this.GetStrBytes(strFd125, 160, 2);
            authFd125.LongOrShort = this.GetStrBytes(strFd125, 164, 1);
            authFd125.ReMarked = this.GetStrBytes(strFd125, 168, 8);
            authFd125.DeOpnBrNo = this.GetStrBytes(strFd125, 176, 5);
            authFd125.CrOpnBrNo = this.GetStrBytes(strFd125, 181, 5);
            authFd125.CrTrAddAmt = (this.GetStrBytes(strFd125, 186, 16).ToDecimal() * 0.01).toString();

            return authFd125;
        }
    }

    // 定义相关的接口和类
    export interface IPage {
        // 这里可以根据实际情况补充 IPage 接口的具体属性和方法
    }

    export class Message {
        Fd101_1: string;
        Fd101_2: string;
        Fd101_3: string;
        Fd101_4: string;
        Fd101_5: string;
        Fd101_6: string;
        Fd101_7: string;
        Fd101_8: string;
        Fd101_9: string;
        Fd101_A: string;
        Fd101_B: string;
        Fd101_C: string;
        Fd101_D: string;
        Fd101_E: string;
        Fd101_F: string;
        Fd101_G: string;
        Fd102_1: string;
        Fd102_2: string;
        Fd102_3: string;
        Fd102_4: string;
        Fd102_5: string;
        Fd102_6: string;
        Fd102_7: string;
        Fd102_8: string;
        Fd102_9: string;
        Fd102_A: string;
        Fd102_B: string;
        Fd102_C: string;
        Fd102_D: string;
        Fd102_E: string;
        Fd102_F: string;
        Fd102_G: string;
        Fd102_H: string;
        Fd102_I: string;
        Fd102_J: string;
        Fd102_K: string;
        Fd102_L: string;
        Fd102_M: string;
        Fd102_N: string;
        Fd102_O: string;
        Fd102_P: string;
        Fd102_Q: string;
        Fd102_R: string;
        Fd102_S: string;
        Fd102_T: string;
        Fd102_U: string;
        Fd102_V: string;
        Fd102_W: string;
        Fd102_X: string;
        Fd102_Y: string;
        Fd102_Z: string;
        // 这里可以根据实际情况补充 Message 类的其他属性和方法
    }

    /**
     * 存款交易结构体
     */
    export class FD101 {
        /**
         * 1011  DATA_TYPE="0" DATA_LENG="24"  DATA_DEC="0" 账号
         */
        ACTNO24: string;

        /**
         * 1012  DATA_TYPE="1" DATA_LENG="4"   DATA_DEC="0" 账户序号
         */
        ACTSEQ: string;

        /**
         * 1013  DATA_TYPE="4" DATA_LENG="16"  DATA_DEC="2" 金额
         */
        RTXAMT: string;

        /**
         * 1014  DATA_TYPE="0" DATA_LENG="3"   DATA_DEC="0" 凭证种类
         */
        VOCTYPE: string;

        /**
         * 1015  DATA_TYPE="0" DATA_LENG="16"  DATA_DEC="0" 凭证号
         */
        VOCNUM: string;

        /**
         * 1016  DATA_TYPE="0" DATA_LENG="20"  DATA_DEC="0" 摘要
         */
        DESC10: string;

        /**
         * 1017  DATA_TYPE="0" DATA_LENG="1"   DATA_DEC="0" 有折无折
         */
        NBFLAG: string;

        /**
         * 1018  DATA_TYPE="0" DATA_LENG="60"  DATA_DEC="0" 户名
         */
        CUSTNAME: string;

        /**
         * 1019  DATA_TYPE="4" DATA_LENG="16"  DATA_DEC="2" 余额
         */
        AVBAL: string;

        /**
         * 101A  DATA_TYPE="0" DATA_LENG="2"   DATA_DEC="0" 币种
         */
        CURCD: string;

        /**
         * 101B  DATA_TYPE="0" DATA_LENG="1"   DATA_DEC="0" 现转标志
         */
        CASHFLAG: string;

        /**
         * 101C  DATA_TYPE="0" DATA_LENG="3"   DATA_DEC="0" 摘要选项
         */
        DSCPT: string;

        /**
         * 101D  DATA_TYPE="0" DATA_LENG="1"   DATA_DEC="0" 帐户类型
         */
        ACTTYPE: string;

        /**
         * 101E  DATA_TYPE="1" DATA_LENG="4"   DATA_DEC="0" 账户序号2
         */
        ACTSEQ_2: string;

        /**
         * 101F  DATA_TYPE="0" DATA_LENG="30"  DATA_DEC="0" 产品名称
         */
        PRODNAME: string;

        /**
         * 101G  DATA_TYPE="0" DATA_LENG="4"   DATA_DEC="0" 介质种类
         */
        MDMTYPE: string;
    }

    /**
     * 取款交易结构体
     */
    export class FD102 {
        /**
         * 1021  DATA_TYPE="0"  DATA_LENG="24  DATA_DEC="0" 账号
         */
        ACTNO24: string;

        /**
         * 1022  DATA_TYPE="1"  DATA_LENG="4"  DATA_DEC="0" 账户序号
         */
        ACTSEQ: string;

        /**
         * 1023  DATA_TYPE="0"  DATA_LENG="3"  DATA_DEC="0" 凭证种类
         */
        VOCTYPE: string;

        /**
         * 1024  DATA_TYPE="0"  DATA_LENG="16" DATA_DEC="0" 凭证号
         */
        VOCNUM: string;

        /**
         * 1025  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 是否任意支取
         */
        FETCHFLAG: string;

        /**
         * 1026  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 是否凭密
         */
        PWDFLAG: string;

        /**
         * 1027  DATA_TYPE="6"  DATA_LENG="16" DATA_DEC="0" 查询密码
         */
        QUE_PWD: string;

        /**
         * 1028  DATA_TYPE="6"  DATA_LENG="16" DATA_DEC="0" 支取密码
         */
        PAY_PWD: string;

        /**
         * 1029  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 是否凭证件支取
         */
        CERTFLAG: string;

        /**
         * 102A  DATA_TYPE="0"  DATA_LENG="20" DATA_DEC="0" 证件号码
         */
        ID_NO: string;

        /**
         * 102B  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 是否凭印鉴支取
         */
        STAMPFLAG: string;

        /**
         * 102C  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 是否支付密码器
         */
        PWDFLAG2: string;

        /**
         * 102D  DATA_TYPE="0"  DATA_LENG="16" DATA_DEC="0" 支付密码
         */
        PWD16: string;

        /**
         * 102E  DATA_TYPE="1"  DATA_LENG="8"  DATA_DEC="0" 出票日期
         */
        PRINTDATE: string;

        /**
         * 102F  DATA_TYPE="4"  DATA_LENG="16" DATA_DEC="2" 金额
         */
        TXAMT15: string;

        /**
         * 102G  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 证件类型
         */
        CERTTYPE: string;

        /**
         * 102H  DATA_TYPE="0"  DATA_LENG="60" DATA_DEC="0" 户名
         */
        CUSTNAME: string;

        /**
         * 102I  DATA_TYPE="4"  DATA_LENG="16" DATA_DEC="2" 余额
         */
        AVBAL: string;

        /**
         * 102J  DATA_TYPE="0"  DATA_LENG="2"  DATA_DEC="0" 币种
         */
        CURCD: string;

        /**
         * 102K  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 现转标志
         */
        CT_IND: string;

        /**
         * 102L  DATA_TYPE="0"  DATA_LENG="3"  DATA_DEC="0" 摘要选项
         */
        DSCPT: string;

        /**
         * 102M  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 帐户类型
         */
        ACTTYPE: string;

        /**
         * 102N  DATA_TYPE="1"  DATA_LENG="4"  DATA_DEC="0" 账户序号2
         */
        ACTSEQ_2: string;

        /**
         * 102O  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 打折标志
         */
        ACCOUNTFLAG: string;

        /**
         * 102P  DATA_TYPE="4"  DATA_LENG="16" DATA_DEC="2" 应付利息
         */
        INTEREST1: string;

        /**
         * 102Q  DATA_TYPE="4"  DATA_LENG="16" DATA_DEC="2" 利息税
         */
        INTEREST2: string;

        /**
         * 102R  DATA_TYPE="4"  DATA_LENG="16" DATA_DEC="2" 保值利息
         */
        INTEREST3: string;

        /**
         * 102S  DATA_TYPE="4"  DATA_LENG="16" DATA_DEC="2" 应付总额
         */
        PAYAMOUNT: string;

        /**
         * 102T  DATA_TYPE="0"  DATA_LENG="16" DATA_DEC="0" 新凭证号
         */
        NEWVOCNUM: string;

        /**
         * 102U  DATA_TYPE="0"  DATA_LENG="20" DATA_DEC="0" 特殊标志
         */
        SPEFLAG: string;

        /**
         * 102V  DATA_TYPE="0"  DATA_LENG="30" DATA_DEC="0" 产品名称
         */
        PRODNAME: string;

        /**
         * 102W  DATA_TYPE="0"  DATA_LENG="24" DATA_DEC="0" 账号
         */
        ACTNO_24: string;

        /**
         * 102X  DATA_TYPE="0"  DATA_LENG="1"  DATA_DEC="0" 教育证明
         */
        EDUCREDIT: string;

        /**
         * 102Y  DATA_TYPE="0"  DATA_LENG="20" DATA_DEC="0" 摘要
         */
        DESC10: string;

        /**
         * 102Z  DATA_TYPE="0"  DATA_LENG="4"  DATA_DEC="0" 介质种类
         */
        MDMCODE: string;
    }

    /**
     * 大额类业务
     * 将 123 域划分子域
     */
    export class hvFd123 {
        /**
         * Cmt 编号
         */
        CMTNO: string;

        /**
         * 原 Cmt 编号
         */
        O_CMTNO: string;

        /**
         * 发起清算行
         */
        PAY_QS_NO: string;

        /**
         * 发起行行号
         */
        OR_BR_NO: string;

        /**
         * 发报中心代码
         */
        SENDCO: string;

        /**
         * 付款人开户行
         */
        PAY_OPN_BR_NO: string;

        /**
         * 记账账号
         */
        PAY_AC_NO: string;

        /**
         * 付款人名称
         */
        PAY_NAME: string;

        /**
         * 付款人地址
         */
        PAY_ADDR: string;

        /**
         * 接收清算行
         */
        CASH_QS_NO: string;

        /**
         * 接收行行号,原发起行
         */
        AC_BR_NO: string;

        /**
         * 收报中心代码
         */
        RECECO: string;

        /**
         * 收款人开户行
         */
        CASH_OPN_BR_NO: string;

        /**
         * 收款人帐号
         */
        CASH_AC_NO: string;

        /**
         * 收款人姓名
         */
        CASH_NAME: string;

        /**
         * 收款人地址
         */
        CASH_ADDR: string;

        /**
         * 业务种类描述
         */
        YW_TYPE: string;

        /**
         * 支付交易序号
         */
        ORDERNO: string;

        /**
         * 备注
         */
        CONTENT: string;

        /**
         * 拆借利率
         */
        RATE: string;

        /**
         * 拆借期限
         */
        RATEDATE: string;

        /**
         * 原支付交易序号
         */
        OORDERNO: string;

        /**
         * 处理码
         */
        PROCODE: string;

        /**
         * 查复日期
         */
        CR_DATE: string;

        /**
         * 查复行号
         */
        CR_BR_NO: string;

        /**
         * 查复书号
         */
        CR_TX_NUM: string;

        /**
         * 查询行号  原查询行号
         */
        QR_BR_NO: string;

        /**
         * 原查询书号 查询书号
         */
        QR_TX_NUM: string;

        /**
         * 原交易种类
         */
        TX_TYPE: string;

        /**
         * 退回备注
         */
        NOTPAY_CONTENT: string;

        /**
         * 退回申请号
         */
        NOTPAY_ORDERNO: string;

        /**
         * 原退回申请号
         */
        O_NOTPAY_ORDERNO: string;

        /**
         * Mbfe 应答码
         */
        RESP1: string;

        /**
         * 优先级别 0、一般；1、紧急；2、特急
         */
        OPERLEVEL: string;

        /**
         * 处理状态
         */
        PROC_STS: string;

        /**
         * 操作代码
         */
        OPCD: string;

        /**
         * 来往标识 1-往 2-来
         */
        LW_IND: string;

        /**
         * 汇票是否核押
         */
        HP_ADD_PWD: string;

        /**
         * 包序号
         */
        PACKID: string;

        /**
         * 原包序号
         */
        O_PACKID: string;

        /**
         * 包日期,定期用
         */
        PACK_DATE: string;

        /**
         * 原包日期
         */
        O_PACK_DATE: string;

        /**
         * 业务编号
         */
        TXNUM: string;

        /**
         * 原业务编号
         */
        O_TXNUM: string;

        /**
         * 业务类型编号
         */
        LV_YW_IND: string;

        /**
         * 应答日期
         */
        RESP_DATE: string;

        /**
         * 业务回执状态
         */
        RCP_STS: string;

        /**
         * Ccpc 应答码
         */
        RESP2: string;

        /**
         * 退汇原因
         */
        RETCODE: string;

        /**
         * 明细序号
         */
        YW_DTL_ID: string;

        /**
         * 实时业务付款人帐户类型
         */
        ACC_IND: string;

        /**
         * 申请类型 0＝单笔  1＝整包
         */
        REQ_TYPE: string;

        /**
         * 对方手续费
         */
        DF_FREEAMT: string;

        /**
         * 多方协议号
         */
        PROTNO: string;

        /**
         * 汇票号码
         */
        PO_NO: string;

        /**
         * 汇票类型
         */
        PO_TYPE: string;

        /**
         * 1-大额，2－小额
         */
        OTYPE: string;

        /**
         * 存款交易日期
         */
        CR_TX_DATE: string;

        /**
         * 存款交易柜员
         */
        CR_TX_TLNO: string;

        /**
         * 存款交易流水
         */
        CR_TX_TRCE: string;
    }

    /**
     * 小额业务 123 域划分
     */
    export class lvFd123 {
        /**
         * 借贷标志 0 借 1 贷
         */
        CRDB: string;

        /**
         * 操作代码  10－录入之录入 11－录入之查询 20－修改之修改 21－修改之查询 30－复核 40－发送之发送
         *            41－发送之查询 50－应答 60－接收 70－错账处理之入帐
         *            71－错账处理之修改 72－错账处理之查询 80 - 冲来帐 90--冲往帐
         */
        OPCD: string;

        /**
         * Package 编号
         */
        PKGNO: string;

        /**
         * 支付交易序号
         */
        ORDERNO: string;

        /**
         * 业务类型编号
         */
        TXNUM: string;

        /**
         * 发起行行号
         */
        OR_BR_NO: string;

        /**
         * 接收行行号
         */
        AC_BR_NO: string;

        /**
         * 交易金额
         */
        TX_AMT: string;

        /**
         * 付款人开户行
         */
        PAY_OPN_BR_NO: string;

        /**
         * 付款清算行行号
         */
        PAY_QS_NO: string;

        /**
         * 付款人帐号
         */
        PAY_AC_NO: string;

        /**
         * 付款人名称
         */
        PAY_NAME: string;

        /**
         * 付款人地址
         */
        PAY_ADDR: string;

        /**
         * 储户密码
         */
        PWD: string;

        /**
         * 收款清算行行号
         */
        CASH_QS_NO: string;

        /**
         * 收款人开户行行号
         */
        CASH_OPN_BR_NO: string;

        /**
         * 收款人帐号
         */
        CASH_AC_NO: string;

        /**
         * 收款人名称
         */
        CASH_NAME: string;

        /**
         * 收款人地址
         */
        CASH_ADDR: string;

        /**
         * 业务种类
         */
        YW_TYPE: string;

        /**
         * 附言
         */
        BRF: string;

        /**
         * 委托日期
         */
        WT_DATE: string;

        /**
         * 优先级
         */
        OPERLEVEL: string;

        /**
         * 授权操作员号
         */
        AUTH_TEL: string;

        /**
         * 授权操作员密码
         */
        AUTH_PWD: string;

        /**
         * 凭证种类
         */
        NOTE_TYPE: string;

        /**
         * 凭证号码
         */
        NOTE_NO: string;

        /**
         * 包日期,定期用
         */
        PACK_DATE: string;

        /**
         * 包 id,定期用
         */
        PACK_ID: string;

        /**
         * 手续费标志
         */
        CHRG_IND: string;

        /**
         * 手续费金额
         */
        CHRG_AMT: string;

        /**
         * 期待应答日期
         */
        RESP_DAY: string;

        /**
         * 票据类型
         */
        PO_TYPE: string;

        /**
         * 票据日期
         */
        PO_DATE: string;

        /**
         * 票据号码
         */
        PO_NUM: string;

        /**
         * 赔偿金额
         */
        PC_TX_AMT: string;

        /**
         * 拒付金额
         */
        JF_TX_AMT: string;

        /**
         * 原委托日期
         */
        O_WT_DATE: string;

        /**
         * 原业务编号
         */
        O_TXNUM: string;

        /**
         * 原支付交易序号
         */
        O_ORDERNO: string;

        /**
         * 退汇原因代码
         */
        RET_CODE: string;

        /**
         * 业务回执状态
         */
        RCP_STS: string;

        /**
         * Mbfe 应答码
         */
        RESP1: string;

        /**
         * Ccpc 应答码
         */
        RESP2: string;

        /**
         * 包结束标志
         */
        PACK_END: string;

        /**
         * 交易状态
         */
        LV_STS: string;

        /**
         * 来往标识
         */
        LW_IND: string;

        /**
         * 多方协议号
         */
        DFXYNO: string;

        /**
         * 原发起行行号
         */
        OORBRNO: string;

        /**
         * 原接收行行号
         */
        OACBRNO: string;

        /**
         * 付款账户
         */
        PAY_AC_NO1: string;

        /**
         * 记账帐户存款交易日期
         */
        CR_TX_DATE: string;

        /**
         * 记账帐户存款交易柜员
         */
        CR_TX_TLNO: string;

        /**
         * 记账帐户存款交易流水
         */
        CR_TX_TRCE: string;
    }

    /**
     * 授权要素 125 域划分
     */
    export class AuthFd125 {
        /**
         * 借方账号
         */
        DeAccount: string;

        /**
         * 借方账号序号
         */
        DeAccountSeqn: string;

        /**
         * 贷方账号
         */
        CrAccount: string;

        /**
         * 2、  CTSigns:转账/现金标志
         * 值 ：1：转账；2：现金
         */
        CTSigns: string;

        /**
         * 3、  DeAccountType:借方账户类型（对公、对私、内部）
         * 值 ：1：对公；2：对私；3：内部
         */
        DeAccountType: string;

        /**
         * 4、  CrAccountType:贷方账户类型（对公、对私、内部）
         * 值 ：1：对公；2：对私；3：内部
         */
        CrAccountType: string;

        /**
         * 5、  DeAccountPro:借方账户属性（基本户、一般户、专用户、临时户）
         * 值 ：1：基本户；2：一般户；3：专用户；4：临时户
         */
        DeAccountPro: string;

        /**
         * 6、  CrAccountPro:贷方账户属性（基本户、一般户、专用户、临时户）
         * 值 ：1：基本户；2：一般户；3：专用户；4：临时户
         */
        CrAccountPro: string;

        /**
         * 7、  DeIsOwn:借方是否是本行账号
         * 值 ：Y：是；N：否
         */
        DeIsOwn: string;

        /**
         * 8、  CrIsOwn:贷方是否是本行账号
         * 值 ：Y：是；N：否
         */
        CrIsOwn: string;

        /**
         * 9、  DCIsSameCus:借记方和贷记方是否是同一客户
         * 值 ：Y：是；N：否
         */
        DCIsSameCus: string;

        /**
         * 10、  SinAmt:单笔金额
         * 值 ：金额，如：100000.00
         */
        SinAmt: string;

        /**
         * 11、DeCaAddAmt:当日借方现金累计金额
         * 类型：数字类型
         * 值 ：金额，如：100000.00
         */
        DeCaAddAmt: string;

        /**
         * 12、DeTrAddAmt:当日借方转账累计金额
         * 类型：数字类型
         * 值 ：金额，如：100000.00
         */
        DeTrAddAmt: string;

        /**
         * 13、LossMarked:挂失标志
         * 类型：布尔类型
         * 值 ：Y：是；N：否
         */
        LossMarked: string;

        /**
         * 14、DocumentType：凭证种类
         * 类型：字符类型
         */
        DocumentType: string;

        /**
         * 15、DeductMarked：扣划标志
         * 类型：字符类型
         */
        DeductMarked: string;

        /**
         * 16、LongOrShort：长短款标志
         * 类型：布尔类型
         * 值 ：Y：是；N：否
         */
        LongOrShort: string;

        /**
         * 备用标志
         */
        ReMarked: string;

        /**
         * 借方账户开户行号
         */
        DeOpnBrNo: string;

        /**
         * 贷方账户开户行号
         */
        CrOpnBrNo: string;

        /**
         * 当日贷方现金累计金额
         */
        CrTrAddAmt: string;

        /**
         * 备注
         */
        Remarks: string;
    }
}

// 导出命名空间中的类和接口
export {
    TellerSystem
};