﻿using Platform.Common.LogSystem;
using Platform.Common.RunningParameters;
using Platform.Print;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Printing;
using System.Text;
using System.Windows;
using TellerSystem.Communication;
using TellerSystem.Communication.MessageHelper;
using TellerSystem.PrintTemplate.Extension;
using TellerSystem.ServiceProxy.Ext;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;
using TellerSystem.PrintTemplate.Base;
using TellerSystem.PrintTemplate.ElecSeal.Cfca;
using Spire.Pdf;
using Spire.Pdf.Graphics;
using System.Text.RegularExpressions;

namespace TellerSystem.PrintTemplate.ElecSeal
{
    /// <summary>
    /// 签章模式
    /// </summary>
    public enum SealMode
    {
        /// <summary>
        /// 无
        /// </summary>
        None,
        /// <summary>
        /// 本地资源加签
        /// </summary>
        Local,
        /// <summary>
        /// cfca加签
        /// </summary>
        Cfca,
        /// <summary>
        /// 无纸化
        /// </summary>
        Paperless,
    }

    /// <summary>
    /// 签章类型
    /// </summary>
    public enum SealKind
    {
        /// <summary>
        /// 业务章
        /// </summary>
        Biz,
        /// <summary>
        /// 回单章
        /// </summary>
        Receipt,
        /// <summary>
        /// 对账章
        /// </summary>
        Verify,
    }

    public class ElecSealHelper
    {
        #region Ctor
        public static SealMode _MODE_ = ServiceProxy.Ext.ServiceSettings.GetConfig("ElecSeal.Mode", x =>
        {
            SealMode mode;
            if (Enum.TryParse(x, out mode))
            {
                //若无纸化情况下，检测当前机构匹配情况
                if (mode == SealMode.Paperless && !Regex.IsMatch(LoginUserInfo.KinbrNo, ServiceSettings.GetConfig("PaperlessBranchMatch", "")))
                    return SealMode.None;
                return mode;
            }
            return SealMode.None;
        });
        #endregion

        #region Helper
        internal static ElecSealHelper GetHelper(DependencyObject obj)
        {
            return (ElecSealHelper)obj.GetValue(HelperProperty);
        }

        internal static void SetHelper(DependencyObject obj, ElecSealHelper value)
        {
            obj.SetValue(HelperProperty, value);
        }

        // Using a DependencyProperty as the backing store for Helper.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty HelperProperty =
            DependencyProperty.RegisterAttached("Helper", typeof(ElecSealHelper), typeof(PrintContainer), new PropertyMetadata(null));
        #endregion

        /// <summary>
        /// 采用的签章模型
        /// </summary>
        public SealMode Mode { get { return _MODE_; } }
        public Stopwatch watch = new Stopwatch();
        public Stopwatch totaltime = new Stopwatch();
        public string tmplog = string.Empty;
        /// <summary>
        /// 签章的位置集合
        /// </summary>
        private Dictionary<string, Tuple<Point, SealKind>> _points = new Dictionary<string, Tuple<Point, SealKind>>();

        /// <summary>
        /// 指定位置加盖电子印章，若kind=null,则代表取消
        /// </summary>
        /// <param name="p"></param>
        /// <param name="kind"></param>
        /// <returns></returns>
        public string Sign2Point(Point p, SealKind? kind = SealKind.Biz)
        {
            var key = string.Format("[{0},{1}]", p.X, p.Y);
            if (kind.HasValue)
                _points[key] = new Tuple<Point, SealKind>(p, ServiceSettings.GetConfig("ElecSeal.All2Biz", true) ? SealKind.Biz : kind.Value);
            else
                _points.Remove(key);
            return key;
        }

        /// <summary>
        /// 关键字
        /// </summary>
        public string KeyCode { get; private set; }

        #region GetCode
        public static string GetCode(PrintAttributes p)
        {
            var business = p.BusinessTransaction;
            var defV = "None";
            if (!string.IsNullOrEmpty(p.SystemSerialNumber))//采用对系统流水做16进制转换
                defV = Convert.ToString(Convert.ToInt64(p.SystemSerialNumber), 16).ToUpper();
            //无纸化模式下，采用新的编码方案
            if (_MODE_ == SealMode.Paperless)
            {
                return defV;
            }
            try
            {
                var _msg = new Message();
                _msg.Init();
                CommSend(_msg, business);
                _msg.Fd16 = "8001";
                _msg.Fd18 = "DZQZ";
                _msg.Fd37 = "CIBAS";
                CallAgn(_msg, "CallAgn");
                if (_msg.Fd12 == "0000")
                {
                    return _msg.Fd27.Trim();
                }
                else
                {
                    TradeHandle.WriteImportantLog("电子印章编码错误", "通讯返回12域内容：" + _msg.Fd12, _msg.Fd16, "DoTrade", "1", string.Join(",", TellerSystem.Communication.SocketListener.SocketManager.GetLocalIpAddressList()));
                }
            }
            catch (Exception ex)
            {
                Platform.Common.LogSystem.PlatformLogger.SystemErrorInfo("ElecSealHelper.GetCode发生异常！", ex);
            }
            return defV;
        }

        /// <summary>
        /// 初始化公共域
        /// </summary>
        /// <param name="message"></param>
        static void CommSend(Message message, string business)
        {
            //开户行机构号
            message.Fd2 = LoginUserInfo.KinbrNo;// LoginUserInfo.Opnbr;
            //机构号
            message.Fd3 = LoginUserInfo.KinbrNo;
            //业务流水
            message.Fd4 = business;// this.PrintAttribute.BusinessTransaction;
            //交易日期
            message.Fd5 = LoginUserInfo.TradeDate;
            //发生时间
            message.Fd6 = DateTime.Now.ToString("HHmmss");
            //柜员号
            message.Fd7 = LoginUserInfo.TellerNo;
            //终端号
            message.Fd10 = LoginUserInfo.TtyName;
            message.Fd96_9 = "0100"; //本行柜面
            message.Fd96_7 = "0";
            message.Fd96_8 = "0";
            message.Fd96_C = "0100"; //本行柜面
            message.Fd96_D = LoginUserInfo.TradeDate;
            message.Fd96_E = message.Fd6;
        }

        /// <summary>
        /// 和平台通讯API
        /// </summary>
        /// <param name="message"></param>
        /// <param name="transitType">TransitType枚举值</param>
        /// <returns></returns>
        static bool CallAgn(Message message, string transitType)
        {
            var returnValue = false;
            string preTradeNo = message.Fd16;
            message.TransitNode = TransitType.CallAgnRun;
            message.BitMapType = TellerSystem.Communication.Message.BitMap.Agn;
            returnValue = message.DoTransit();
            if (message.Fd16 != preTradeNo && message.Fd12 == "0000")
            {
                message.Fd12 = "COMN";
            }
            return returnValue;
        }
        #endregion

        /// <summary>
        /// 创建电子印章的列表
        /// </summary>
        /// <returns></returns>
        internal Dictionary<Point, UIElement> CreateElecSealList(PrintAttributes p)
        {
            var ret = new Dictionary<Point, UIElement>();
            if (Mode == SealMode.None || _points.Count == 0) return ret;
            if (string.IsNullOrEmpty(KeyCode))
                KeyCode = GetCode(p);
            //增加一个隐藏控件，用于登记KeyCode
            ret.Add(new Point(0, 0), new System.Windows.Controls.TextBlock { FontSize = 1, Text = "KeyCode=>" + KeyCode });
            _points.ToList().ForEach(x =>
            {
                var codekey = GenKeyCode(x.Value.Item2);
                var seal = new E_Seal(codekey, Mode == SealMode.Local);
                if (codekey.EndsWith("-R") || Mode == SealMode.Paperless)//无纸化模式下，也默认不显示签章编码
                {
                    seal.Display = false;
                }
                ret.Add(x.Value.Item1, seal);
            });
            return ret;
        }

        private string GenKeyCode(SealKind kind)
        {
            var code = KeyCode;
            switch (kind)
            {
                case SealKind.Biz:
                    code += "-B";
                    break;
                case SealKind.Receipt:
                    code += "-R";
                    break;
                case SealKind.Verify:
                    code += "-V";
                    break;
                default:
                    break;
            }
            return code;
        }

        /// <summary>
        /// 是否需要加签，打印时；暂仅支持cfca
        /// </summary>
        internal bool NeedSignature
        {
            get
            {
                return Mode == SealMode.Cfca && _points.Count > 0;
            }
        }

        /// <summary>
        /// 处理打印的xps文件,执行加签操作
        /// </summary>
        /// <param name="xps">原始打印文件,如加签成功，则会更新该文件</param>
        /// <returns>处理结果</returns>
        internal bool Process2Signature(FileInfo xps)
        {
            watch.Start();
            totaltime.Start();
            tmplog += "##**加签开始: ";
            if (!NeedSignature) return false;
            byte[] data = null;
            try
            {
                data = File.ReadAllBytes(xps.FullName);
                //xps转换成pdf
                data = XpsConvert(data);
                watch.Stop();
                tmplog += "##**XPS转换成PDF:" + watch.ElapsedMilliseconds;
                //按照印章的类型进行多次加签操作
                var kinds = _points.GroupBy(x => x.Value.Item2).Select(x => x.Key).ToList();
                switch (Mode)
                {
                    case SealMode.Cfca:
                        data = PdfSeal_Cfca(data, kinds);
                        break;
                    default:
                        throw new NotSupportedException("暂不支持的印章模型：" + Mode);
                }
                //pdf转回xps
                watch.Restart();
                data = XpsConvert(data, false);
                watch.Stop();
                tmplog += "##**PDF转换成XPS:" + watch.ElapsedMilliseconds;
                totaltime.Stop();
                tmplog += "##**凭证总耗时:" + totaltime.ElapsedMilliseconds;
                TradeHandle.WriteImportantLog("无纸化耗时日志", tmplog, "无纸化凭证打印");
                tmplog = "";
                File.WriteAllBytes(xps.FullName, data);
                return true;
            }
            catch (Exception ex)
            {
                PlatformLogger.SystemErrorInfo("xps加电子印章发生错误！", ex);
            }
            finally
            {
                //转换后的数据 不上传
                //try
                //{   //xps 文件
                //    if (data != null)
                //    {
                //        //上传文件到服务器
                //        var backup = string.Format(@"/sealpdf/{0}_{1}_{2}", LoginUserInfo.TradeDate, LoginUserInfo.TellerNo, KeyCode);
                //        Platform.ServiceProxy.ServiceHelper.FilesHandle.SaveFileData(backup, data, false);
                //    }
                //}
                //catch (Exception ex)
                //{
                //    PlatformLogger.SystemErrorInfo("xps加电子印章上传失败！", ex);
                //}
            }
            return false;
        }

        private byte[] PdfSeal_Cfca(byte[] pdf, List<SealKind> kinds)
        {
            if (pdf == null || pdf.Length == 0 || kinds == null || kinds.Count == 0) throw new ArgumentException("PdfSeal_Cfca:参数异常，无法进行有效处理！");
            if (kinds.Count == 1) return PdfSeal_Cfca(pdf, kinds[0]);
            try
            {
                //一次性盖多个章，改用复合签章接口
                PlatformLogger.SystemInfo("ElecSealHelper.PdfSeal_Cfca:进入加签！复合签章");
                watch.Restart();
                var list = kinds.Select(kind =>
                {
                    var keyCode = GenKeyCode(kind);
                    string[] cp = null;
                    if (kind == SealKind.Receipt)
                    {
                        cp = ServiceSettings.GetConfig("ElecSeal.Cfca." + LoginUserInfo.KinbrNo.Substring(0, 3) + "." + kind.ToString()).Split('|');
                    }
                    else
                    {
                        cp = ServiceSettings.GetConfig("ElecSeal.Cfca." + LoginUserInfo.KinbrNo + "." + kind.ToString()).Split('|');
                    }
                    var sealCode = cp[0];
                    var sealPassword = cp[1];
                    SealUserInfo sealUserInfo = new SealUserInfo(LoginUserInfo.TellerNo, LoginUserInfo.KinbrNo, "");
                    // 3=关键字签章
                    SignLocation signLocation = null;
                    if (kind == SealKind.Biz)
                    {
                        signLocation = new SignLocation(keyCode, "C", 0, 20);
                    }
                    else
                    {
                        signLocation = new SignLocation(keyCode, "C", 0, 0);
                    }
                    // 生成签章策略文件
                    string sealStrategyXml = CommonMethod.CreateSealStrategyXml(sealCode, sealPassword, "", "font-size:13;x-ratio:0.0;y-ratio:0.0",
                            sealUserInfo, signLocation);
                    return sealStrategyXml;
                });
                //获取参数
                var url_prefix = ServiceSettings.GetConfig("ElecSeal.Cfca.Url_Prefix");
                var channelCode = ServiceSettings.GetConfig("ElecSeal.Cfca.ChannelCode");
                var operatorCode = ServiceSettings.GetConfig("ElecSeal.Cfca.OperatorCode_" + LoginUserInfo.KinbrNo.Substring(0, 3));
                PaperlessClient paperlessClient = new PaperlessClient();
                var pdfBeanXml = CommonMethod.BuildPdfBeanXml(new PdfBean(Convert.ToBase64String(pdf)));
                var multiData = "";
                String compoundSealStrategyXml = "<List>" + string.Join("", list) + "</List>";
                String channelBeanXml = @"<Channel>
                                        <ChannelCode>" + channelCode + @"</ChannelCode>
                                        <MerchantNo></MerchantNo>
                                        <MerchantName></MerchantName>
                                        <TerminalNo></TerminalNo>
                                    </Channel>";
                watch.Stop();
                tmplog += "  多签章通讯前数据准备完成:" + watch.ElapsedMilliseconds;
                watch.Restart();
                PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:多签章通讯开始！");
                var resultByte = paperlessClient.CompoundSealAutoPdf(url_prefix, pdfBeanXml, multiData, compoundSealStrategyXml, operatorCode, channelBeanXml);
                PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:多签章通讯结束！");
                watch.Stop();
                tmplog += "  多签章通讯:" + watch.ElapsedMilliseconds;
                if (CommonMethod.CheckResultDataValid(resultByte))
                {
                    //登记日志  交易编号，交易时间，柜员号，是否加签成功，pdf名称，保存路径，关键字 机构号， 渠道编码，签章编码，操作人
                    //var sql = string.Format("INSERT INTO FINGER_PRINTS (TELLER_NO, FINGER_L1, FINGER_L2, FINGER_L3, FINGER_L4, FINGER_L5, FINGER_R1, FINGER_R2, FINGER_R3, FINGER_R4, FINGER_R5, CREATE_TIME, UPDATE_TIME) VALUES ('{0}', '{1}', '{2}', '{3}', '{4}', '{5}','{6}', '{7}', '{8}','{9}', '{10}', SYSDATE, '')",                       

                    //    TellerNo.Text.Trim(), arrayFinger[1], arrayFinger[2], arrayFinger[3], arrayFinger[4], arrayFinger[5], arrayFinger[6], arrayFinger[7], arrayFinger[8], arrayFinger[9], arrayFinger[10]);
                    //result = UserManagerHandle.ExecuteSql(sql);
                    try
                    {
                        //上传加签后PDF 文件
                        if (resultByte != null)
                        {
                            //上传文件到服务器
                            var backup = string.Format(@"/sealpdf/{0}_{1}_{2}", LoginUserInfo.TradeDate, LoginUserInfo.TellerNo, KeyCode + ".PDF");
                            watch.Restart();
                            PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:开始上传服务器！" + KeyCode);
                            //Platform.ServiceProxy.ServiceHelper.FilesHandle.SaveFileData(backup, resultByte, false);
                            PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:上传完成！" + KeyCode);
                            watch.Stop();
                            tmplog += "  多次签章后数据上传至服务器:" + watch.ElapsedMilliseconds + " keycode" + KeyCode;
                        }
                    }
                    catch (Exception ex)
                    {
                        PlatformLogger.SystemErrorInfo("xps加电子印章上传失败！", ex);
                    }
                }
                else
                {
                    string result = Encoding.UTF8.GetString(resultByte);
                    TradeHandle.WriteImportantLog("电子印章", result, "交易");
                    PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:发生异常，电子印章加签异常！");
                }
                return resultByte;
            }
            catch (Exception ex)
            {
                PlatformLogger.SystemErrorInfo("ElecSealHelper.PrintPdf:发生异常，不支持pdf打印！", ex);
                return null;
            }
        }

        /// <summary>
        /// pdf加签操作
        /// </summary>
        /// <param name="xps"></param>
        /// <param name="keyCode"></param>
        /// <param name="bizCode"></param>
        /// <returns></returns>
        private byte[] PdfSeal_Cfca(byte[] pdf, SealKind kind)
        {
            watch.Restart();
            var keyCode = GenKeyCode(kind);
            byte[] resultByte = null;
            PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:进入加签！");
            //获取参数
            var url_prefix = ServiceSettings.GetConfig("ElecSeal.Cfca.Url_Prefix");
            var channelCode = ServiceSettings.GetConfig("ElecSeal.Cfca.ChannelCode");
            var operatorCode = string.Empty;
            //测试运河

            operatorCode = ServiceSettings.GetConfig("ElecSeal.Cfca.OperatorCode_" + LoginUserInfo.KinbrNo.Substring(0, 3));
            string[] cp = null;
            if (kind.ToString() == "Receipt")
            {
                cp = ServiceSettings.GetConfig("ElecSeal.Cfca." + LoginUserInfo.KinbrNo.Substring(0, 3) + "." + kind.ToString()).Split('|');
            }
            else
            {
                cp = ServiceSettings.GetConfig("ElecSeal.Cfca." + LoginUserInfo.KinbrNo + "." + kind.ToString()).Split('|');

            }
            var sealCode = cp[0];
            var sealPassword = cp[1];

            //pdf加签
            try
            {
                PaperlessClient paperlessClient = new PaperlessClient();
                SealUserInfo sealUserInfo = new SealUserInfo(LoginUserInfo.TellerNo, LoginUserInfo.KinbrNo, "");

                // 3=关键字签章
                SignLocation signLocation = null;
                if (kind.ToString() == "Biz")
                {
                    signLocation = new SignLocation(keyCode, "C", 0, 20);
                }
                else
                {
                    signLocation = new SignLocation(keyCode, "C", 0, 0);
                }

                // 生成签章策略文件
                string sealStrategyXml = CommonMethod.CreateSealStrategyXml(sealCode, sealPassword, "", "font-size:13;x-ratio:0.0;y-ratio:0.0",
                        sealUserInfo, signLocation);
                watch.Stop();
                tmplog += "  签章通讯前数据准备完成:" + watch.ElapsedMilliseconds;
                watch.Restart();
                PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:签章通讯开始！" );
                resultByte = paperlessClient.SealAutoPdf(url_prefix, pdf, sealStrategyXml, operatorCode, channelCode);
                PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:签章通讯结束！" );
                watch.Stop();
                tmplog += "  签章通讯:" + watch.ElapsedMilliseconds;
                if (CommonMethod.CheckResultDataValid(resultByte))
                {
                    //登记日志  交易编号，交易时间，柜员号，是否加签成功，pdf名称，保存路径，关键字 机构号， 渠道编码，签章编码，操作人
                    //var sql = string.Format("INSERT INTO FINGER_PRINTS (TELLER_NO, FINGER_L1, FINGER_L2, FINGER_L3, FINGER_L4, FINGER_L5, FINGER_R1, FINGER_R2, FINGER_R3, FINGER_R4, FINGER_R5, CREATE_TIME, UPDATE_TIME) VALUES ('{0}', '{1}', '{2}', '{3}', '{4}', '{5}','{6}', '{7}', '{8}','{9}', '{10}', SYSDATE, '')",                       

                    //    TellerNo.Text.Trim(), arrayFinger[1], arrayFinger[2], arrayFinger[3], arrayFinger[4], arrayFinger[5], arrayFinger[6], arrayFinger[7], arrayFinger[8], arrayFinger[9], arrayFinger[10]);
                    //result = UserManagerHandle.ExecuteSql(sql);
                    try
                    {
                        //上传加签后PDF 文件
                        if (resultByte != null)
                        {
                            //上传文件到服务器
                            var backup = string.Format(@"/sealpdf/{0}_{1}_{2}", LoginUserInfo.TradeDate, LoginUserInfo.TellerNo, KeyCode + ".PDF");
                            watch.Restart();
                            PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:开始上传服务器！" + KeyCode);
                            //Platform.ServiceProxy.ServiceHelper.FilesHandle.SaveFileData(backup, resultByte, false);
                            PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:上传完成！" + KeyCode);
                            watch.Stop();
                            tmplog += "  签章后数据上传至服务器:" + watch.ElapsedMilliseconds + " keycode" + KeyCode;
                        }
                    }
                    catch (Exception ex)
                    {
                        PlatformLogger.SystemErrorInfo("xps加电子印章上传失败！", ex);
                    }
                }
                else
                {
                    string result = Encoding.UTF8.GetString(resultByte);
                    TradeHandle.WriteImportantLog("电子印章", result, "交易");
                    PlatformLogger.SystemInfo("ElecSealHelper.PrintPdf:发生异常，电子印章加签异常！");
                }
                return resultByte;
            }
            catch (Exception ex)
            {

                PlatformLogger.SystemErrorInfo("ElecSealHelper.PrintPdf:发生异常，不支持pdf打印！", ex);
                return null;
            }
            //return ElecSeal_Cfca.Helper.PdfSeal(url_prefix, channelCode, operatorCode, pdfStream.ToArray(), keyCode, sealCode, sealPassword, bizCode);
        }

        /// <summary>
        /// 触发补打操作
        /// </summary>
        /// <param name="xps"></param>
        public static bool Reprint(FileInfo xps, string bizNo)
        {
            var helper = new ElecSealHelper();
            var title = string.Empty;
            var size = new Size(0, 0);
            if (helper.Mode != SealMode.None && helper.Mode != SealMode.Local)
            {
                //分析补打文件，捕获加签的位置
                var xpsDocument = new System.Windows.Xps.Packaging.XpsDocument(xps.FullName, FileAccess.Read);
                try
                {
                    //获取凭证参数
                    title = xpsDocument.CoreDocumentProperties.Title;
                    //解析xps文件
                    var reader = xpsDocument.FixedDocumentSequenceReader;
                    var page = reader.FixedDocuments[0].FixedPages[0];
                    var xrdr = page.XmlReader;
                    while (xrdr.Read())
                    {
                        switch (xrdr.NodeType)
                        {
                            case System.Xml.XmlNodeType.Element:
                                //命中页面，提取凭证尺寸
                                if (xrdr.Name == "FixedPage")
                                {
                                    size.Width = double.Parse(xrdr["Width"]);
                                    size.Height = double.Parse(xrdr["Height"]);
                                }
                                else if (xrdr.Name == "Glyphs")
                                {
                                    var key = xrdr["UnicodeString"];
                                    //命中隐藏编号，提取KeyCode
                                    if (key.StartsWith("KeyCode=>"))
                                    {
                                        helper.KeyCode = key.Substring(9);
                                    }
                                    //命中印章编号，分析印章位置和类型
                                    else if (helper.KeyCode != null && key.StartsWith(helper.KeyCode))
                                    {
                                        var point = new Point(double.Parse(xrdr["OriginX"]), double.Parse(xrdr["OriginY"]));
                                        //判断SealKind
                                        var kind = SealKind.Biz;
                                        if (key.EndsWith("-R"))
                                            kind = SealKind.Receipt;
                                        else if (key.EndsWith("-V"))
                                            kind = SealKind.Verify;
                                        helper.Sign2Point(point, kind);
                                    }
                                }
                                break;
                            default:
                                break;
                        }
                    }
                }
                finally
                {
                    xpsDocument.Close();
                }
            }
            //补打是否需要走加签流程
            var flag = helper._points.Count > 0;
            //预览，走加签流程则不送打印机
            var dialog = new PrintPreviewWindow(xps, flag ? PrintDevice.Screen : PrintDevice.Printer, true);
            dialog.Owner = Application.Current.MainWindow;
            dialog.ShowInTaskbar = false;
            dialog.ShowDialog();
            if (!flag)
                return dialog.IsPrinted;
            //获取新的打印文件
            var xps_re = new FileInfo(System.IO.Path.Combine("print", xps.Name));
            //补打文件不存在，客户取消
            if (!xps_re.Exists) return false;
            //加签操作
            helper.Process2Signature(xps_re);
            PrintManager.SendToPrinter(xps_re, size.Width, size.Height, title);
            return false;
        }

        /// <summary>
        /// Xps2Pdf转换，由参数指定流动方向
        /// </summary>
        /// <param name="data"></param>
        /// <param name="isXps"></param>
        /// <returns></returns>
        public static byte[] XpsConvert(byte[] data, bool xps2Pdf = true)
        {
            Spire.Pdf.PdfDocument doc = new PdfDocument();
            try
            {
                using (var ms = new MemoryStream())
                {
                    //xps->pdf
                    if (xps2Pdf)
                    {
                        doc.LoadFromXPS(data);
                        doc.SaveToStream(ms, FileFormat.PDF);
                    }
                    else//pdf->xps
                    {
                        doc.LoadFromBytes(data);
                        doc.SaveToStream(ms, FileFormat.XPS);
                    }
                    return ms.ToArray();
                }
            }
            finally
            {
                doc.Close();
            }
        }

        /// <summary>
        /// pdf合成处理
        /// </summary>
        /// <param name="data"></param>
        /// <param name="pos"></param>
        /// <returns></returns>
        public static byte[] Composite(byte[] data, Dictionary<byte[], List<System.Windows.Point>> pos)
        {
            if (pos == null || pos.Count == 0) return data;
            Spire.Pdf.PdfDocument doc = new PdfDocument();
            try
            {
                doc.LoadFromBytes(data);
                var page = doc.Pages[0];
                foreach (var item in pos)
                {
                    using (var ms = new MemoryStream(item.Key))
                    {
                        //作为附件添加
                        if (item.Value == null)
                        {
                            var attachment = new Spire.Pdf.Attachments.PdfAttachment("attachment.txt", ms);
                            doc.Attachments.Add(attachment);
                            continue;
                        }
                        //加载图片到Image对象
                        System.Drawing.Image image = null;
                        try { image = System.Drawing.Image.FromStream(ms); }
                        catch { }
                        //非图片格式，认定为文本
                        if (image == null)
                        {
                            var s = PlatformSettings.Encoding.GetString(item.Key);
                            var font = new System.Drawing.Font("宋体", 14, System.Drawing.FontStyle.Regular);
                            var trueTypeFont = new PdfTrueTypeFont(font, true);
                            item.Value.ForEach(p =>
                            {
                                page.Canvas.DrawString(s, trueTypeFont, PdfBrushes.Black, new System.Drawing.PointF((float)p.X, page.Size.Height - (float)p.Y - 15));
                            });
                            continue;
                        }
                        //调整图片大小
                        int width = image.Width;
                        int height = image.Height;
                        float scale = 100f / width;// 0.8f;  //缩放比例
                        var size = new System.Drawing.SizeF(width * scale, height * scale);
                        item.Value.ForEach(p =>
                        {
                            //加载缩放后的图片到PdfImage对象
                            PdfImage pdfImage = PdfImage.FromImage(image);
                            //设置图片位置
                            float x = (float)(p.X);
                            float y = page.Size.Height - (float)(p.Y) - size.Height;
                            //在指定位置绘入图片
                            page.Canvas.DrawImage(pdfImage, new System.Drawing.PointF(x, y), size);
                        });
                    }
                }
                using (var ms = new MemoryStream())
                {
                    //保存文档
                    doc.SaveToStream(ms);
                    return ms.ToArray();
                }
            }
            catch (Exception ex)
            {
                return null;
            }
            finally
            {
                doc.Close();
            }
        }

        /// <summary>
        /// 获取机构签署情况
        /// </summary>
        /// <returns></returns>
        public Dictionary<SealKind, string> GenBranchSignature()
        {
            if (Mode != SealMode.Paperless) return new Dictionary<SealKind, string>();
            return _points.GroupBy(x => x.Value.Item2).Select(x => x.Key).ToDictionary(x => x, x => GenKeyCode(x));
        }
    }
}
