﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Forms.Integration;
using Microsoft.Reporting.WinForms;
using Platform.Common.LogSystem;
using TellerSystem.PrintTemplate.Base;
using TellerSystem.PrintTemplate.Extension;
using System.Xml;
using System.Windows.Interop;
using Platform.Common.RunningParameters;
using TellerSystem.ServiceProxy.Ext.ServiceHelper;

namespace TellerSystem.PrintTemplate.Report
{
    public class ReportEntity : DependencyObject
    {
        #region 属性
        public string ReportName { get; set; }
        public List<DataTable> Datas { get; set; }
        public List<ReportParameter> Parameters { get; set; }
        #endregion

        #region 方法
        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="reportName">报表名称</param>
        public ReportEntity(string reportName)
        {
            this.ReportName = reportName;
            Datas = new List<DataTable>();
            Parameters = new List<ReportParameter>();
        }

        /// <summary>
        /// 解析格式化的数据,并转化为相应的ReportParameter和DataTable填充到该对象的属性中
        /// </summary>
        /// <param name="formattedData"></param>
        public void Analyze(string formattedData)
        {
            Datas.Clear();
            using (StringReader sr = new StringReader(formattedData))
            {
                string input = null;
                input = sr.ReadLine();
                while (input != null)
                {
                    string node = input.Trim();
                    switch (node)
                    {
                        case "[parameters]":
                            string para = sr.ReadLine();
                            while (para != null)
                            {
                                string par = para.Trim();
                                if (par != ";")
                                {
                                    int index = par.IndexOf("=");
                                    if (index != -1)
                                    {
                                        string name = par.Substring(0, index);
                                        string value = par.Substring(index + 1);
                                        ReportParameter parameter = new ReportParameter(name, value);
                                        Parameters.Add(parameter);
                                        para = sr.ReadLine();
                                    }
                                    else
                                    {
                                        MessageBox.Show("参数传输格式不正确");
                                        //写入日志文件
                                        return;
                                    }
                                }
                                else
                                {
                                    break;
                                }
                            }
                            break;

                        case "[datatable]":
                            string tableName = sr.ReadLine().Trim();
                            string str = sr.ReadLine().Trim();
                            if (str[str.Length - 1] == '|')
                            {
                                str = str.Remove(str.Length - 1);
                            }
                            string[] columnNames = str.Split('|');
                            //该字典的目的是确保列名唯一
                            Dictionary<string, object> columnDic = new Dictionary<string, object>();
                            for (int i = 0; i < columnNames.Length; i++)
                            {
                                if (columnDic.ContainsKey(columnNames[i]))
                                {
                                    columnNames[i] = columnNames[i] + Guid.NewGuid().ToString();
                                }
                                columnDic.Add(columnNames[i], new object());
                            }
                            int columnNumber = columnNames.Length;
                            DataTable dt = new DataTable();
                            dt.TableName = tableName;
                            foreach (var columnName in columnNames)
                            {
                                DataColumn dc = new DataColumn();
                                dc.ColumnName = columnName.Trim();
                                dt.Columns.Add(dc);
                            }
                            string dataLine = sr.ReadLine();
                            while (dataLine != null)
                            {
                                string data = dataLine.Trim();
                                if (data != ";")
                                {
                                    if (data.Contains('|'))
                                    {
                                        string[] rowData = data.Split('|');
                                        DataRow row = dt.NewRow();
                                        for (int i = 0; i < rowData.Length; i++)
                                        {
                                            if (i < columnNumber)
                                            {
                                                row[i] = rowData[i].Trim();
                                            }
                                        }
                                        dt.Rows.Add(row);
                                        dataLine = sr.ReadLine();
                                    }
                                    else
                                    {
                                        MessageBox.Show("表格数据传输格式不正确");
                                        //写入日志文件
                                        return;
                                    }
                                }
                                else
                                {
                                    break;
                                }

                            }
                            Datas.Add(dt);
                            break;

                        default:
                            break;
                    }
                    input = sr.ReadLine();
                }
            }
        }

        /// <summary>
        /// 生成可浏览的报表打印窗口
        /// </summary>
        public void Print()
        {
            List<ReportDataSource> sources = new List<ReportDataSource>();
            for (int i = 0; i < Datas.Count; i++)
            {
                ReportDataSource source = new ReportDataSource(Datas[i].TableName, Datas[i]);
                sources.Add(source);
            }
            PrintReport(ReportName, sources, Parameters);
        }

        public void Print(IEnumerable<DataTable> tables, Dictionary<string, string> parameterList)
        {
            List<ReportDataSource> sources = new List<ReportDataSource>();
            if (tables != null)
            {
                foreach (var item in tables)
                {
                    sources.Add(new ReportDataSource(item.TableName, item));
                }
            }

            List<ReportParameter> parameters = new List<ReportParameter>();
            if (parameterList != null)
            {
                foreach (var item in parameterList)
                {
                    parameters.Add(new ReportParameter(item.Key, item.Value));
                }
            }
            PrintReport(ReportName, sources, parameters);
        }

        /// <summary>
        /// 生成可浏览的报表打印窗口
        /// </summary>
        /// <param name="rdlcName">报表名称</param>
        /// <param name="tableSources">数据源集合</param>
        /// <param name="parameter">参数集合</param>
        private void PrintReport(string rdlcName, IEnumerable<ReportDataSource> tableSources, IEnumerable<ReportParameter> parameter)
        {
            if (!string.IsNullOrEmpty(rdlcName))
            {
                //弹窗之前杀进程
                Process[] processes = Process.GetProcessesByName("Platform.PrintService");
                if (processes.Length > 0)
                {
                    foreach (Process process in processes)
                    {
                        if (!process.HasExited)
                            process.Kill();
                    }
                }
                //System.Windows.Forms.Form form = new System.Windows.Forms.Form();
                Microsoft.Reporting.WinForms.ReportViewer rv = new Microsoft.Reporting.WinForms.ReportViewer();
                rv.ShowExportButton = TradeManagerHandle.TT_SYSCONFIG_GetConfigValueByConfigID("Is_ShowExportButton") == "true" ? true : false;
                rv.Dock = System.Windows.Forms.DockStyle.Fill;
                rv.SetDisplayMode(Microsoft.Reporting.WinForms.DisplayMode.Normal);
                //form.Controls.Add(rv);
                //rv.LocalReport.ReportEmbeddedResource = rdlcName;
                var rdlc = GetRdlcPath(rdlcName);
                if (!File.Exists(rdlc))
                    throw new ArgumentException("Report加载失败:请检查资源！" + rdlcName);
                rv.LocalReport.EnableExternalImages = true;//开启外部图片支持
                rv.LocalReport.ReportPath = rdlc;
                //检查rdlc文件，是否添加额外参数;若参数列表已添加该key，代表外部已处理，则无需处理
                if (parameter != null && parameter.Count(x => x.Name == _KEY_Background_Seal) == 0)
                {
                    var param = Rdlc2Background_Seal(rdlc, PrintContainerExt.GetPrintElement(this));
                    if (param != null)
                        rv.LocalReport.SetParameters(param);
                }
                if (tableSources != null)
                {
                    foreach (var tableSource in tableSources)
                    {
                        rv.LocalReport.DataSources.Add(tableSource);
                    }
                }
                try
                {
                    if (parameter != null)
                        rv.LocalReport.SetParameters(parameter);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                    //ToDo 写入日志信息
                    PlatformLogger.TradeErrorInfo(ex.Message, ex);
                }
                finally
                {
                    rv.SetDisplayMode(DisplayMode.PrintLayout);
                    rv.ZoomMode = ZoomMode.PageWidth;
                    rv.RefreshReport();
                    Window w = new Window() { Content = new WindowsFormsHost() { Child = rv }, Width = 800, Height = 600, ShowInTaskbar = false, WindowStyle = WindowStyle.ToolWindow };
                    w.Closing += (sender, e) =>
                    {
                        var ret = MessageBox.Show("是否关闭窗体？请确认打印完毕...", "打印窗体", MessageBoxButton.OKCancel, MessageBoxImage.Question, MessageBoxResult.Cancel);
                        if (ret == MessageBoxResult.Cancel)
                            e.Cancel = true;
                    };
                    w.Owner = Application.Current.MainWindow;
                    w.ShowDialog();
                    processes = Process.GetProcessesByName("Platform.PrintService");
                    if (processes.Length > 0)
                    {
                        foreach (Process process in processes)
                        {
                            if (!process.HasExited)
                                process.Kill();
                        }
                    }
                }
            }
        }

        private const string _KEY_Background_Seal = "Background_Seal";

        public static ReportParameter Rdlc2Background_Seal(string rdlc, PrintAttributes printAttributes = null, Point? location = null)
        {
            bool hasBackground = false;
            try
            {
                if (!File.Exists(rdlc))
                    rdlc = GetRdlcPath(rdlc);
                //读取rdlc配置,是否存在参数
                hasBackground = File.ReadAllText(rdlc).Contains("<ReportParameter Name=\"Background_Seal\">");
                if (hasBackground && E_Seal.HasE_Seal)
                {
                    //印章流水的处理,从printAttributes参数计算
                    var text = TellerSystem.PrintTemplate.ElecSeal.ElecSealHelper.GetCode(printAttributes);
                    //构建电子印章
                    var file = CreateBackground_Seal(location ?? new Point(30, 55), text);
                    //提取参数
                    if (file != null)
                        return new ReportParameter(_KEY_Background_Seal, "file:///" + file.FullName);
                }
            }
            catch (Exception ex)
            {
                PlatformLogger.SystemErrorInfo("检查rdlc文件失败", ex);
            }
            return hasBackground ? new ReportParameter(_KEY_Background_Seal, "file:///") : null;
        }

        private static FileInfo CreateBackground_Seal(Point location, String text = null)
        {
            try
            {
                var uri = RES.GetUri("ESeal/" + LoginUserInfo.KinbrName + ".png");
                using (var ms = new MemoryStream(File.ReadAllBytes(uri.LocalPath)))
                {
                    var seal = new System.Drawing.Bitmap(ms);
                    var dpi = 0.00;
                    using (var source = new HwndSource(new HwndSourceParameters()))
                        dpi = 96.0 * source.CompositionTarget.TransformToDevice.M11;
                    var r = (5.0 / 2.54 * dpi) / seal.Width;//打印换算6cm宽对应的系数
                    var rect = new System.Drawing.Rectangle((int)location.X, (int)location.Y, (int)(seal.Width * r), (int)(seal.Height * r));
                    var background = new System.Drawing.Bitmap(rect.Width + rect.X, rect.Height + rect.Y);
                    using (var g = System.Drawing.Graphics.FromImage(background))
                    {
                        //绘制印章
                        g.DrawImage(seal, rect);// (int)location.X, (int)location.Y, size.Width, size.Height);
                        //加载文本
                        if (!string.IsNullOrEmpty(text))
                        {
                            var format = new System.Drawing.StringFormat();
                            format.Alignment = System.Drawing.StringAlignment.Center;
                            format.LineAlignment = System.Drawing.StringAlignment.Center;
                            rect.Y += 18;//对准位置
                            g.DrawString(text, new System.Drawing.Font("宋体", 11), System.Drawing.Brushes.Black, rect, format);
                        }
                        g.Flush();
                    }
                    //保存
                    var file = new FileInfo(System.IO.Path.Combine("temp", Guid.NewGuid().ToString() + ".png"));
                    if (!file.Directory.Exists) file.Directory.Create();
                    background.Save(file.FullName);
                    return file;
                }
            }
            catch (Exception ex)
            {
                PlatformLogger.SystemErrorInfo("创建背景印章失败", ex);
            }
            return null;
        }

        public static string GetRdlcPath(string rdlcName)
        {
            try
            {
                //TellerSystem.PrintTemplate.Report.RTYZ.JZReport.rdlc
                var res = RES.GetUri(rdlcName.Replace("TellerSystem.PrintTemplate.", "").Replace(".rdlc", "").Replace('.', '/') + ".rdlc");
                return res.LocalPath;
            }
            catch
            {
                //throw new ArgumentException("Report加载失败:检查资源！" + rdlcName);
            }
            return string.Empty;
        }
        #endregion

    }
}
