import { existsSync, readFileSync, writeFileSync } from 'fs';
import { spawn } from 'child_process';
import { join } from 'path';

// 模拟 PlatformLogger，实际使用时需要实现具体逻辑
class PlatformLogger {
    static SystemErrorInfo(message: string, error?: Error) {
        console.error(message, error);
    }
}

/**
 * 进程驱动 exe
 * @param path 执行文件路径
 * @param args 参数列表
 * @param timeout 超时时间，单位秒，默认 120 秒
 * @param action 进程启动后的回调函数
 * @returns 是否执行成功以及输出信息
 */
async function ProcessExe(path: string, args: string[], timeout: number = 120, action?: (process: any) => void): Promise<{ success: boolean; output: string }> {
    try {
        // 检查执行文件是否存在
        let exe = join(process.cwd(), path);
        if (!existsSync(exe)) {
            const output = `ERR:执行文件不存在！${exe}`;
            if (!existsSync(path)) {
                return { success: false, output };
            }
            exe = path;
        }

        // 处理参数列表
        const argumentsStr = args.map(arg => `"${arg}"`).join(' ');

        // 启动进程
        const childProcess = spawn(exe, args, {
            shell: false,
            windowsHide: true,
            stdio: ['pipe', 'pipe', 'pipe']
        });

        // 交互动作
        if (action) {
            action(childProcess);
        }

        let output = '';
        let errorOutput = '';

        childProcess.stdout.on('data', (data) => {
            output += data.toString();
        });

        childProcess.stderr.on('data', (data) => {
            errorOutput += data.toString();
        });

        let isTimeout = false;
        const timeoutPromise = new Promise((_, reject) => {
            if (timeout > 0) {
                setTimeout(() => {
                    isTimeout = true;
                    try {
                        childProcess.kill();
                    } catch (err) {
                        // 忽略错误
                    }
                    reject(new Error('调用超时！'));
                }, timeout * 1000);
            }
        });

        const exitPromise = new Promise((resolve) => {
            childProcess.on('close', () => {
                resolve(true);
            });
        });

        try {
            await Promise.race([exitPromise, timeoutPromise]);
            if (isTimeout) {
                output = `调用超时！Cmd:\n${output}`;
                return { success: false, output };
            }
            if (errorOutput) {
                output = errorOutput;
                return { success: false, output };
            }
            return { success: true, output };
        } catch (ex) {
            const errorMessage = `发生异常！${(ex as Error).message}`;
            PlatformLogger.SystemErrorInfo(`ProcessExe发生异常！${path}`, ex as Error);
            return { success: false, output: errorMessage };
        }
    } catch (ex) {
        const errorMessage = `发生异常！${(ex as Error).message}`;
        PlatformLogger.SystemErrorInfo(`ProcessExe发生异常！${path}`, ex as Error);
        return { success: false, output: errorMessage };
    }
}

/**
 * 日期转换，按格式约定处理
 * @param src 日期字符串
 * @param format 日期格式
 * @param def 默认日期
 * @returns 转换后的日期对象或 null
 */
function ToDateTime(src: string, format: string, def: Date | null = null): Date | null {
    // 简单实现，JavaScript 原生不支持指定格式解析，这里使用第三方库会更准确，如 date-fns
    const date = new Date(src);
    if (!isNaN(date.getTime())) {
        return date;
    }
    PlatformLogger.SystemErrorInfo(`无法将[${src}]转换成有效的[${format}]对象`, null);
    return def;
}

export { ProcessExe, ToDateTime };