// 定义 TransitEntity 类型
interface TransitEntity {
    ServiceName: string;
    FuncName: string;
    Parameters: { [key: string]: string };
}

// 定义 AttachFile 类型
interface AttachFile {
    AttachValue: any;
}

// 定义 ServiceManager 类，这里只是模拟，需要根据实际情况实现
class ServiceManager {
    private static instance: ServiceManager;
    private constructor() {}

    static getInstance(): ServiceManager {
        if (!this.instance) {
            this.instance = new ServiceManager();
        }
        return this.instance;
    }

    Commit(entity: TransitEntity): AttachFile[] | null {
        // 这里需要实现实际的提交逻辑
        return null;
    }
}

/**
 * 客户端加密类
 */
export class EncrypteHandle {
    /**
     * 更新秘钥
     * 此接口用于在某一周期之后调用更新设备的工作秘钥和保护秘钥
     * @param workKeyName 工作秘钥（pinpad.终端号.zpk）
     * @param proKeyName 保护秘钥（pinpad.终端号.zmk）
     * @returns 
     */
    static GetNewKey(workKeyName: string, proKeyName: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "encryptservice",
                FuncName: "GetZPKOrZAK",
                Parameters: { "1": workKeyName, "2": proKeyName }
            };
            const tmpresult: AttachFile[] | null = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 与核心转加密
     * @param account 账户
     * @param pinBlock1 PIN密文
     * @param workKeyName 工作秘钥
     * @returns 
     */
    static TranslateEncrypt(account: string, pinBlock1: string, workKeyName: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "encryptservice",
                FuncName: "TranslateEncrypt",
                Parameters: { "1": account, "2": pinBlock1, "3": workKeyName }
            };
            const tmpresult: AttachFile[] | null = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 发服务端验证MAC
     * @param lenOfMac 
     * @param macData 
     * @param mac 
     * @returns 
     */
    static VerifyMack(lenOfMac: string, macData: string, mac: string): boolean {
        let tradeReturn: boolean = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "encryptservice",
                FuncName: "CheckMack",
                Parameters: { "1": lenOfMac, "2": macData, "3": mac }
            };
            const tmpresult: AttachFile[] | null = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue.toString());
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 服务端算MAC
     * @param macData 
     * @param dataLenth 
     * @returns 
     */
    static GetMac(macData: string, dataLenth: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "encryptservice",
                FuncName: "GetMac",
                Parameters: { "1": macData, "2": dataLenth }
            };
            const tmpresult: AttachFile[] | null = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 调用银联加密 多法人
     * @param account 
     * @param pwd 
     * @param legal_pwd 
     * @returns 
     */
    static EncrypteUnion(account: string, pwd: string, legal_pwd: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "encryptservice",
                FuncName: "EncrypteUnion",
                Parameters: { "1": account, "2": pwd, "3": legal_pwd }
            };
            const tmpresult: AttachFile[] | null = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 调用银联加密
     * @param account 
     * @param pwd 
     * @returns 
     */
    static EncrypteUnionSingle(account: string, pwd: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "encryptservice",
                FuncName: "EncrypteUnion",
                Parameters: { "1": account, "2": pwd }
            };
            const tmpresult: AttachFile[] | null = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 调用银联加密-国密
     * @param account 
     * @param pwd 
     * @param legal_pwd 
     * @param encrypteType 
     * @returns 
     */
    static EncrypteUnion_GM(account: string, pwd: string, legal_pwd: string, encrypteType: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "encryptservice",
                FuncName: "EncrypteUnion",
                Parameters: { "1": account, "2": pwd, "3": legal_pwd, "4": encrypteType }
            };
            const tmpresult: AttachFile[] | null = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 调用银联渠道加密
     * @param account 
     * @param pwd 
     * @returns 
     */
    static EncrypteUnion_Sec(account: string, pwd: string): string | null {
        let tradeReturn: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: "encryptservice",
                FuncName: "EncrypteUnion_Sec",
                Parameters: { "1": account, "2": pwd }
            };
            const tmpresult: AttachFile[] | null = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = tmpresult[0].AttachValue.toString();
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }

    /**
     * 更新密钥
     * @param seqno 设备序号
     * @param date 交易日期
     * @returns 
     */
    static IsUpdateNewKey(seqno: string, date: string): boolean {
        let tradeReturn: boolean = false;
        try {
            const entity: TransitEntity = {
                ServiceName: "UtilService",
                FuncName: "getUpdateDeviceKeyState",
                Parameters: { "1": seqno, "2": date }
            };
            const tmpresult: AttachFile[] | null = ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                tradeReturn = Boolean(tmpresult[0].AttachValue.toString());
            }
        } catch (e) {
            throw e;
        }
        return tradeReturn;
    }
}