import axios from 'axios';

// 模拟 TransitEntity 类型
interface TransitEntity {
    ServiceName: string;
    FuncName: string;
    Parameters?: { [key: string]: string };
    AttachFiles?: AttachFile[];
}

// 模拟 AttachFile 类型
interface AttachFile {
    AttachIndex: string;
    AttachType: string;
    AttachValue: any;
}

// 模拟 ServiceManager 类
class ServiceManager {
    private static instance: ServiceManager;

    static getInstance() {
        if (!this.instance) {
            this.instance = new ServiceManager();
        }
        return this.instance;
    }

    async Commit(entity: TransitEntity): Promise<AttachFile[]> {
        // 这里模拟调用服务，实际需要根据后端接口修改
        const response = await axios.post('/api/PrintService', entity);
        return response.data;
    }
}

// 模拟 Platform.ServiceProxy.CommonFunction 类
class CommonFunction {
    static ChangeTableToEntitys<T>(dataTable: any): T[] {
        // 这里简单模拟转换，实际需要根据数据结构实现
        return [] as T[];
    }

    static ChangeEntityToTable(data: any): any {
        // 这里简单模拟转换，实际需要根据数据结构实现
        return {};
    }

    static ChangArrayToStr(arr: string[]): string {
        return arr.join(',');
    }

    static ChangStrToArray(str: string): string[] {
        return str.split(',');
    }
}

// 模拟各种实体类型
interface TT_FormKind { }
interface TT_Form { }
interface TT_TradeForm { }
interface TT_PrintData {
    PrintData: number[];
}
interface TT_PrintDataDetail { }
interface TT_PrintDataDetail2 { }
interface PrintDataLimit { }
interface PrintExtData {
    Attributes: string;
    TradeBus: string;
    Message: number[];
    MsgFile: number[];
}

// 模拟 Types 常量
const Types = {
    TableType: 'TableType',
    ByteType: 'ByteType'
};

// 模拟 PlatformSettings 类
class PlatformSettings {
    static Encoding = {
        GetBytes: (str: string): number[] => {
            return Array.from(new TextEncoder().encode(str));
        },
        GetString: (bytes: number[]): string => {
            return new TextDecoder().decode(new Uint8Array(bytes));
        }
    };
}

// 模拟 PlatformLogger 类
class PlatformLogger {
    static SystemErrorInfo(message: string, error: Error) {
        console.error(message, error);
    }
}

export class PrintManagerHandle {
    /**
     * 增加一个表单分类
     */
    static async AddFormKind(id: string, name: string, flag: string): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintAddTradeForm',
                Parameters: { "1": id, "2": name, "3": flag }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 修改一个表单分类
     */
    static async UpdateFormKind(id: string, name: string, flag: string): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintModTradeForm',
                Parameters: { "1": id, "2": name, "3": flag }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 删除一个表单分类
     */
    static async DeleteFormKind(id: string): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintDelTradeForm',
                Parameters: { "1": id }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 获得全部分类信息
     */
    static async GetFormKind(): Promise<TT_FormKind[]> {
        let result: TT_FormKind[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintGetTotalFormData'
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_FormKind>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }

        return result || [];
    }

    /**
     * 获得表单信息，参数为空字符串就是查询全部，参数有值就是表示分类编号，查询这个分类下的表单
     */
    static async GetForm(id: string): Promise<TT_Form[]> {
        let result: TT_Form[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintGetPrintForm',
                Parameters: { "1": id }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_Form>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }

        return result || [];
    }

    /**
     * 删除一个表单
     */
    static async DeleteForm(id: string): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintDelPrintForm',
                Parameters: { "1": id }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 修改一个表单
     */
    static async UpdateForm(id: string, formId: string, formName: string, description: string): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintModPrintForm',
                Parameters: { "1": id, "2": formId, "3": formName, "4": description }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 增加一个表单
     */
    static async AddForm(id: string, formId: string, formName: string, description: string): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintAddPrintForm',
                Parameters: { "1": id, "2": formId, "3": formName, "4": description }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 增加一个表单
     */
    static async AddKindForm(id: string, formId: string[]): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintAddTradeFormKind',
                Parameters: { "1": id, "2": CommonFunction.ChangArrayToStr(formId) }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 删除某个分类下的某表单
     */
    static async DeleteKindForm(id: string, formid: string[]): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintDelTradeFormKind',
                Parameters: { "1": id, "2": CommonFunction.ChangArrayToStr(formid) }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 获得表单信息，参数为空字符串就是查询全部，参数有值就是表示分类编号，查询这个分类下的表单
     */
    static async GetTradeForm(tradeNo: string): Promise<TT_TradeForm[]> {
        let result: TT_TradeForm[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintGetTradeFormByTradeId',
                Parameters: { "1": tradeNo }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_TradeForm>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }

        return result || [];
    }

    /**
     * 增加一个交易表单
     */
    static async AddTradeForm(form: TT_TradeForm[]): Promise<boolean> {
        let result: string | null = null;
        try {
            const attach: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(form)
            };

            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintAddTradeFormByTradeId',
                AttachFiles: [attach]
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 修改一个交易下某表单
     */
    static async UpdateTradeForm(tradeNo: string, formId: string, sort: string, authLevel: string, printFlag: string): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintModTradeFormByTradeId',
                Parameters: { "1": tradeNo, "2": formId, "3": sort, "4": authLevel, "5": printFlag }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 删除一个表单分类
     */
    static async DeleteTradeForm(tradeNo: string, id: string): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintDelTradeFormByTradeId',
                Parameters: { "1": tradeNo, "2": id }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 增加打印表单数据，此函数的服务端对应函数会过滤掉没有权限的表单，并返回有权限的表单ID
     */
    static async AddPrintData(tradeNo: string, printData: TT_PrintData[]): Promise<string[]> {
        let result: string[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintAddPrintDataByLimit',
                Parameters: { "1": tradeNo },
                AttachFiles: []
            };

            const attach: AttachFile = {
                AttachIndex: "2",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(printData)
            };
            entity.AttachFiles.push(attach);

            const attcheds2: AttachFile = {
                AttachIndex: "3",
                AttachType: Types.ByteType,
                AttachValue: new Uint8Array()
            };
            let memoryStream = new Uint8Array();
            printData.forEach(tmp => {
                const newStream = new Uint8Array(memoryStream.length + tmp.PrintData.length);
                newStream.set(memoryStream);
                newStream.set(tmp.PrintData, memoryStream.length);
                memoryStream = newStream;
            });
            attcheds2.AttachValue = memoryStream;
            entity.AttachFiles.push(attcheds2);

            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangStrToArray(tmpresult[0].AttachValue as string);
            }
        } catch (e) {
            throw e;
        }

        return result || [];
    }

    /**
     * 获得打印数据
     */
    static async GetPrintData(TellerSerialId: string, ThirdSerialNo: string, TellerNo: string, PrintState: string, businessdate: string): Promise<TT_PrintData[]> {
        let result: TT_PrintData[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintGetPrintDataByLimit',
                Parameters: { "1": TellerSerialId, "2": ThirdSerialNo, "3": TellerNo, "4": PrintState, "5": businessdate }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_PrintData>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }

        return result || [];
    }

    /**
     * 获得打印数据明细
     */
    static async GetPrintDataDetail(id: string, tradeNo: string): Promise<TT_PrintDataDetail2[]> {
        let result: TT_PrintDataDetail2[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintGetPrintRecordByDataId',
                Parameters: { "1": id, "2": tradeNo }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_PrintDataDetail2>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }

        return result || [];
    }

    /**
     * 获得流水号/日期/柜员号
     */
    static async GetRevokePrintData(seriousNum: string, data: string, tradeNo: string): Promise<TT_FormKind[]> {
        let result: TT_FormKind[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'GetFormKind',
                Parameters: { "1": seriousNum.padStart(6, '0'), "2": data, "3": tradeNo }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<TT_FormKind>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }

        return result || [];
    }

    /**
     * 增加一个打印历史记录
     */
    static async AddPrintDataHistory(data: TT_PrintDataDetail): Promise<boolean> {
        let result: string | null = null;
        try {
            const attach: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable([data])
            };

            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintAddPrintRecord',
                AttachFiles: [attach]
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 获得指定的打印数据
     */
    static async GetPrintDataHistory(id: string[]): Promise<number[][]> {
        let result: number[][] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintGetFormDataBlobByPrintIds',
                Parameters: { "1": CommonFunction.ChangArrayToStr(id) }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);

            if (tmpresult) {
                result = tmpresult.map(tmp => tmp.AttachValue as number[]);
            }
        } catch (e) {
            throw e;
        }

        return result || [];
    }

    /**
     * 判断一个表单编号是否存在
     */
    static async FormIsExist(formId: string): Promise<boolean> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintIsFormIdExists',
                Parameters: { "1": formId }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result !== "0";
    }

    /**
     * 获得打印票号
     */
    static async GetPrintNum(tellerNo: string, businessdate: string, serialid: string): Promise<string | null> {
        let result: string | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'PrintModPrintNumBySerId',
                Parameters: { "1": tellerNo, "2": businessdate, "3": serialid }
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result;
    }

    /**
     * 获取表单打印控制信息
     */
    static async GetPrintLimit(): Promise<PrintDataLimit[]> {
        let result: PrintDataLimit[] | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'GetPrintLimit'
            };
            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = CommonFunction.ChangeTableToEntitys<PrintDataLimit>(tmpresult[0].AttachValue);
            }
        } catch (e) {
            throw e;
        }

        return result || [];
    }

    /**
     * 将打印数据插入到 T_PRINTDATA 表中
     */
    static async SavePrintData(printData: TT_PrintData[]): Promise<void> {
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'SavePrintData',
                AttachFiles: []
            };

            const attach: AttachFile = {
                AttachIndex: "1",
                AttachType: Types.TableType,
                AttachValue: CommonFunction.ChangeEntityToTable(printData)
            };
            entity.AttachFiles.push(attach);

            const attcheds2: AttachFile = {
                AttachIndex: "2",
                AttachType: Types.ByteType,
                AttachValue: new Uint8Array()
            };
            let memoryStream = new Uint8Array();
            printData.forEach(tmp => {
                const newStream = new Uint8Array(memoryStream.length + tmp.PrintData.length);
                newStream.set(memoryStream);
                newStream.set(tmp.PrintData, memoryStream.length);
                memoryStream = newStream;
            });
            attcheds2.AttachValue = memoryStream;
            entity.AttachFiles.push(attcheds2);

            await ServiceManager.getInstance().Commit(entity);
        } catch (e) {
            throw e;
        }
    }

    /**
     * 提交前登记打印扩展数据
     */
    static async RecordPrintExt(serialid: string, tradeAction: string, classFullName: string, attributes: string, tradeBus: string): Promise<boolean> {
        let result: string = '';
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'RecordPrintExt',
                Parameters: { "1": serialid, "2": tradeAction, "3": classFullName },
                AttachFiles: []
            };

            const attFile: AttachFile = {
                AttachIndex: "4",
                AttachType: Types.ByteType,
                AttachValue: PlatformSettings.Encoding.GetBytes(attributes)
            };
            entity.AttachFiles.push(attFile);

            const busFile: AttachFile = {
                AttachIndex: "5",
                AttachType: Types.ByteType,
                AttachValue: PlatformSettings.Encoding.GetBytes(tradeBus)
            };
            entity.AttachFiles.push(busFile);

            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue as string;
            }
        } catch (e) {
            throw e;
        }

        return result === "0000";
    }

    /**
     * 获取可补打列表
     */
    static async GetPrintExtListData(businessDate: string, tellerNo: string, isHasNormal: boolean = false): Promise<any | null> {
        let result: any | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'GetPrintExtListData',
                Parameters: { "1": businessDate, "2": tellerNo, "3": isHasNormal ? "1" : "0" }
            };

            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 1) {
                result = tmpresult[0].AttachValue;
            }
        } catch (e) {
            PlatformLogger.SystemErrorInfo('获取可补打列表异常：', e);
        }

        return result;
    }

    /**
     * 获取指定打印数据
     */
    static async GetPrintExtDataById(serialId: string): Promise<PrintExtData | null> {
        let result: PrintExtData | null = null;
        try {
            const entity: TransitEntity = {
                ServiceName: 'PrintService',
                FuncName: 'GetPrintExtDataById',
                Parameters: { "1": serialId }
            };

            const tmpresult = await ServiceManager.getInstance().Commit(entity);
            if (tmpresult && tmpresult.length === 5) {
                const dt = tmpresult[0].AttachValue;
                if (!dt || dt.columns.length !== 4 || !tmpresult[3].AttachValue || (tmpresult[3].AttachValue as number[]).length === 0) {
                    return null;
                }
                result = CommonFunction.ChangeTableToEntitys<PrintExtData>(dt)[0];
                result.Attributes = PlatformSettings.Encoding.GetString(tmpresult[1].AttachValue as number[]);
                result.TradeBus = PlatformSettings.Encoding.GetString(tmpresult[2].AttachValue as number[]);
                result.Message = tmpresult[3].AttachValue as number[];
                result.MsgFile = tmpresult[4].AttachValue as number[];
            }
        } catch (e) {
            throw e;
        }

        return result;
    }
}