import axios, { AxiosRequestConfig, AxiosResponse } from 'axios';

// 假设 Result 类型，需根据实际情况调整
interface Result<T> {
    errcode: number;
    errmsg: string;
    data: T;
}

export class RestClient {
    public BaseUri: string;

    constructor(baseUri: string) {
        this.BaseUri = baseUri;
    }

    // Get 请求，带泛型
    public async Get<T>(uri: string, para?: { [key: string]: string }, token?: string): Promise<T> {
        let url = this.buildUrl(uri, para);
        let json = await this.getRequest(url, token);
        return this.ResultHandle<T>(json);
    }

    // 基础 Get 请求
    private async getRequest(uri: string, token?: string): Promise<string> {
        let serviceUrl = `${this.BaseUri}/${uri}`;
        let config: AxiosRequestConfig = {
            headers: {}
        };

        if (token) {
            config.headers['Authorization'] = token;
        }

        try {
            let response: AxiosResponse = await axios.get(serviceUrl, config);
            return response.data;
        } catch (error) {
            throw error;
        }
    }

    // Post 请求，带泛型
    public async Post<T>(uri: string, obj: any, token?: string): Promise<T> {
        let data = JSON.stringify(obj);
        let json = await this.postRequest(uri, data, token);
        return this.ResultHandle<T>(json);
    }

    // 基础 Post 请求
    private async postRequest(uri: string, data: string, token?: string): Promise<string> {
        let serviceUrl = `${this.BaseUri}/${uri}`;
        let config: AxiosRequestConfig = {
            headers: {
                'Content-Type': 'application/json',
                'Accept': '*/*'
            }
        };

        if (token) {
            config.headers['Authorization'] = token;
        }

        try {
            let response: AxiosResponse = await axios.post(serviceUrl, data, config);
            return response.data;
        } catch (error) {
            throw error;
        }
    }

    // 文件上传，带泛型
    public async UploadFile<T>(uri: string, fileName: string, fileByte: Uint8Array, token?: string): Promise<T> {
        let json = await this.uploadFileRequest(uri, fileName, fileByte, token);
        return this.ResultHandle<T>(json);
    }

    // 基础文件上传请求
    private async uploadFileRequest(uri: string, fileName: string, fileByte: Uint8Array, token?: string): Promise<string> {
        // // 假设 ServiceSettings 存在，这里简单模拟
        // let serviceUrl = 'https://example.com/auth-file'; 
        // let formData = new FormData();
        // let blob = new Blob([fileByte], { type: 'application/octet-stream' });
        // formData.append('file', blob, fileName);

        // let config: AxiosRequestConfig = {
        //     headers: {
        //         'Content-Type': `multipart/form-data; boundary=${formData.getBoundary()}`
        //     }
        // };

        // if (token) {
        //     config.headers['Authorization'] = token;
        // }

        // try {
        //     let response: AxiosResponse = await axios.post(serviceUrl, formData, config);
        //     return response.data;
        // } catch (error) {
        //     if (axios.isAxiosError(error) && error.code === 'ETIMEDOUT') {
        //         throw new Error(error.message);
        //     }
        //     throw error;
        // }
        return "";
        //hulei 与原文件实现逻辑不一样需要重写
    }

    // 结果处理
    public ResultHandle<T>(returnJson: string): T {
        let result: Result<T> = JSON.parse(returnJson);
        if (result && result.errcode !== 0) {
            throw new Error(result.errmsg);
        } else if (!result) {
            throw new Error('网络连接超时，稍后重试');
        }
        return result.data;
    }

    // 构建 URL
    private buildUrl(uri: string, para?: { [key: string]: string }): string {
        let url = uri;
        if (para) {
            let params = Object.entries(para).map(([key, value]) => `${key}=${value}`).join('&');
            url += `?${params}`;
        }
        return url;
    }
}
