type PropertyChangedCallback = (newValue: any, oldValue: any) => void;

class DependencyProperty {
    private static _registeredProperties: Map<Function, Map<string, DependencyProperty>> = new Map();

    constructor(
        public readonly name: string,
        public readonly ownerType: Function,
        public readonly defaultValue?: any,
        public readonly metadata?: {
            validate?: (value: any) => boolean;
            coerce?: (value: any) => any;
        }
    ) { }

    static register(
        ownerType: Function,
        name: string,
        defaultValue?: any,
        metadata?: {
            validate?: (value: any) => boolean;
            coerce?: (value: any) => any;
        }
    ): DependencyProperty {
        if (!this._registeredProperties.has(ownerType)) {
            this._registeredProperties.set(ownerType, new Map());
        }

        const prop = new DependencyProperty(name, ownerType, defaultValue, metadata);
        this._registeredProperties.get(ownerType)!.set(name, prop);
        return prop;
    }

    static getProperty(ownerType: Function, name: string): DependencyProperty | undefined {
        return this._registeredProperties.get(ownerType)?.get(name);
    }
}

class DependencyObject {
    private _localValues: Map<string, any> = new Map();
    private _propertyChangedCallbacks: Map<string, PropertyChangedCallback[]> = new Map();

    getValue(property: DependencyProperty): any {
        // 1. Check for local value
        if (this._localValues.has(property.name)) {
            return this._localValues.get(property.name);
        }

        // 2. Return default value from property
        return property.defaultValue;
    }

    setValue(property: DependencyProperty, value: any): void {
        const oldValue = this.getValue(property);

        // Validate the value
        if (property.metadata?.validate && !property.metadata.validate(value)) {
            throw new Error(`Invalid value for property ${property.name}`);
        }

        // Coerce the value
        let newValue = value;
        if (property.metadata?.coerce) {
            newValue = property.metadata.coerce(newValue);
        }

        // Store the value
        this._localValues.set(property.name, newValue);

        // Notify changes
        this.notifyPropertyChanged(property.name, newValue, oldValue);
    }

    addPropertyChangedCallback(propertyName: string, callback: PropertyChangedCallback): void {
        if (!this._propertyChangedCallbacks.has(propertyName)) {
            this._propertyChangedCallbacks.set(propertyName, []);
        }
        this._propertyChangedCallbacks.get(propertyName)!.push(callback);
    }

    removePropertyChangedCallback(propertyName: string, callback: PropertyChangedCallback): void {
        const callbacks = this._propertyChangedCallbacks.get(propertyName);
        if (callbacks) {
            const index = callbacks.indexOf(callback);
            if (index >= 0) {
                callbacks.splice(index, 1);
            }
        }
    }

    private notifyPropertyChanged(propertyName: string, newValue: any, oldValue: any): void {
        const callbacks = this._propertyChangedCallbacks.get(propertyName);
        if (callbacks) {
            // Clone array to avoid issues if callbacks are removed during iteration
            [...callbacks].forEach(callback => callback(newValue, oldValue));
        }
    }
}
